Package: BBmisc
Title: Miscellaneous Helper Functions for B. Bischl
Version: 1.13
Authors@R: c(
    person("Bernd", "Bischl", , "bernd_bischl@gmx.net", role = c("aut", "cre")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "aut"),
    person("Jakob", "Bossek", , "j.bossek@gmail.com", role = "aut"),
    person("Daniel", "Horn", , "daniel.horn@tu-dortmund.de", role = "aut"),
    person("Jakob", "Richter", , "code@jakob-r.de", role = "aut"),
    person("Dirk", "Surmann", , "surmann@statistik.tu-dortmund.de", role = "aut")
  )
Description: Miscellaneous helper functions for and from B. Bischl and
    some other guys, mainly for package development.
License: BSD_2_clause + file LICENSE
URL: https://github.com/berndbischl/BBmisc
BugReports: https://github.com/berndbischl/BBmisc/issues
Imports: checkmate (>= 1.8.0), data.table, methods, stats, utils
Suggests: codetools, microbenchmark, testthat
ByteCompile: yes
Encoding: UTF-8
RoxygenNote: 7.2.1
NeedsCompilation: yes
Packaged: 2022-09-29 09:08:21 UTC; michel
Author: Bernd Bischl [aut, cre],
  Michel Lang [aut],
  Jakob Bossek [aut],
  Daniel Horn [aut],
  Jakob Richter [aut],
  Dirk Surmann [aut]
Maintainer: Bernd Bischl <bernd_bischl@gmx.net>
Repository: CRAN
Date/Publication: 2022-09-29 10:30:08 UTC
Built: R 4.2.0; aarch64-apple-darwin20; 2023-04-01 17:13:23 UTC; unix
Archs: BBmisc.so.dSYM

Package: BBmisc
Title: Miscellaneous Helper Functions for B. Bischl
Version: 1.13.1
Authors@R: c(
    person("Bernd", "Bischl", , "bernd_bischl@gmx.net", role = c("aut", "cre")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "aut"),
    person("Jakob", "Bossek", , "j.bossek@gmail.com", role = "aut"),
    person("Daniel", "Horn", , "daniel.horn@tu-dortmund.de", role = "aut"),
    person("Jakob", "Richter", , "code@jakob-r.de", role = "aut"),
    person("Dirk", "Surmann", , "surmann@statistik.tu-dortmund.de", role = "aut")
  )
Description: Miscellaneous helper functions for and from B. Bischl and
    some other guys, mainly for package development.
License: BSD_2_clause + file LICENSE
URL: https://github.com/berndbischl/BBmisc
BugReports: https://github.com/berndbischl/BBmisc/issues
Imports: checkmate (>= 1.8.0), data.table, methods, stats, utils
Suggests: codetools, microbenchmark, testthat
ByteCompile: yes
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-01-21 10:42:03 UTC; marc
Author: Bernd Bischl [aut, cre],
  Michel Lang [aut],
  Jakob Bossek [aut],
  Daniel Horn [aut],
  Jakob Richter [aut],
  Dirk Surmann [aut]
Maintainer: Bernd Bischl <bernd_bischl@gmx.net>
Repository: CRAN
Date/Publication: 2026-01-21 11:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:35:39 UTC; unix
Archs: BBmisc.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(normalize,data.frame)
S3method(normalize,matrix)
S3method(normalize,numeric)
export("%btwn%")
export("%nin%")
export(addClasses)
export(argsAsNamedList)
export(asMatrixCols)
export(asMatrixRows)
export(asQuoted)
export(binPack)
export(cFactor)
export(capitalizeStrings)
export(catf)
export(checkArg)
export(checkListElementClass)
export(chunk)
export(clipString)
export(coalesce)
export(collapse)
export(collapsef)
export(computeMode)
export(convertColsToList)
export(convertDataFrameCols)
export(convertDfCols)
export(convertInteger)
export(convertIntegers)
export(convertListOfRowsToDataFrame)
export(convertMatrixType)
export(convertRowsToList)
export(convertToShortString)
export(dapply)
export(do.call2)
export(dropNamed)
export(ensureVector)
export(explode)
export(extractSubList)
export(filterNull)
export(getAttributeNames)
export(getBestIndex)
export(getClass1)
export(getFirst)
export(getLast)
export(getMaxIndex)
export(getMaxIndexOfCols)
export(getMaxIndexOfRows)
export(getMinIndex)
export(getMinIndexOfCols)
export(getMinIndexOfRows)
export(getOperatingSystem)
export(getRelativePath)
export(getUnixTime)
export(getUsedFactorLevels)
export(hasAttributes)
export(insert)
export(is.error)
export(isDarwin)
export(isDirectory)
export(isEmptyDirectory)
export(isExpensiveExampleOk)
export(isFALSE)
export(isLinux)
export(isProperlyNamed)
export(isScalarCharacter)
export(isScalarComplex)
export(isScalarFactor)
export(isScalarInteger)
export(isScalarLogical)
export(isScalarNA)
export(isScalarNumeric)
export(isScalarValue)
export(isSubset)
export(isSuperset)
export(isUnix)
export(isValidName)
export(isWindows)
export(itostr)
export(lib)
export(listToShortString)
export(load2)
export(lsort)
export(makeDataFrame)
export(makeFileCache)
export(makeProgressBar)
export(makeS3Obj)
export(makeSimpleFileLogger)
export(mapValues)
export(messagef)
export(namedList)
export(names2)
export(normalize)
export(optimizeSubInts)
export(pause)
export(printHead)
export(printStrToChar)
export(printToChar)
export(rangeVal)
export(requirePackages)
export(rowLapply)
export(rowSapply)
export(save2)
export(seq_col)
export(seq_row)
export(setAttribute)
export(setClasses)
export(setColNames)
export(setRowNames)
export(setValue)
export(sortByCol)
export(splitPath)
export(splitTime)
export(stopf)
export(strrepeat)
export(suppressAll)
export(symdiff)
export(system3)
export(toRangeStr)
export(vcapply)
export(viapply)
export(vlapply)
export(vnapply)
export(warningf)
export(which.first)
export(which.last)
import(checkmate)
import(data.table)
import(stats)
import(utils)
importFrom(methods,is)
useDynLib(BBmisc,c_getMaxIndex)
useDynLib(BBmisc,c_getMaxIndexOfCols)
useDynLib(BBmisc,c_getMaxIndexOfRows)

#' @title Compute statistical mode of a vector (value that occurs most frequently)
#'
#' @description
#' Works for integer, numeric, factor and character vectors.
#' The implementation is currently not extremely efficient.
#'
#' @param x [\code{vector}]\cr
#'   Factor, character, integer, numeric or logical vector.
#' @param na.rm [\code{logical(1)}]\cr
#'   If \code{TRUE}, missing values in the data removed.
#'   if \code{FALSE}, they are used as a separate level and this level could therefore
#'   be returned as the most frequent one.
#'   Default is \code{TRUE}.
#' @param ties.method [\code{character(1)}]\cr
#'   \dQuote{first}, \dQuote{random}, \dQuote{last}: Decide which value to take in case of ties.
#'   Default is \dQuote{random}.
#' @return Modal value of length 1, data type depends on data type of \code{x}.
#' @export
#' @examples
#' computeMode(c(1,2,3,3))
computeMode = function(x, ties.method = "random", na.rm = TRUE) {
  assertAtomicVector(x)
  assertChoice(ties.method, c("first", "random", "last"))
  assertFlag(na.rm)
  tab = as.data.table(x)[,  .N, by = x]
  if (na.rm) tab = na.omit(tab)
  ind = (tab$N == max(tab$N))
  mod = tab$x[ind]
  if (is.factor(mod))
    mod = as.character(mod)
  if (length(mod) > 1L)
    ind = switch(ties.method, first = mod[1L], random = sample(mod, 1L), last = mod[length(mod)])
  else mod
}

context("computeMode")

test_that("computeMode", {
  # factor
  expect_equal(computeMode(as.factor(c(1:2, 2L, 2L))), "2")
  #character
  expect_equal(computeMode(c("1","2","3"), ties.method="last"), "3")
  # numeric
  expect_equal(computeMode(c(1,1,2,3)), 1)
  # integer
  expect_equal(computeMode(c(1:2, 2L, 2L), ties.method="first"), 2L)
  expect_equal(computeMode(c(1:2, 2L, 2L), ties.method="random"), 2L)
  expect_equal(computeMode(c(1:2, 2L, 2L), ties.method="last"), 2L)
  # logical
  expect_equal(computeMode(c(TRUE, FALSE, FALSE)), FALSE)
  expect_equal(computeMode(c(TRUE, TRUE, FALSE)), TRUE)

  # na.rm
  expect_equal(computeMode(c(1,1,2,3, NA, NA, NA), na.rm=FALSE), as.numeric(NA))
  expect_equal(computeMode(c(1,1,2,3, NA, NA, NA), na.rm=TRUE), 1)
})

#' @title Convert Integers to Strings
#'
#' @description
#' This is the counterpart of \code{\link[base]{strtoi}}.
#' For a base greater than \sQuote{10}, letters \sQuote{a} to \sQuote{z}
#' are used to represent \sQuote{10} to \sQuote{35}.
#'
#' @param x [\code{integer}]\cr
#'  Vector of integers to convert.
#' @param base [\code{integer(1)}]\cr
#'  Base for conversion. Values between 2 and 36 (inclusive) are allowed.
#' @return \code{character(length(x))}.
#' @export
#' @examples
#' # binary representation of the first 10 natural numbers
#' itostr(1:10, 2)
#'
#' # base36 encoding of a large number
#' itostr(1e7, 36)
itostr = function(x, base = 10L) {
  x = asInteger(x, any.missing = FALSE, lower = 0L)
  base = asInt(base, na.ok = FALSE, lower = 2L, upper = 36L)
  .Call("itostr", x, base, PACKAGE = "BBmisc")
}

context("itostr")

test_that("itostr", {
  x = 0:200
  base = c(2, 10, 16, 24, 32, 36)

  for (b in base) {
    res = itostr(x, b)
    expect_true(is.character(res))
    expect_true(length(x) == length(res))
    expect_equal(strtoi(res, b), x)
  }

  expect_error(itostr(1, 0))
  expect_error(itostr(1, 1))
  expect_error(itostr(-1, 2))
})

#' @title Conversion for single integer
#'
#' @description
#' Convert single numeric to integer only if the numeric represents a single integer,
#' e.g. 1 to 1L.
#' Otherwise the argument is returned unchanged.
#'
#' @param x [any]\cr
#'   Argument.
#' @return Either a single integer if conversion was done or \code{x} unchanged.
#' @export
#' @examples
#' str(convertInteger(1.0))
#' str(convertInteger(1.3))
#' str(convertInteger(c(1.0, 2.0)))
#' str(convertInteger("foo"))
convertInteger = function(x) {
  if (is.integer(x) || length(x) != 1L)
    return(x)
  if (is.na(x))
    return(as.integer(x))
  if (is.numeric(x)) {
    xi = as.integer(x)
    if (isTRUE(all.equal(x, xi)))
      return(xi)
  }
  return(x)
}

#' @title Conversion for integer vector
#'
#' @description
#' Convert numeric vector to integer vector if the numeric vector fully represents
#' an integer vector,
#' e.g. \code{c(1, 5)} to \code{c(1L, 5L)}.
#' Otherwise the argument is returned unchanged.
#'
#' @param x [any]\cr
#'   Argument.
#' @return Either an integer vector if conversion was done or \code{x} unchanged.
#' @export
#' @examples
#' str(convertIntegers(1.0))
#' str(convertIntegers(1.3))
#' str(convertIntegers(c(1.0, 2.0)))
#' str(convertIntegers("foo"))
convertIntegers = function(x) {
  if (is.integer(x))
    return(x)
  if (length(x) == 0L || (is.atomic(x) && all(is.na(x))))
    return(as.integer(x))
  if (is.numeric(x)) {
    xi = as.integer(x)
    if (isTRUE(all.equal(x, xi, check.names = FALSE)))
      return(setNames(xi, names(x)))
  }
  return(x)
}

context("convert ints")

test_that("convertInteger", {
  expect_true(identical(convertInteger(1), 1L))
  expect_true(identical(convertInteger(1L), 1L))
  expect_true(identical(convertInteger(c(1,4)), c(1, 4)))
  expect_true(identical(convertInteger("a"), "a"))
  expect_true(identical(convertInteger(NA), as.integer(NA)))
  expect_true(identical(convertInteger(as.integer(NA)), as.integer(NA)))
  expect_true(identical(convertInteger(as.numeric(NA)), as.integer(NA)))
  expect_true(identical(convertInteger(c(1, NA)), c(1, NA)))
})


test_that("convertIntegers", {
  expect_true(identical(convertIntegers(1), 1L))
  expect_true(identical(convertIntegers(1L), 1L))
  expect_true(identical(convertIntegers(c(1,4)), c(1L, 4L)))
  expect_true(identical(convertIntegers("a"), "a"))
  expect_true(identical(convertIntegers(NA), as.integer(NA)))
  expect_true(identical(convertIntegers(c(NA, NA)), as.integer(c(NA, NA))))
  expect_true(identical(convertIntegers(as.integer(c(NA, NA))), as.integer(c(NA, NA))))
  expect_true(identical(convertIntegers(as.numeric(c(NA, NA))), as.integer(c(NA, NA))))
  expect_true(identical(convertIntegers(c(1, NA)), as.integer(c(1, NA))))  
  expect_true(identical(convertIntegers(c()), integer()))
  expect_true(identical(convertIntegers(c(x = 1, y = 4)), c(x = 1L, y = 4L)))
})

#' @title A wrapper for \code{class(x) = classes}
#'
#' @description
#' Sets the class attribute of an object and returns the modified object.
#'
#' @param x [any]\cr
#'   Your object.
#' @param classes [\code{character}]\cr
#'  New classes.
#' @return Changed object \code{x}.
#' @export
#' @examples
#' setClasses(list(), c("foo1", "foo2"))
setClasses = function(x, classes) {
  class(x) = classes
  x
}

context("setClasses")

test_that("setClasses", {
  x = list(a=1)
  expect_equal(setClasses(x, "foo"), structure(list(a=1), class="foo"))
  expect_equal(setClasses(x, c("foo1", "foo2")), structure(list(a=1), class=c("foo1", "foo2")))
})

#' @title A wrapper to add to the class attribute
#'
#' @description
#' Adds classes to the class attribute of an object.
#'
#' @param x [any]\cr
#'   Your object.
#' @param classes [\code{character}]\cr
#'  Classes to add. Will be added in front (specialization).
#' @return Changed object \code{x}.
#' @export
#' @examples
#' x = list()
#' print(class(x))
#' x = addClasses(x, c("foo1", "foo2"))
#' print(class(x))
addClasses = function(x, classes) {
  class(x) = c(classes, class(x))
  x
}

context("addClasses")

test_that("addClasses", {
  x = list(a=1)
  x = addClasses(x, "foo1")
  expect_equal(x, structure(list(a=1), class=c("foo1", "list")))
  x = addClasses(x, c("foo2", "foo3"))
  expect_equal(x, structure(list(a=1), class=c("foo2", "foo3", "foo1", "list")))
})

#' @title Wrapper for \code{class(x)[1]}
#'
#' @description
#' Returns the first class of an object.
#'
#' @param x [any]\cr
#'   Input object.
#' @return [\code{character(1)}].
#' @note \code{getClass} is a function in \code{methods}. Do not confuse.
#' @export
getClass1 = function(x) {
  class(x)[1L]
}

context("getClass1")

test_that("getClass1", {
  expect_equal(getClass1(iris), "data.frame")
  expect_equal(getClass1(1), "numeric")
  expect_equal(getClass1(NULL), "NULL")
  x = makeS3Obj(c("C1", "C2"), foo = 2)
  expect_equal(getClass1(x), "C1")
})

#' @title Are all elements of a list / vector uniquely named?
#'
#' @description
#' \code{NA} or \dQuote{} are not allowed as names.
#'
#' @param x [\code{vector}]\cr
#'   The vector or list.
#' @return [\code{logical(1)}].
#' @export
#' @examples
#' isProperlyNamed(list(1))
#' isProperlyNamed(list(a = 1))
#' isProperlyNamed(list(a = 1, 2))
isProperlyNamed = function(x) {
  ns = names2(x)
  length(x) == 0L || !(any(is.na(ns)) || anyDuplicated(ns))
}

context("isProperlyNamed")

test_that("isProperlyNamed", {
  expect_true(isProperlyNamed(list()))
  expect_true(isProperlyNamed(list(x=1)))
  expect_true(isProperlyNamed(list(x=1, y=2)))
  expect_true(!isProperlyNamed(list(1,2)))
  xs = list(1,2)
  names(xs)[1] = "a"
  expect_true(!isProperlyNamed(xs))
  names(xs)[2] = "b"
  expect_true(isProperlyNamed(xs))
})

#' @title Convert a numerical vector into a range string
#'
#' @description
#' Converts a vector of integers into a compact range string representation.
#'
#' @param x [\code{integer}]\cr
#'   Vector to convert into a range string.
#' @param range.sep [\code{character(1)}]\cr
#'   Separator between the first and last element of a range of consecutive
#'   elements in \code{x}.
#'   Default is \dQuote{ - }.
#' @param block.sep [\code{character(1)}]\cr
#'   Separator between non consecutive elements of \code{x} or ranges.
#'   Default is \dQuote{, }.
#' @return [\code{character(1)}]
#' @examples
#' x = sample(1:10, 7)
#' toRangeStr(x)
#' @export
toRangeStr = function(x, range.sep = " - ", block.sep = ", ") {
  if (testIntegerish(x))
    x = as.integer(x)
  else
    assertNumeric(x, any.missing = FALSE)
  assertString(range.sep)
  assertString(block.sep)

  findRange = function(x) seq_len(max(which(x == x[1L] + 0:(length(x)-1L))))
  x = sort(unique(x))
  x = unname(split(x, c(0L, cumsum(diff(x) > 1L))))
  combine = function(x)
    if (length(x) == 1L)
      as.character(x)
    else
      sprintf("%i%s%i", x[1L], range.sep, x[length(x)])
  collapse(vapply(x, combine, character(1L), USE.NAMES = FALSE), block.sep)
}

context("toRangeStr")

expect_range = function(x, str, ...) {
  expect_equal(toRangeStr(x, ...), str)
  expect_equal(toRangeStr(sample(x), ...), str)
  expect_equal(toRangeStr(sample(c(x, x)), ...), str)
  expect_equal(toRangeStr(sample(c(x, x)), ...), str)
}

test_that("continuous ranges", {
  x = c(1, 2, 3, 4, 5, 6)
  expect_range(x, "1 - 6")
  expect_range(x, "1_6", range.sep="_")
})

test_that("single number", {
  x = 1
  expect_range(x, "1")
})

test_that("negative numbers", {
  x = -2:4
  expect_range(x, "-2 - 4")
})

test_that("noncontinuous ranges", {
  x = c(-5, -4, -2, 0, 2, 3, 4, 7)
  expect_range(x, "-5 - -4, -2, 0, 2 - 4, 7")
})

#' @title Insert elements from one list/vector into another list/vector
#'
#' @description
#' Inserts elements from \code{xs2} into \code{xs1} by name,
#' overwriting elements of equal names.
#'
#' @param xs1 [\code{list}]\cr
#'   First list/vector.
#' @param xs2 [\code{list}]\cr
#'   Second vector/list. Must be fully and uniquely named.
#' @param elements [\code{character}]\cr
#'   Elements from \code{xs2} to insert into \code{xs1}.
#'   Default is all.
#' @return \code{x1} with replaced elements from \code{x2}.
#' @export
#' @examples
#' xs1 = list(a = 1, b = 2)
#' xs2 = list(b = 1, c = 4)
#' insert(xs1, xs2)
#' insert(xs1, xs2, elements = "c")
insert = function(xs1, xs2, elements) {
  if (length(xs2) > 0L) {
    if (missing(elements)) {
      xs1[names(xs2)] = xs2
    } else {
      elements = intersect(elements, names(xs2))
      xs1[elements] = xs2[elements]
    }
  }
  return(xs1)
}

context("insert")

test_that("insert", {
  # list        
  xs1 = list(a=1, b=2)
  expect_equal(insert(xs1, list(a=99, c=5)), list(a=99, b=2, c=5))
  expect_equal(insert(xs1, list(a=list(99), c=5)), list(a=list(99), b=2, c=5))
  # vector 
  xs1 = c(a=1, b=2) 
  expect_equal(insert(xs1, c(a=99, c=5)), c(a=99, b=2, c=5))
  expect_equal(insert(xs1, c()), xs1)
})

#' @title Check if some values are covered by the range of the values in a second vector
#'
#' @description
#' Tests whether values in x are between the minimum and maximum of y.
#'
#' @param x [\code{numeric(n)}]\cr
#'   Value(s) that should be within the range of \code{y}.
#' @param y [\code{numeric}]\cr
#'   Numeric vector which defines the range.
#' @return [\code{logical(n)}]. For each value in \code{x}: Is it in the range of \code{y}?
#' @usage x \%btwn\% y
#' @rdname btwn
#' @examples
#' x = 3
#' y = c(-1,2,5)
#' x %btwn% y
#' @export
`%btwn%` = function(x, y) {
  r = range(y)
  x <= r[2] & x >= r[1]
}

#' @title Check subset relation on two vectors
#'
#' @description
#' Checks if all elements of x are contained in y.
#'
#' @param x [\code{vector}]\cr
#'   Source vector.
#' @param y [\code{vector}]\cr
#'   Vector of the same mode as \code{x}.
#' @param strict [\code{logical(1)}]\cr
#'   Checks for strict/proper subset relation.
#' @return [\code{logical(1)}]
#'   \code{TRUE} if each element of \code{x} is also contained in \code{y}, i. e.,
#'   if \code{x} is a subset of \code{y} and \code{FALSE} otherwise.
#' @export
isSubset = function(x, y, strict = FALSE) {
  assertFlag(strict)
  if (length(x) == 0L)
    return(TRUE)
  res = all(x %in% y)
  if (strict)
    res = res & !isSubset(y, x)
  return(res)
}

context("isSubset (and isSuperset)")

test_that("isSubset/isSuperset", {
  x = 1:10
  y = 1:11
  expect_true(isSubset(x, y))
  expect_false(isSubset(y, x))
  expect_true(isSubset(x, y, strict = TRUE))

  x = y
  expect_true(isSubset(x, y))
  expect_false(isSubset(x, y, strict = TRUE))
  expect_true(isSubset(y, x))
  expect_false(isSubset(y, x, strict = TRUE))
})

#' @title Apply function to rows of a data frame
#'
#' @description
#' Just like an \code{\link[base]{lapply}} on data frames,
#' but on the rows.
#'
#' @param df [\code{data.frame}]\cr
#'   Data frame.
#' @param fun [\code{function}]\cr
#'   Function to apply. Rows are passed as list or vector,
#'   depending on argument \code{unlist}, as first argument.
#' @param ... [\code{ANY}]\cr
#'   Additional arguments for \code{fun}.
#' @param unlist [\code{logical(1)}]\cr
#'   Unlist the row? Note that automatic conversion may be triggered for
#'   lists of mixed data types
#'   Default is \code{FALSE}.
#' @param simplify [\code{logical(1)} | character(1)]\cr
#'   Should the result be simplified?
#'   See \code{\link{sapply}}.
#'   If \dQuote{cols}, we expect the call results to be vectors of the same length and they are
#'   arranged as the columns of the resulting matrix.
#'   If \dQuote{rows}, likewise, but rows of the resulting matrix.
#'   Default is \code{TRUE}.
#' @param use.names [\code{logical(1)}]\cr
#'   Should result be named by the row names of \code{df}?
#'   Default is \code{TRUE}.
#' @return [\code{list} or simplified object]. Length is \code{nrow(df)}.
#' @export
#' @examples
#'  rowLapply(iris, function(x) x$Sepal.Length + x$Sepal.Width)
rowLapply = function(df, fun, ..., unlist = FALSE) {
  assertDataFrame(df)
  fun = match.fun(fun)
  assertFlag(unlist)
  if (unlist) {
    .wrap = function(.i, .df, .fun, ...)
      .fun(unlist(.df[.i, , drop = FALSE], recursive = FALSE, use.names = TRUE), ...)
  } else {
    .wrap = function(.i, .df, .fun, ...)
      .fun(as.list(.df[.i, , drop = FALSE]), ...)
  }

  lapply(seq_row(df), .wrap, .fun = fun, .df = df, ...)
}

#' @export
#' @rdname rowLapply
rowSapply = function(df, fun, ..., unlist = FALSE, simplify = TRUE, use.names = TRUE) {
  assert(checkFlag(simplify), checkChoice(simplify, c("cols", "rows")))
  assertFlag(use.names)
  ys = rowLapply(df, fun, ..., unlist = unlist)

  # simplify result
  if (length(ys) > 0L) {
    if (isTRUE(simplify)) {
      ys = simplify2array(ys)
    } else if (simplify == "rows") {
      ys = asMatrixRows(ys)
    } else if (simplify == "cols") {
      ys = asMatrixCols(ys)
    }
  }

  # set names
  if (use.names) {
    if (is.matrix(ys)) {
      colnames(ys) = rownames(df)
      rownames(ys) = NULL
    } else {
      names(ys) = rownames(df)
    }
  } else {
    names(ys) = NULL
  }

  return(ys)
}

context("rowLapply / rowSapply")

test_that("rowLapply", {
  df = data.frame(a = 1:10, b = 10:1)
  expect_true(all(rowLapply(df, length, unlist = TRUE) == 2))
  expect_true(all(rowLapply(df, sum, unlist = TRUE) == 11))
  expect_true(all(unlist(rowLapply(df, Negate(is.list), unlist = TRUE))))
  expect_true(all(unlist(rowLapply(df, is.list))))
  fun = function(x, y) sum(c(unlist(x), y))
  expect_equal(rowLapply(df[, 1L, drop = FALSE], fun, y = 1), as.list(2:11))
})


test_that("rowSapply", {
  df = data.frame(a = 1:10, b = 10:1)
  rownames(df) = letters[1:10]
  y1 = rep(2, nrow(df))
  y2 = setNames(y1, rownames(df))
  expect_equal(rowSapply(df, length, simplify = TRUE, use.names = FALSE), y1)
  expect_equal(rowSapply(df, length, simplify = TRUE, use.names = TRUE), y2)
  expect_equal(rowSapply(df, length, simplify = FALSE, use.names = FALSE), as.list(y1))
  expect_equal(rowSapply(df, length, simplify = FALSE, use.names = TRUE), as.list(y2))
  x1 = rowSapply(df, unlist, simplify = TRUE, use.names = TRUE)
  x2 = sapply(1:nrow(df), function(i) unlist(df[i,]), simplify = TRUE, USE.NAMES = FALSE)
  rownames(x2) = NULL; colnames(x2) = rownames(df)
  expect_equal(x1, x2)
  x1 = rowSapply(df, unlist, simplify = "rows", use.names = FALSE)
  x2 = as.matrix(data.frame(a = 1:10, b = 10:1))
  expect_equal(x1, x2)
  x1 = rowSapply(data.frame(a = 1:2), function(r) r$a, simplify = "rows", use.names = FALSE)
  x2 = matrix(1:2, nrow = 2)
  expect_equal(x1, x2)
})

#' @title Check for a function argument
#'
#' @description
#' Throws exception if checks are not passed. Note that argument is evaluated when checked.
#'
#' This function is superseded by the package \pkg{checkmate} and might get deprecated
#' in the future. Please
#'
#' @param x [any]\cr
#'   Argument.
#' @param cl [\code{character}]\cr
#'   Class that argument must \dQuote{inherit} from.
#'   If multiple classes are given, \code{x} must \dQuote{inherit} from at least one of these.
#'   See also argument \code{s4}.
#' @param s4 [\code{logical(1)}]\cr
#'   If \code{TRUE}, use \code{is} for checking class \code{cl}, otherwise use \code{\link{inherits}}, which
#'   implies that only S3 classes are correctly checked. This is done for speed reasons
#'   as calling \code{\link{is}} is pretty slow.
#'   Default is \code{FALSE}.
#' @param len [\code{integer(1)}]\cr
#'   Length that argument must have.
#'   Not checked if not passed, which is the default.
#' @param min.len [\code{integer(1)}]\cr
#'   Minimal length that argument must have.
#'   Not checked if not passed, which is the default.
#' @param max.len [\code{integer(1)}]\cr
#'   Maximal length that argument must have.
#'   Not checked if not passed, which is the default.
#' @param choices [any]\cr
#'   Discrete number of choices, expressed by a vector of R objects.
#'   If passed, argument must be identical to one of these and nothing else is checked.
#' @param subset [any]\cr
#'   Discrete number of choices, expressed by a vector of R objects.
#'   If passed, argument must be identical to a subset of these and nothing else is checked.
#' @param lower [\code{numeric(1)}]\cr
#'   Lower bound for numeric vector arguments.
#'   Default is \code{NA}, which means not required.
#' @param upper [\code{numeric(1)}]\cr
#'   Upper bound for numeric vector arguments.
#'   Default is \code{NA}, which means not required.
#' @param na.ok [\code{logical(1)}]\cr
#'   Is it ok if a vector argument contains NAs?
#'   Default is \code{TRUE}.
#' @param formals [\code{character}]\cr
#'   If this is passed, \code{x} must be a function.
#'   It is then checked that \code{formals} are the names of the
#'   (first) formal arguments in the signature of \code{x}.
#'   Meaning \code{checkArg(function(a, b), formals = "a")} is ok.
#'   Default is missing.
#' @return Nothing.
#' @export
checkArg = function(x, cl, s4 = FALSE, len, min.len, max.len, choices, subset, lower = NA, upper = NA, na.ok = TRUE, formals) {
  s = deparse(substitute(x))
  if (missing(x))
    stop("Argument ", s, " must not be missing!")
  cl2 = class(x)[1]
  len2 = length(x)
  matchEl = function(x, xs) any(sapply(xs, function(y) identical(y, x)))
  # choices must be done first
  if (!missing(choices)) {
    if (!matchEl(x, choices))
      stop("Argument ", s, " must be any of: ", collapse(choices), "!")
  } else if (!missing(subset)) {
    if (!all(sapply(x, matchEl, xs = subset)))
      stop("Argument ", s, " must be subset of: ", collapse(subset), "!")
  } else if (!missing(formals)) {
    if (!is.function(x))
      stop("Argument ", s, " must be of class ", "function", " not: ", cl2, "!")
    fs = names(formals(x))
    if (length(fs) < length(formals) || !all(formals == fs[seq_along(formals)]))
      stop("Argument function must have first formal args: ", paste(formals, collapse = ","), "!")
  } else {
    mycheck = function(x, cc)
      if(identical(cc, "numeric"))
        is.numeric(x)
      else if(identical(cc, "integer"))
        is.integer(x)
      else if(identical(cc, "vector"))
        is.vector(x)
      else if (!s4)
        inherits(x, cc)
      else if (s4)
        is(x, cc)
    if (!any(sapply(cl, mycheck, x = x)))
      stop("Argument ", s, " must be of class ", collapse(cl, " OR "), ", not: ", cl2, "!")
    if (!missing(len) && len2 != len)
      stop("Argument ", s, " must be of length ", len, " not: ", len2, "!")
    if (!missing(min.len) && len2 < min.len)
      stop("Argument ", s, " must be at least of length ", min.len, " not: ", len2, "!")
    if (!missing(max.len) && len2 > max.len)
      stop("Argument ", s, " must be at most of length ", max.len, " not: ", len2, "!")
    if (!na.ok && anyNA(x))
      stop("Argument ", s, " must not contain any NAs!")
    if (is.numeric(x) && !is.na(lower) && ((anyNA(x) && !na.ok) || (!any(is.na(x)) && any(x < lower))))
      stop("Argument ", s, " must be greater than or equal ", lower, "!")
    if (is.numeric(x) && !is.na(upper) && ((anyNA(x) && !na.ok) || (!any(is.na(x)) && any(x > upper))))
      stop("Argument ", s, " must be less than or equal ", upper, "!")
  }
}

context("argument check")

test_that("checkArg", {
  f = function(x) checkArg(x, cl="integer", len=1)
  expect_silent(f(1L))
  expect_error(f(1))
  expect_error(f(1:2))
  f = function(x) checkArg(x, cl="integer", min.len=2)
  expect_silent(f(1:2))
  expect_error(f(1L))
  f = function(x) checkArg(x, cl="integer", max.len=2)
  expect_silent(f(1:2))
  expect_error(f(1:3))
  f = function(x) checkArg(x, cl="integer", len=2, na.ok=FALSE)
  expect_silent(f(1:2))
  expect_error(f(c(3L, NA)))
  f = function(x) checkArg(x, cl="numeric", lower=2)
  expect_silent(f(4:6))
  expect_error(f(1:3))
  f = function(x) checkArg(x, cl="numeric", upper=2)
  expect_silent(f(1))
  expect_error(f(3), "less than or equal 2")
  f = function(x) checkArg(x, cl="numeric", lower=1, upper=2)
  expect_silent({f(1); f(1.5); f(2)})
  expect_error(f(0), "greater than or equal 1")
  expect_error(f(3), "less than or equal 2")

  f = function(x) checkArg(x, cl="numeric", lower=1, upper=5)
  expect_silent(f(1:5))
  expect_error(f(0:5), "greater than or equal 1")
  expect_error(f(1:6), "less than or equal 5")

  f = function(x) checkArg(x, formals=c("foo", "bar"))
  expect_silent(f(function(foo, bar) 1))
  expect_silent(f(function(foo, bar, bla) 1))
  expect_error(f(1), "must be of class function not: numeric")
  expect_error(f(function(blubb) 1), "must have first formal args")
  expect_error(f(function(foo) 1), "must have first formal args")

  expect_silent(checkArg(1, "vector"))
  expect_silent(checkArg(1L, "vector"))
  expect_silent(checkArg(TRUE, "vector"))
  expect_silent(checkArg("a", "vector"))
  expect_silent(checkArg(list(), "vector"))
  expect_silent(checkArg(list(1), "vector"))
})


test_that("checkArg with choices", {
  f = function(x) checkArg(x, choices=c("a", "b"))
  f("a")
  f("b")
  expect_error(f(c("a", "b")), "must be")
  expect_error(f(1), "must be")
  expect_error(f(NULL), "must be")
  expect_error(f(NA))

  f = function(x) checkArg(x, choices=list(NULL, 1L, data.frame()))
  f(1L)
  f(NULL)
  f(data.frame())
  expect_error(f(1), "must be")
  expect_error(f(list(1)), "must be")
})


test_that("checkArg with subset", {
  f = function(x) checkArg(x, subset=c("a", "b"))
  f("a")
  f("b")
  f(c("a", "b"))
  f(character(0))

  expect_error(f(1), "must be")
  expect_error(f(NA), "must be")

  f = function(x) checkArg(x, subset=list(NULL, 1L, data.frame()))
  f(1L)
  f(NULL)
  f(data.frame())
  f(list(NULL, data.frame()))
  expect_error(f(1), "must be")
  expect_error(f(list(1)), "must be")
})

test_that("checkArg with missing arg", {
  f = function(x) checkArg(x, "numeric")
  expect_error(f(), "Argument x must not be missing!")
})

test_that("checkArg with classes / s3 and s4", {
  x = 1
  class(x) = c("foo2", "foo1")
  expect_silent(checkArg(x, "foo1"))
  expect_silent(checkArg(x, "foo1", s4=FALSE))
  expect_silent(checkArg(x, "foo1", s4=TRUE))
  expect_silent(checkArg(x, "foo2"))
  expect_silent(checkArg(x, "foo2", s4=FALSE))
  expect_silent(checkArg(x, "foo2", s4=TRUE))
  mys41 = setClass("mys41", representation(x="numeric"))
  mys42 = setClass("mys42", contains="mys41", representation(y="numeric"))
  obj1 = mys41(x=3)
  obj2 = mys42(x=3, y=4)
  expect_silent(checkArg(obj1, "mys41", s4=TRUE))
  expect_silent(checkArg(obj2, "mys41", s4=TRUE))
  expect_silent(checkArg(obj2, "mys42", s4=TRUE))
})

test_that("checkArg with multiple classes", {
  expect_silent(checkArg(1, c("numeric", "list")))
  expect_silent(checkArg(1, c("numeric", "foo")))
  expect_silent(checkArg(1L, c("integer", "list")))
  expect_silent(checkArg(1L, c("integer", "foo")))
  expect_silent(checkArg(1L, c("numeric", "list")))
  expect_silent(checkArg(1L, c("numeric", "foo")))
})

#' @title Find the index of first/last \code{TRUE} value in a logical vector
#'
#' @description
#' Returns the index of the first or last TRUE value in a logical vector.
#'
#' @param x [\code{logical}]\cr
#'   Logical vector.
#' @param use.names [\code{logical(1)}]\cr
#'   If \code{TRUE} and \code{x} is named, the result is also
#'   named.
#' @return [\code{integer(1)} | \code{integer(0)}].
#'   Returns the index of the first/last \code{TRUE} value in \code{x} or
#'   an empty integer vector if none is found.
#' @export
#' @examples
#'  which.first(c(FALSE, TRUE))
#'  which.last(c(FALSE, FALSE))
which.first = function(x, use.names = TRUE) {
  wf(x, use.names)
}

#' @rdname which.first
#' @export
which.last = function(x, use.names = TRUE) {
  wl(x, use.names)
}

context("which.first / which.last")

test_that("which.first / which.last", {
  x = c(FALSE, TRUE, FALSE, TRUE)
  expect_equal(which.first(x), 2L)
  expect_equal(which.last(x), 4L)

  x = setNames(x, head(letters, length(x)))
  expect_equal(which.first(x), setNames(2L, "b"))
  expect_equal(which.last(x), setNames(4L, "d"))
  expect_equal(which.first(x, use.names=FALSE), 2L)
  expect_equal(which.last(x, use.names=FALSE), 4L)

  x = c(NA, TRUE, NA, TRUE, NA)
  expect_equal(which.first(x), 2L)
  expect_equal(which.last(x), 4L)

  x = logical(0L)
  expect_equal(which.first(x), integer(0L))
  expect_equal(which.last(x), integer(0L))
  expect_equal(which.first(x, use.names=FALSE), integer(0L))
  expect_equal(which.last(x, use.names=FALSE), integer(0L))

  x = c(NA, NA)
  expect_equal(which.first(x), integer(0L))
  expect_equal(which.last(x), integer(0L))
})

#' @import stats
#' @import checkmate
#' @import utils
#' @import data.table
#' @importFrom methods is

.onLoad = function(libname, pkgname) {
  options(BBmisc.ProgressBar.stream = getOption("BBmisc.ProgressBar.stream", "stderr"))
  options(BBmisc.ProgressBar.style = getOption("BBmisc.ProgressBar.style", "text"))
  options(BBmisc.ProgressBar.width = getOption("BBmisc.ProgressBar.width", getOption("width")))
}

#' @title Split a path into components
#'
#' @description
#' The first normalized path is split on forward and backward slashes and its components returned as
#' character vector. The drive or network home are extracted separately on windows systems and
#' empty on all other systems.
#'
#' @param path [\code{character(1)}]\cr
#'  Path to split as string
#' @return \code{named list}: List with components \dQuote{drive} (\code{character(1)}
#'  and \dQuote{path} (\code{character(n)}.
#' @export
splitPath = function(path) {
  assertString(path)
  path = normalizePath(path, mustWork = FALSE)
  if (isWindows()) {
    pattern = "^([[:alpha:]]:)|(\\\\[[:alnum:]]+)"
    m = regexpr(pattern, path)
    if (length(m) == 1L && m == -1L)
      stop("Error extracting the drive letter")
    drive = regmatches(path, m)
    regmatches(path, m) = ""
  } else {
    drive = character(0L)
  }
  list(drive = drive, path = Filter(nzchar, strsplit(path, "[/\\]+")[[1L]]))
}

context("splitPath")

test_that("splitPath", {
  p = tempfile()
  res = splitPath(p)
  expect_true(is.list(res))
  expect_equal(names(res), c("drive", "path"))
  expect_true(length(res$drive) == as.integer(isWindows()))
  expect_true(is.character(res$path))
  expect_true(length(res$path) >= 1L)

  p = c("tmp", "foo", "", "bar")
  res = splitPath(collapse(p, "/"))
  expect_equal(tail(res$path, 3), p[-3])
  res = splitPath(collapse(p, "\\"))
  expect_equal(tail(res$path, 3), p[-3])
})

context("getRelativePath")

test_that("getRelativePath", {
  base = tempfile("")
  a = file.path(base, "foo")
  b = file.path(base, "bar")
  c = file.path(base, "bar", "foobar")
  lapply(c(a, b, c), dir.create, recursive = TRUE)

  expect_equal(getRelativePath(a, from = base), "foo")
  expect_equal(getRelativePath(b, from = base), "bar")
  expect_equal(getRelativePath(base, from = a), "..")
  expect_equal(getRelativePath(base, from = b), "..")
  expect_equal(getRelativePath(a, from = b), file.path("..", "foo"))
  expect_equal(getRelativePath(b, from = a), file.path("..", "bar"))
  expect_equal(getRelativePath(base, from = c), file.path("..", ".."))
  expect_equal(getRelativePath(c, from = base), file.path("bar", "foobar"))
  expect_equal(getRelativePath(c, from = a), file.path("..", "bar", "foobar"))
  if (!isWindows())
    expect_equal(getRelativePath("/", from = a), do.call(file.path, as.list(rep("..", length(splitPath(a)$path)))))
})

#' @title Helper function for determining the vector of attribute names of a given object
#'
#' @description
#' Returns the names of all attributes of an object.
#'
#' @param obj [any]\cr
#'   Source object.
#' @return [\code{character}]
#'   Vector of attribute names for the source object.
#' @export
getAttributeNames = function(obj) {
  names(attributes(obj))
}

context("getAttributeNames")

test_that("getAttributeNames", {
  x = 1:10
  expect_true(is.null(getAttributeNames(x)))

  attr(x, "size") = length(x)
  expect_equal(getAttributeNames(x), c("size"))
})

# FIXME: not used anywhere?
matchDataFrameSubset = function(df, ss, factors.as.chars = TRUE) {
  # checkArg(df, c("list", "data.frame"))
  # checkArg(ss, c("list", "data.frame"))
  if (!isProperlyNamed(df))
    stop("'df' is not proberbly named")
  if (!isProperlyNamed(ss))
    stop("'ss' is not proberbly named")
  if (any(names(ss) %nin% names(df)))
    stop("Names of 'ss' not found in 'df'")
  if (is.list(df))
    df = as.data.frame(df, stringsAsFactors = FALSE)
  if (is.list(ss))
    ss = as.data.frame(ss, stringsAsFactors = FALSE)

  df = subset(df, select = names(ss))

  if (factors.as.chars) {
    df = convertDataFrameCols(df, factors.as.char = TRUE)
    ss = convertDataFrameCols(ss, factors.as.char = TRUE)
  }

  conv = function(x) rawToChar(serialize(x, connection = NULL, ascii = TRUE))
  match(rowSapply(ss, conv, use.names = FALSE), rowSapply(df, conv, use.names = FALSE))
}

#' @title Replacement for names which always returns a vector
#'
#' @description
#' A simple wrapper for \code{\link[base]{names}}.
#' Returns a vector even if no names attribute is set.
#' Values \code{NA} and \code{""} are treated as missing and
#' replaced with the value provided in \code{missing.val}.
#'
#' @param x [\code{ANY}]\cr
#'   Object, probably named.
#' @param missing.val [\code{ANY}]\cr
#'   Value to set for missing names. Default is \code{NA_character_}.
#' @return [\code{character}]: vector of the same length as \code{x}.
#' @export
#' @examples
#' x = 1:3
#' names(x)
#' names2(x)
#' names(x[1:2]) = letters[1:2]
#' names(x)
#' names2(x)
names2 = function(x, missing.val = NA_character_) {
  n = names(x)
  if (is.null(n))
    return(rep.int(missing.val, length(x)))
  replace(n, is.na(n) | n == "", missing.val)
}

#' @title Simple constructor for S3 objects based on lists
#'
#' @description
#' Simple wrapper for \code{as.list} and \code{\link{setClasses}}.
#'
#' @param classes [\code{character}]\cr
#'   Class(es) for constructed object.
#' @param ... [any]\cr
#'   Key-value pairs for class members.
#' @return Object.
#' @export
#' @examples
#' makeS3Obj("car", speed = 100, color = "red")
makeS3Obj = function(classes, ...) {
  assertCharacter(classes, min.len = 1L, any.missing = FALSE)
  setClasses(list(...), classes = classes)
}

#' @title More meaningful \code{head(df)} output
#'
#' @description
#' The behaviour is similar to \code{print(head(x, n))}. The difference is, that if
#' the number of rows in a data.frame/matrix or the number of elements in a list
#' or vector is larger than \code{n}, additional information is printed about
#' the total number of rows or elements respectively.
#'
#' @param x [\code{data.frame} | \code{matrix} | \code{list} | \code{vector}]\cr
#'   Object.
#' @param n [\code{integer(1)}]\cr
#'   Single positive integer: number of rows for a matrix/data.frame or number of
#'   elements for vectors/lists respectively.
#' @return Nothing.
#' @export
printHead = function(x, n = 6L) {
  assertCount(n, positive = TRUE)
  print(head(x, n = n))
  if ((is.data.frame(x) || is.matrix(x)) && nrow(x) > n)
    catf("... (#rows: %i, #cols: %i)", nrow(x), ncol(x))
  else if (length(x) > n)
    catf("... (#elements: %i)", length(x))
  invisible(NULL)
}

context("printHead")

test_that("printHead", {
  expect_output(printHead(1:10), "...")
  expect_output(printHead(as.list(1:10)), "...")
  expect_output(printHead(iris), "...")
  expect_output(printHead(matrix(runif(20), ncol = 2L)), "...")
})

#' @title Combine multiple factors and return a factor
#'
#' @description
#' Note that function does not inherit from \code{\link{c}} to not change R semantics behind your back when this
#' package is loaded.
#'
#' @param ... [\code{factor}]\cr
#'   The factors.
#' @return [\code{factor}].
#' @export
#' @examples
#' f1 = factor(c("a", "b"))
#' f2 = factor(c("b", "c"))
#' print(c(f1, f2))
#' print(cFactor(f1, f2))
cFactor = function(...) {
  args = lapply(list(...), as.factor)
  newlevels = sort(unique(unlist(lapply(args, levels))))
  ans = unlist(lapply(args, function(x) {
        m = match(levels(x), newlevels)
        m[as.integer(x)]
      }))
  levels(ans) = newlevels
  setClasses(ans, "factor")
}

#' @title Construct a path relative to another
#'
#' @description
#' Constructs a relative path from path \code{from} to path \code{to}.
#' If this is not possible (i.e. different drive letters on windows systems),
#' \code{NA} is returned.
#'
#' @param to [\code{character(1)}]\cr
#'  Where the relative path should point to.
#' @param from [\code{character(1)}]\cr
#'  From which part to start.
#'  Default is \code{\link[base]{getwd}}.
#' @param ignore.case [\code{logical(1)}]\cr
#'  Should path comparisons be made case insensitve?
#'  Default is \code{TRUE} on Windows systems and \code{FALSE} on other systems.
#' @return [character(1)]: A relative path.
#' @export
getRelativePath = function(to, from = getwd(), ignore.case = isWindows()) {
  numberCommonParts = function(p1, p2) {
    for (i in seq_len(min(length(p1), length(p2)))) {
      if (p1[i] != p2[i])
        return(i - 1L)
    }
    return(if (is.null(i)) 0L else i)
  }

  from = splitPath(from)
  to = splitPath(to)
  assertFlag(ignore.case)
  if (length(from$drive) != length(to$drive))
    return(NA_character_)
  if (length(from$drive) > 0L && length(to$drive) > 0L && from$drive != to$drive)
    return(NA_character_)

  if (ignore.case)
    i = numberCommonParts(tolower(from$path), tolower(to$path))
  else
    i = numberCommonParts(from$path, to$path)

  res = c(rep.int("..", length(from$path) - i), tail(to$path, ifelse(i == 0L, Inf, -i)))
  if (length(res) == 0L)
    res = "."
  collapse(res, .Platform$file.sep)
}

#' @title Print \code{str(x)} of an object to a string / character vector
#'
#' @description
#' Captures the output of str() as a character string.
#'
#' @param x [any]\cr
#'   Object to print
#' @param collapse [\code{character(1)}]\cr
#'   Used to collapse multiple lines.
#'   \code{NULL} means no collapsing, vector is returned.
#'   Default is \dQuote{\\n}.
#' @return [\code{character}].
#' @export
#' @examples
#' printStrToChar(iris)
printStrToChar = function(x, collapse = "\n") {
  d = printToChar(str(x), collapse = NULL)
  # remove NULL from str
  collapse(d[-length(d)], collapse)
}

context("printStrToChar")

test_that("printStrToChar", {
  x = 1L
  s = printStrToChar(x, collapse=NULL)
  expect_equal(s, " int 1")
  s = printStrToChar(iris, collapse=NULL)
  expect_true(is.character(s) && length(s) == 6)
  s = printStrToChar(iris)
  expect_true(is.character(s) && length(s) == 1)
})

#' @title Collapse vector to string
#'
#' @description
#' A simple wrapper for \code{collapse(sprintf, ...)}.
#'
#' Useful for vectorized call to \code{\link{sprintf}}.
#'
#' @param ... [any]\cr
#'   See \code{\link{sprintf}}.
#' @param sep [\code{character(1)}]\cr
#'   See \code{\link{collapse}}.
#' @return [\code{character(1)}].
#' @export
collapsef = function(..., sep = ",") {
  paste0(sprintf(...), collapse = sep)
}

#' @title Return index of maximal/minimal/best element in numerical vector
#'
#' @description
#' If \code{x} is empty or only contains NAs which are to be removed,
#' -1 is returned.
#'
#' @note
#' Function \code{getBestIndex} is a simple wrapper for \code{getMinIndex} or
#' \code{getMaxIndex} respectively depending on the argument \code{minimize}.
#'
#' @param x [\code{numeric}]\cr
#'   Input vector.
#' @param weights [\code{numeric}]\cr
#'   Weights (same length as \code{x}).
#'   If these are specified, the index is selected from \code{x * w}.
#'   Default is \code{NULL} which means no weights.
#' @param ties.method [\code{character(1)}]\cr
#'   How should ties be handled?
#'   Possible are: \dQuote{random}, \dQuote{first}, \dQuote{last}.
#'   Default is \dQuote{random}.
#' @param na.rm [\code{logical(1)}]\cr
#'   If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
#'   If \code{TRUE}, NAs are disregarded.
#'   Default is \code{FALSE}
#' @param minimize [\code{logical(1)}]\cr
#'   Minimal element is considered best?
#'   Default is \code{TRUE}.
#' @param ... [any]\cr
#'   Further arguments passed down to the delegate.
#' @return [\code{integer(1)}].
#' @export
#' @useDynLib BBmisc c_getMaxIndex
getMaxIndex = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  ties.method = switch(ties.method, random = 1L, first = 2L, last = 3L,
                       stop("Unknown ties method"))
  assertFlag(na.rm)
  assertNumeric(weights, null.ok = TRUE, len = length(x))
  .Call(c_getMaxIndex, as.numeric(x), as.numeric(weights), ties.method, na.rm, PACKAGE = "BBmisc")
}


#' @export
#' @rdname getMaxIndex
getMinIndex = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  getMaxIndex(-as.numeric(x), weights, ties.method, na.rm)
}


#' @export
#' @rdname getMaxIndex
getBestIndex = function(x, weights = NULL, minimize = TRUE, ...) {
  assertFlag(minimize)
  getIndex = if (minimize) getMinIndex else getMaxIndex
  getIndex(x, weights, ...)
}

context("getMaxIndex")

test_that("getMaxIndex", {
  expect_equal(getMaxIndex(c(1, 9)), 2L)
  expect_equal(getMaxIndex(c(9, 1)), 1L)
  expect_equal(getMaxIndex(c(-9, -1)), 2L)
  expect_equal(getMaxIndex(c(-9, 1)), 2L)
  expect_equal(getMaxIndex(c(1, Inf, 9)), 2L)
  expect_equal(getMaxIndex(c(1, NA, 9)), NA_integer_)
  expect_equal(getMaxIndex(c(1, NaN, 9)), NA_integer_)
  expect_equal(getMaxIndex(c(1, NA, 9), na.rm = TRUE), 3L)
  expect_equal(getMaxIndex(c(1, NaN, 9), na.rm = TRUE), 3L)
  expect_equal(getMaxIndex(numeric(0)), integer(0))
  expect_equal(getMaxIndex(c()), integer(0))
  expect_equal(getMaxIndex(c(NA, NaN), na.rm = TRUE), integer(0))
})

test_that("getMinIndex", {
  expect_equal(getMinIndex(c(1, 9)), 1L)
  expect_equal(getMinIndex(c(9, 1)), 2L)
  expect_equal(getMinIndex(c(-9, -1)), 1L)
  expect_equal(getMinIndex(c(-9, 1)), 1L)
  expect_equal(getMinIndex(c(1, Inf, 9)), 1L)
  expect_equal(getMinIndex(c(1, NA, 9)), NA_integer_)
  expect_equal(getMinIndex(c(1, NaN, 9)), NA_integer_)
  expect_equal(getMinIndex(c(1, NA, 9), na.rm = TRUE), 1L)
  expect_equal(getMinIndex(c(1, NaN, 9), na.rm = TRUE), 1L)
  expect_equal(getMinIndex(numeric(0)), integer(0))
  expect_equal(getMinIndex(c()), integer(0))
  expect_equal(getMinIndex(c(NA, NaN), na.rm = TRUE), integer(0))
})

test_that("ties", {
  expect_equal(getMaxIndex(c(1, 9, 9), ties.method = "first"), 2L)
  expect_equal(getMaxIndex(c(1, 9, 9), ties.method = "last"), 3L)
  expect_equal(getMaxIndex(3, ties.method = "first"), 1L)
  expect_equal(getMaxIndex(3, ties.method = "last"), 1L)
  expect_equal(getMaxIndex(c(9, 1, 9, 9), ties.method = "first"), 1L)
  expect_equal(getMaxIndex(c(9, 1, 9, 9), ties.method = "last"), 4L)
})

test_that("getBestIndex", {
  # here we simply compare with the output of the already tested delegates
  expect_equal(getBestIndex(c(9, 1, 3)), getMinIndex(c(9, 1, 3)))
  expect_equal(getBestIndex(c(9, 1, 3), minimize = FALSE), getMaxIndex(c(9, 1, 3)))
  expect_equal(getBestIndex(c(NA), na.rm = TRUE), getMinIndex(c(NA), na.rm = TRUE))
  expect_equal(getBestIndex(c(1, 1, 9, 5), ties.method = "last"), getMinIndex(c(1, 1, 9, 5), ties.method = "last"))
})

test_that("getMaxIndex with weights", {
  expect_equal(getMaxIndex(c(1, 9), c(1, 1)), 2L)
  expect_equal(getMaxIndex(c(1, 9), c(1, -1)), 1L)
  expect_equal(getMaxIndex(c(1, 9), c(100, 1)), 1L)

  expect_equal(getMinIndex(c(1, 9), c(1, 1)), 1L)
  expect_equal(getMinIndex(c(1, 9), c(1, -1)), 2L)
  expect_equal(getMinIndex(c(1, 9), c(100, 1)), 2L)

  is.na(getMaxIndex(c(1, NA, 2), c(1, 1, 1), na.rm = FALSE))
  expect_equal(getMaxIndex(c(1, NA, 2), c(1, 1, 1), na.rm = TRUE), 3L)
  expect_equal(getMaxIndex(c(1, NA, 2), c(5, 1, 1), na.rm = TRUE), 1L)
  expect_equal(getMaxIndex(c(1, NA, 2), c(NA, 1, 1), na.rm = TRUE), 3L)
})

#' @title Apply a function with a predefined return value
#'
#' @description
#' These are just wrappers around \code{\link[base]{vapply}} with
#' argument \code{FUN.VALUE} set.
#' The function is expected to return a single \code{logical}, \code{integer},
#' \code{numeric} or \code{character} value, depending on the second letter
#' of the function name.
#'
#' @param x [\code{vector} or \code{list}]\cr
#'   Object to apply function on.
#' @param fun [\code{function}]\cr
#'   Function to apply on each element of \code{x}.
#' @param ... [\code{ANY}]\cr
#'   Additional arguments for \code{fun}.
#' @param use.names [\code{logical(1)}]\cr
#'   Should result be named?
#'   Default is \code{TRUE}.
#' @export
vlapply = function(x, fun, ..., use.names = TRUE) {
  vapply(X = x, FUN = fun, ..., FUN.VALUE = NA, USE.NAMES = use.names)
}

#' @rdname vlapply
#' @export
viapply = function(x, fun, ..., use.names = TRUE) {
  vapply(X = x, FUN = fun, ..., FUN.VALUE = NA_integer_, USE.NAMES = use.names)
}

#' @rdname vlapply
#' @export
vnapply = function(x, fun, ..., use.names = TRUE) {
  vapply(X = x, FUN = fun, ..., FUN.VALUE = NA_real_, USE.NAMES = use.names)
}

#' @rdname vlapply
#' @export
vcapply = function(x, fun, ..., use.names = TRUE) {
  vapply(X = x, FUN = fun, ..., FUN.VALUE = NA_character_, USE.NAMES = use.names)
}

#' @title Generate sequences along rows or cols
#'
#' @description
#' A simple convenience wrapper around \code{\link[base]{seq_len}}.
#'
#' @param x [\code{data.frame} | \code{matrix}]\cr
#'   Data frame, matrix or any object which supports \code{\link[base]{nrow}}
#'   or \code{\link[base]{ncol}}, respectively.
#' @return Vector of type [\code{integer}].
#' @export
#' @examples
#' data(iris)
#' seq_row(iris)
#' seq_col(iris)
seq_row = function(x) {
  seq_len(nrow(x))
}

#' @export seq_col
#' @rdname seq_row
seq_col = function(x) {
  seq_len(ncol(x))
}

#' @title Determines used factor levels
#'
#' @description
#' Determines the factor levels of a factor type vector
#' that are actually occuring in it.
#'
#' @param x [\code{factor}]\cr
#'   The factor.
#' @return [\code{character}]
#' @export
getUsedFactorLevels = function(x) {
  intersect(levels(x), unique(x))
}

context("getUsedFactorLevels")

test_that("getUsedFactorLevels", {
  used.levels = letters[1:3]
  all.levels = letters
  x = factor(x = used.levels, levels = all.levels)
  expect_equal(sort(getUsedFactorLevels(x)), used.levels)

  x = factor(x = used.levels)
  expect_equal(used.levels, levels(x))
})

#' @title Shortens strings to a given length
#'
#' @description
#' Clips strings to a maximum length, appending a tail if shortened.
#'
#' @param x [\code{character}]\cr
#'   Vector of strings.
#' @param len [\code{integer(1)}]\cr
#'   Absolute length the string should be clipped to, including \code{tail}.
#'   Note that you cannot clip to a shorter length than \code{tail}.
#' @param tail [\code{character(1)}]\cr
#'   If the string has to be shortened at least 1 character, the final characters will be \code{tail}.
#'   Default is \dQuote{...}.
#' @return [\code{character(1)}].
#' @export
#' @examples
#' print(clipString("abcdef", 10))
#' print(clipString("abcdef", 5))
clipString = function(x, len, tail = "...") {
  assertCharacter(x, any.missing = TRUE)
  len = asInteger(len, len = 1L, lower = nchar(tail))
  assertString(tail)
  ind = (!is.na(x) & nchar(x) > len)
  replace(x, ind, paste(substr(x[ind], 1L, len - nchar(tail)), tail, sep = ""))
}

context("clipString")

test_that("clipString", {
  expect_equal(clipString("abcdef", 7), "abcdef")
  expect_equal(clipString("abcdef", 6), "abcdef")
  expect_equal(clipString("abcdef", 5), "ab...")
  expect_error(clipString("abcdef", 2))

  expect_equal(clipString(NA_character_, 5), NA_character_)
  expect_equal(clipString(c("aaaaa", NA, "aa"), 4), c("a...", NA, "aa"))
})

#' @title Get the first/last element of a list/vector
#'
#' @description
#' Returns the first or last element of a list or vector.
#'
#' @param x [\code{list} | \code{vector}]\cr
#'   The list or vector.
#' @return Selected element. The element name is dropped.
#' @export
getFirst = function(x) {
  assertVector(x, min.len = 1L)
  x[[1L]]
}

#' @rdname getFirst
#' @export
getLast = function(x) {
  assertVector(x, min.len = 1L)
  x[[length(x)]]
}

context("getFirstLast")

test_that("getFirstLast", {
  expect_equal(getFirst(1:3), 1L)
  expect_equal(getLast(1:3), 3L)
  expect_equal(getFirst(list(iris, 1)), iris)
  expect_equal(getLast(list(iris, 1)), 1)

  expect_equal(getFirst(c(a=1, 2)), 1)
  expect_equal(names(getFirst(c(a=1, 2))), NULL)
  expect_equal(getLast(c(a=1, 2)), 2)
  expect_equal(names(getLast(c(a=1, 2))), NULL)
})

#' @title Wrapper for warning and sprintf
#'
#' @description
#' A wrapper for \code{\link{warning}} with \code{\link{sprintf}} applied to the arguments.
#'
#' @param ... [any]\cr
#'   See \code{\link{sprintf}}.
#' @param immediate [\code{logical(1)}]\cr
#'   See \code{\link{warning}}.
#'   Default is \code{TRUE}.
#' @param warning.length [\code{integer(1)}]\cr
#'   Number of chars after which the warning message
#'   gets truncated, see ?options.
#'   Default is 8170.
#' @return Nothing.
#' @export
#' @examples
#' msg = "a warning"
#' warningf("this is %s", msg)
warningf = function(..., immediate = TRUE, warning.length = 8170L) {
  msg = sprintf(...)
  if (immediate) {
    old = getOption("warn")
    # dont change warn setting if it is 2 (= error)
    if (old <= 0L) {
      on.exit(options(warn = old))
      options(warn = 1L)
    }
  }
  obj = simpleWarning(msg, call = sys.call(sys.parent()))
  warning(obj)
}

context("print*f variants")

test_that("messagef", {
  expect_message(messagef("xxx%ixxx", 123), "xxx123xxx")
})

test_that("catf", {
  expect_output(catf("xxx%ixxx", 123), "xxx123xxx")
})

test_that("catf into file", {
  fn = tempfile()
  catf("xxx%ixxx", 123, file=fn)
  s = readLines(fn)
  expect_equal(s, "xxx123xxx")
  unlink(fn)
})



test_that("warningf", {
  expect_warning(warningf("xxx%ixxx", 123), "xxx123xxx")
  f = function() warningf("123")
  # "Warning: " not caught by gives_warning
  expect_warning(f(), "123")
})

test_that("stopf", {
  expect_error(stopf("xxx%ixxx", 123), "xxx123xxx")
  f = function() stopf("123")
  # because try is called in throws_error
  # (and prints a bit differently of course!!!!)
  # we get an extra space before the :
  expect_error(f(), "123")
})

#' @title Create named list, possibly initialized with a certain element
#'
#' @description
#' Even an empty list will always be named.
#'
#' @param names [\code{character}]\cr
#'   Names of elements.
#' @param init [valid R expression]\cr
#'   If given all list elements are initialized to this, otherwise
#'   \code{NULL} is used.
#' @return [\code{list}].
#' @export
#' @examples
#' namedList(c("a", "b"))
#' namedList(c("a", "b"), init = 1)
namedList = function(names, init) {
  if (missing(names))
    return(setNames(list(), character(0L)))
  n = length(names)
  if (missing(init))
    xs = vector("list", n)
  else
    xs = replicate(n, init, simplify = FALSE)
  setNames(xs, names)
}

context("namedList")

test_that("namedList", {
  expect_equal(namedList(), setNames(vector("list", 0), nm = character()))
  expect_equal(namedList("a"), list(a=NULL))
  expect_equal(namedList(c("a", "b")), list(a=NULL, b=NULL))
  expect_equal(namedList(c("a", "b"), 1), list(a=1, b=1))
  f = function(x) x^2
  expect_equal(namedList(c("a", "b"), f(2)), list(a=4, b=4))
})

#' @title Find row- or columnwise the index of the maximal / minimal element in a matrix
#'
#' @description
#' \code{getMaxIndexOfRows} returns the index of the maximal element of each row.
#' \code{getMinIndexOfRows} returns the index of the minimal element of each row.
#' \code{getMaxIndexOfCols} returns the index of the maximal element of each col.
#' \code{getMinIndexOfCols} returns the index of the minimal element of each col.
#' If a corresponding vector (row or col) is empty, possibly after NA removal, -1 is returned
#' as index.
#'
#' @param x [\code{matrix(n,m)}] \cr
#'   Numerical input matrix.
#' @param weights [\code{numeric}]\cr
#'   Weights (same length as number of rows/cols).
#'   If these are specified, the index is selected from the weighted elements
#'   (see \code{\link{getMaxIndex}}).
#'   Default is \code{NULL} which means no weights.
#' @param ties.method [\code{character(1)}]\cr
#'   How should ties be handled?
#'   Possible are: \dQuote{random}, \dQuote{first}, \dQuote{last}.
#'   Default is \dQuote{random}.
#' @param na.rm [\code{logical(1)}]\cr
#'   If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
#'   If \code{TRUE}, NAs are disregarded.
#'   Default is \code{FALSE}
#' @return [\code{integer(n)}].
#' @export
#' @useDynLib BBmisc c_getMaxIndexOfRows c_getMaxIndexOfCols
#' @examples
#' x = matrix(runif(5 * 3), ncol = 3)
#' print(x)
#' print(getMaxIndexOfRows(x))
#' print(getMinIndexOfRows(x))
getMaxIndexOfRows = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  mode(x) = "numeric"
  ties.method = switch(ties.method, random = 1L, first = 2L, last = 3L,
                       stop("Unknown ties method"))
  assertFlag(na.rm)
  assertNumeric(weights, null.ok = TRUE, len = ncol(x))
  .Call(c_getMaxIndexOfRows, x, as.numeric(weights), ties.method, na.rm, PACKAGE = "BBmisc")
}

#' @export
#' @rdname getMaxIndexOfRows
getMinIndexOfRows = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  getMaxIndexOfRows(-x, weights, ties.method, na.rm)
}

#' @export
#' @rdname getMaxIndexOfRows
getMaxIndexOfCols = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  mode(x) = "numeric"
  ties.method = switch(ties.method, random = 1L, first = 2L, last = 3L,
                       stop("Unknown ties method"))
  assertFlag(na.rm)
  assertNumeric(weights, null.ok = TRUE, len = nrow(x))
  .Call(c_getMaxIndexOfCols, x, as.numeric(weights), ties.method, na.rm, PACKAGE = "BBmisc")
}

#' @export
#' @rdname getMaxIndexOfRows
getMinIndexOfCols = function(x, weights = NULL, ties.method = "random", na.rm = FALSE) {
  getMaxIndexOfCols(-x, weights, ties.method, na.rm)
}

context("getMaxIndexOfRows")

test_that("getMaxIndexOfRows", {
  a = matrix(1:6, nrow=2)
  expect_equal(getMaxIndexOfRows(a), c(3L, 3L))
  a = matrix(6:1, nrow=2)
  expect_equal(getMaxIndexOfRows(a), c(1L, 1L))
  a = rbind(c(1, 999), c(-1, -5))
  expect_equal(getMaxIndexOfRows(a), c(2L, 1L))
  a = matrix(rnorm(50*10), nrow=50)
  expect_equal(getMaxIndexOfRows(a), apply(a, 1, which.max))
})

test_that("getMaxIndexOfCols", {
  a = matrix(1:6, nrow=2)
  expect_equal(getMaxIndexOfCols(a), c(2L, 2L, 2L))
  a = matrix(6:1, nrow=2)
  expect_equal(getMaxIndexOfCols(a), c(1L, 1L, 1L))
  a = rbind(c(1, 999), c(-1, -5))
  expect_equal(getMaxIndexOfCols(a), c(1L, 1L))
  a = matrix(rnorm(50*10), nrow=50)
  expect_equal(getMaxIndexOfCols(a), apply(a, 2, which.max))
})

test_that("normal", {
  expect_equal(getMaxIndexOfRows(diag(10)), 1:10)
  n = 100
  perm = sample(n)
  D = diag(n)
  expect_equal(getMaxIndexOfRows(D[perm, ]), (1:n)[perm])
})

test_that("NA values", {
  n = 300
  m = matrix(runif(n), ncol=3)
  mm = m
  mm[, 2] = NA
  expect_equal(getMaxIndexOfRows(mm), rep(NA_integer_, n/3))

  a = matrix(c(1, NA, 2, 3, NA, NA), nrow=3, byrow=TRUE)
  expect_equal(getMaxIndexOfRows(a, na.rm=FALSE), c(NA, 2L, NA))
  expect_equal(getMaxIndexOfRows(a, na.rm=TRUE), c(1L, 2L, -1))
})

test_that("infinite values", {
  n = 300
  m = matrix(runif(n), ncol=3)
  m[, 2] = Inf
  expect_equal(getMaxIndexOfRows(m), rep(2L, 100L))
})

test_that("max.col oddity", {
  expect_equal(getMaxIndexOfRows(cbind(1:10, 2:11, -Inf)), rep(2, 10))
  expect_equal(getMaxIndexOfRows(cbind(-1e9 * 1:10, 1:10, 2:11)), rep(3, 10))
})

test_that("ties", {
  a = matrix(c(1, 1, 2, 2), nrow=2, byrow=TRUE)
  expect_equal(getMaxIndexOfRows(a, ties.method="first"), c(1L, 1L))
  expect_equal(getMaxIndexOfRows(a, ties.method="last"), c(2L, 2L))
  a = matrix(c(2, 1, 2, 2, 2, 1), nrow=2, byrow=TRUE)
  expect_equal(getMaxIndexOfRows(a, ties.method="first"), c(1L, 1L))
  expect_equal(getMaxIndexOfRows(a, ties.method="last"), c(3L, 2L))
  a = matrix(c(1, 1, 2, 2), nrow=2, byrow=TRUE)
  expect_equal(getMaxIndexOfCols(a, ties.method="first"), c(2L, 2L))
  expect_equal(getMaxIndexOfCols(a, ties.method="last"), c(2L, 2L))
  a = matrix(c(2, 1, 2, 2, 2, 1), nrow=2, byrow=TRUE)
  expect_equal(getMaxIndexOfCols(a, ties.method="first"), c(1L, 2L, 1L))
  expect_equal(getMaxIndexOfCols(a, ties.method="last"), c(2L, 2L, 1L))
})


test_that("getMaxIndexOfRows with weights", {
  a = matrix(1:6, nrow=2)
  expect_equal(getMaxIndexOfRows(a, c(1, 1, 1)), c(3L, 3L))
  expect_equal(getMaxIndexOfRows(a, c(1, 1.51, 1)), c(3L, 2L))
  expect_equal(getMaxIndexOfRows(a, c(-1, -1, -1)), c(1L, 1L))

  expect_equal(getMinIndexOfRows(a, c(1, 1, 1)), c(1L, 1L))
  expect_equal(getMinIndexOfRows(a, c(-1, -1, -1)), c(3L, 3L))

  expect_equal(getMaxIndexOfCols(a, c(1, 1)), c(2L, 2L, 2L))
  expect_equal(getMaxIndexOfCols(a, c(1, -1)), c(1L, 1L, 1L))

  expect_equal(getMinIndexOfCols(a, c(1, 1)), c(1L, 1L, 1L))
  expect_equal(getMinIndexOfCols(a, c(1, -1)), c(2L, 2L, 2L))
})

#' @title Suppresses all output except for errors
#'
#' @description
#' Evaluates an expression and suppresses all output except for errors, 
#' meaning: prints, messages, warnings and package startup messages.
#' 
#' @param expr [valid R expression]\cr
#'   Expression. 
#' @return Return value of expression invisibly. 
#' @export 
#' @examples
#' suppressAll({
#'   print("foo")
#'   message("foo")
#'   warning("foo")
#' })
suppressAll = function(expr) {
  capture.output({
    z = suppressWarnings(
      suppressMessages(
        suppressPackageStartupMessages(force(expr))                  
      )
    )
  })
  invisible(z)
}

context("suppressAll")

test_that("suppressAll", {
  expect_equal(suppressAll(123), 123)
  expect_output({print(123);0}, "123")
  expect_silent(suppressAll({print(123);0}))

  expect_warning(warning(123), "123")
  expect_silent(suppressAll({warning(123);0}))

  expect_message(message(123), "123")
  expect_silent(suppressAll(message(123)))
})

#' @title Returns first non-missing, non-null argument
#'
#' @description
#' Returns first non-missing, non-null argument, otherwise \code{NULL}.
#'
#' We have to perform some pretty weird \code{\link{tryCatch}} stuff internally,
#' so you should better not pass complex function calls into the arguments that can throw exceptions,
#' as these will be completely muffled, and return \code{NULL} in the end.
#'
#' @param ... [any]\cr
#'   Arguments.
#' @return [any].
#' @export
#' @examples
#' f = function(x,y) {
#'   print(coalesce(NULL, x, y))
#' }
#' f(y = 3)
coalesce = function(...) {
  dots = match.call(expand.dots = FALSE)$...
  for (arg in dots) {
    ismissing = if (is.symbol(arg)) {
      eval(substitute(missing(symbol), list(symbol = arg)), envir = parent.frame())
    } else {
      FALSE
    }
    if (!ismissing) {
      value = tryCatch(eval(arg, envir = parent.frame()), error = function(...) NULL)
      if (!is.null(value)) {
        return(value)
      }
    }
  }
  NULL
}

context("coalesce")

test_that("coalesce", {
  expect_identical(coalesce(NULL), NULL)
  expect_identical(coalesce(1, NULL), 1)
  expect_identical(coalesce(NULL, 1), 1)
  f = function(a, b, c) coalesce(a,b,c)
  expect_identical(f(b = NULL, c = 1), 1)
})

test_that("coalesce works with functions", {
  ff = function(x) 123
  expect_identical(coalesce(NULL, ff), ff)
  # FIXME: this is a problem in R / missing. see issue 48
  #expect_identical(coalesce(NULL, min), min)
})

#' @title Repeat and join a string
#'
#' @description
#' Repeats a string n times and joins the results.
#'
#' @param x [character]\cr
#'   Vector of characters.
#' @param n [\code{integer(1)}]\cr
#'   Times the vector \code{x} is repeated.
#' @param sep [\code{character(1)}]\cr
#'   Separator to use to collapse the vector of characters.
#' @return \code{character(1)}.
#' @export
#' @examples
#' strrepeat("x", 3)
strrepeat = function(x, n, sep = "") {
  paste0(rep.int(x, n), collapse = sep)
}

context("strrepeat")

test_that("strrepeat", {
  expect_identical(strrepeat("x", 3), "xxx")
  expect_identical(strrepeat("x", 3, "y"), "xyxyx")
  expect_identical(strrepeat(c("x", "y"), 2), "xyxy")
})

#' @title Initialize data.frame in a convenient way
#'
#' @description
#' Creates a data frame with specified dimensions, column types and optional initial values.
#'
#' @param nrow [\code{integer(1)}]\cr
#'   Nubmer of rows.
#' @param ncol [\code{integer(1)}]\cr
#'   Number of columns.
#' @param col.types [\code{character(ncol)} | \code{character(1)}]\cr
#'   Data types of columns.
#'   If you only pass one type, it will be replicated.
#'   Supported are all atomic modes also supported by
#'   \code{\link[base]{vector}}, i.e. all common data frame types except factors.
#' @param init [any]\cr
#'   Scalar object to initialize all elements of the data.frame.
#'   You do not need to specify \code{col.types} if you pass this.
#' @param row.names [\code{character} | \code{integer} | \code{NULL}]\cr
#'   Row names.
#'   Default is \code{NULL}.
#' @param col.names [\code{character} | \code{integer}]\cr
#'   Column names.
#'   Default is \dQuote{V1}, \dQuote{V2}, and so on.
#' @export
#' @examples
#' print(makeDataFrame(3, 2, init = 7))
#' print(makeDataFrame(3, 2, "logical"))
#' print(makeDataFrame(3, 2, c("logical", "numeric")))
makeDataFrame = function(nrow, ncol, col.types, init, row.names = NULL, col.names = sprintf("V%i", seq_len(ncol))) {
  nrow = asCount(nrow)
  ncol = asCount(ncol)
  if (!missing(col.types))
    assertCharacter(col.types, min.len = 1L, any.missing = FALSE)
  if (!missing(init)) {
    if(!isScalarValue(init))
      stop("'init' must be a scalar value!")
    if (!missing(col.types)) {
      if (length(col.types) > 1L)
        stop("If 'init' is given, length of col.types must be 1!")
      if (identical(class(init)[1L], "col.types"))
        stop("Class of 'init' must match given column type!")
    }
  }
  if (!missing(col.types) && length(col.types) == 1L)
    col.types = rep.int(col.types, ncol)
  if (!is.null(row.names))
    assert(checkIntegerish(row.names, len = nrow), checkCharacter(row.names, len = nrow))

  assert(checkIntegerish(col.names, len = ncol), checkCharacter(col.names, len = ncol))

  if (nrow == 0L && ncol == 0L)
    df = data.frame()
  else if (ncol == 0L)
    df = data.frame(matrix(nrow = nrow, ncol = 0))
  else if (missing(init))
    df = lapply(col.types, vector, length = nrow)
  else
    df = replicate(ncol, rep.int(init, nrow), simplify = FALSE)
  df = as.data.frame(df, stringsAsFactors = FALSE)
  rownames(df) = row.names
  colnames(df) = col.names
  return(df)
}

#' @title Conditional checking for expensive examples
#'
#' @description
#' Queries environment variable \dQuote{R_EXPENSIVE_EXAMPLE_OK}.
#' Returns \code{TRUE} iff set exactly to \dQuote{TRUE}.
#' This allows conditional checking of expensive examples in packages
#' via R CMD CHECK, so they are not run on CRAN, but at least
#' on your local computer.
#' A better option than \dQuote{dont_run} in many cases, where such examples
#' are not checked at all.
#'
#' @return [\code{logical(1)}].
#' @export
#' @examples
#' # extremely costly random number generation, that we dont want checked on CRAN
#' if (isExpensiveExampleOk()) {
#'   runif(1)
#' }
isExpensiveExampleOk = function() {
  Sys.getenv("R_EXPENSIVE_EXAMPLE_OK") == "TRUE"
}

#' @title Wrapper for message and sprintf
#'
#' @description
#' A simple wrapper for \code{message(sprintf(...))}.
#'
#' @param ... [any]\cr
#'   See \code{\link{sprintf}}.
#' @param .newline [logical(1)]\cr
#'   Add a newline to the message. Default is \code{TRUE}.
#' @return Nothing.
#' @export
#' @examples
#' msg = "a message"
#' warningf("this is %s", msg)
messagef = function(..., .newline = TRUE) {
  message(sprintf(...), appendLF = .newline)
}

#' @title Simple bin packing
#'
#' @description
#' Maps numeric items in \code{x} into groups with sum
#' less or equal than \code{capacity}.
#' A very simple greedy algorithm is used, which is not really optimized
#' for speed. This is a convenience function for smaller vectors, not
#' a competetive solver for the real binbacking problem.
#' If an element of \code{x} exceeds \code{capacity}, an error
#' is thrown.
#'
#' @param x [\code{numeric}]\cr
#'   Numeric vector of elements to group.
#' @param capacity [\code{numeric(1)}]\cr
#'   Maximum capacity of each bin, i.e., elements will be grouped
#'   so their sum does not exceed this limit.
#' @return [\code{integer}]. Integer with values \dQuote{1} to \dQuote{n.bins}
#'   indicating bin membership.
#' @export
#' @examples
#' x = 1:10
#' bp = binPack(x, 11)
#' xs = split(x, bp)
#' print(xs)
#' print(sapply(xs, sum))
binPack = function(x, capacity) {
  assertNumeric(x, min.len = 1L, lower = 0, any.missing = FALSE)
  assertNumber(capacity)

  too.big = which.first(x > capacity, use.names = FALSE)
  if (length(too.big))
    stopf("Capacity not sufficient. Item %i (x=%f) does not fit", too.big, x[too.big])
  if (any(is.infinite(x)))
    stop("Infinite elements found in 'x'")

  ord = order(x, decreasing = TRUE)
  grp = integer(length(x))
  sums = vector(typeof(x), 1L)
  bin.count = 1L

  for(j in ord) {
    new.sums = sums + x[j]
    pos = which.first(new.sums <= capacity, use.names = FALSE)
    if (length(pos)) {
      grp[j] = pos
      sums[pos] = new.sums[pos]
    } else {
      bin.count = bin.count + 1L
      grp[j] = bin.count
      sums[bin.count] = x[j]
    }
  }
  grp
}

context("binPack")

test_that("binPack", {
  x = 1:10
  res = binPack(x, 11L)
  sums = sapply(split(x, res), sum)
  expect_true(is.integer(res))
  expect_equal(sums, setNames(rep(11L, 5L), 1:5))
  expect_true(all(sums <= 11L))

  x = sample(seq(from=0, to=1, by=0.01))
  res = binPack(x, 1)
  sums = sapply(split(x, res), sum)
  expect_true(is.integer(res))
  expect_true(all(head(sums, 50) == 1))

  x = runif(20)
  res = binPack(x, 1)
  sums = sapply(split(x, res), sum)
  expect_true(is.integer(res))
  expect_true(all(sums < 1))

  x = runif(5)
  res = binPack(x, Inf)
  expect_true(is.integer(res))
  expect_true(length(res) == 5 && all(as.numeric(res) == 1))

  expect_error(binPack(c(-5, 3)))
  expect_error(binPack(c(1, 100), 10))
  expect_error(binPack(c(1, Inf), 1))
})

#' @title Is return value of try an exception?
#'
#' @description
#' Checks if an object is of class \dQuote{try-error} or
#' \dQuote{error}.
#'
#' @param x [any]\cr
#'   Any object, usually the return value of \code{\link[base]{try}},
#' \code{\link[base]{tryCatch}}, or a function which may return a
#' \code{\link[base]{simpleError}}.
#' @return [\code{logical(1)}].
#' @export
#' @examples
#' x = try(stop("foo"))
#' print(is.error(x))
#' x = 1
#' print(is.error(x))
is.error = function(x) {
  inherits(x, c("try-error", "error"))
}

#' @title Extracts a named element from a list of lists
#'
#' @description
#' Converts a list of vectors into a matrix with vectors as columns or rows.
#'
#' @param xs [\code{list}]\cr
#'   A list of vectors of the same length.
#' @param row.names [\code{character} | \code{integer} | \code{NULL}]\cr
#'   Row names of result.
#'   Default is to take the names of the elements of \code{xs}.
#' @param col.names [\code{character} | \code{integer} | \code{NULL}]\cr
#'   Column names of result.
#'   Default is to take the names of the elements of \code{xs}.
#' @return [\code{matrix}].
#' @export
asMatrixCols = function(xs, row.names, col.names) {
  assertList(xs)
  n = length(xs)
  if (n == 0L)
    return(matrix(0, nrow = 0L, ncol = 0L))
  assertList(xs, types = "vector")

  m = unique(viapply(xs, length))
  if (length(m) != 1L)
    stopf("Vectors must all be of the same length!")

  if (missing(row.names)) {
    row.names = names(xs[[1L]])
  }

  if (missing(col.names)) {
    col.names = names(xs)
  }

  xs = unlist(xs)
  dim(xs) = c(m, n)
  rownames(xs) = row.names
  colnames(xs) = col.names
  return(xs)
}

#' @rdname asMatrixCols
#' @export
asMatrixRows = function(xs, row.names, col.names) {
  t(asMatrixCols(xs, row.names = col.names, col.names = row.names))
}

context("asMatrix")

test_that("asMatrix", {
  # empty
  expect_equal(
    asMatrixCols(list()),
    matrix(0, nrow = 0L, ncol = 0L)
  )
  expect_equal(
    asMatrixRows(list()),
    matrix(0, nrow = 0L, ncol = 0L)
  )

  # normal
  expect_equal(
    asMatrixCols(list(c(1, 2), c(3, 3), c(4, 4))),
    matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE)
  )
  expect_equal(
    asMatrixRows(list(c(1, 2), c(3, 3), c(4, 4))),
    matrix(c(1, 2, 3, 3, 4, 4), nrow = 3, ncol = 2, byrow = TRUE)
  )

  # names
  expect_equal(
    asMatrixCols(list(a = c(1, 2), b = c(3, 3), c = c(4, 4))),
    setColNames(matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE), c("a", "b", "c"))
  )
  expect_equal(
    asMatrixRows(list(a = c(1, 2), b = c(3, 3), c = c(4, 4))),
    setRowNames(matrix(c(1, 2, 3, 3, 4, 4), nrow = 3, ncol = 2, byrow = TRUE), c("a", "b", "c"))
  )
  expect_equal(
    asMatrixRows(list(a = c(x = 1, y = 2), b = c(3, 3), c = c(4, 4))),
    setColNames(setRowNames(matrix(c(1, 2, 3, 3, 4, 4), nrow = 3, ncol = 2, byrow = TRUE),
        c("a", "b", "c")), c("x", "y"))
  )
  # manually define rownames
  expect_equal(
    asMatrixCols(list(a = c(1, 2), b = c(3, 3), c = c(4, 4)), row.names = c("xx", "yy")),
    setColNames(
      setRowNames(matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE), c("xx", "yy")),
    c("a", "b", "c"))
  )
  # manually define rownames, but use ints
  expect_equal(
    asMatrixCols(list(a = c(1, 2), b = c(3, 3), c = c(4, 4)), row.names = 1:2),
    setColNames(
      setRowNames(matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE), 1:2),
    c("a", "b", "c"))
  )
  # manually define colnames
  expect_equal(
    asMatrixCols(list(a = c(1, 2), b = c(3, 3), c = c(4, 4)), col.names = c("xx", "yy", "zz")),
    setColNames(
      matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE),
    c("xx", "yy", "zz"))
  )
  expect_equal(
    asMatrixRows(list(a = c(1, 2), b = c(3, 3), c = c(4, 4)), col.names = c("xx", "yy")),
    setRowNames(
      setColNames(
        matrix(c(1, 2, 3, 3, 4, 4), nrow = 3, ncol = 2, byrow = TRUE),
        c("xx", "yy")
      ), 
      c("a", "b", "c")
    )
  )
  # manually define colnames, but use ints
  expect_equal(
    asMatrixCols(list(a = c(1, 2), b = c(3, 3), c = c(4, 4)), col.names = 1:3),
    setColNames(
      matrix(c(1, 2, 3, 3, 4, 4), nrow = 2, ncol = 3, byrow = FALSE),
    1:3)
  )
})

#' @title Convert rows (columns) of data.frame or matrix to lists
#'
#' @description
#' For each row, one list/vector is constructed, each entry of
#' the row becomes a list/vector element.
#'
#' @param x [\code{matrix} | \code{data.frame}]\cr
#'   Object to convert.
#' @param name.list [\code{logical(1)}]\cr
#'   Name resulting list with names of rows (cols) of \code{x}?
#'   Default is \code{FALSE}.
#' @param name.vector [\code{logical(1)}]\cr
#'   Name vector elements in resulting list with names of cols (rows) of \code{x}?
#'   Default is \code{FALSE}.
#' @param factors.as.char [\code{logical(1)}]\cr
#'   If \code{x} is a data.frame, convert factor columns to
#'   string elements in the resulting lists?
#'   Default is \code{TRUE}.
#' @param as.vector [\code{logical(1)}]\cr
#'   If \code{x} is a matrix, store rows as vectors in the resulting list - or otherwise as lists?
#'   Default is \code{TRUE}.
#' @return [\code{list} of lists or vectors].
#' @export
convertRowsToList = function(x, name.list = TRUE, name.vector = FALSE,
  factors.as.char = TRUE, as.vector = TRUE) {
  assert(checkMatrix(x), checkDataFrame(x))
  assertFlag(name.list)
  assertFlag(name.vector)
  assertFlag(factors.as.char)
  assertFlag(as.vector)
  ns.list = if (name.list) rownames(x) else NULL
  ns.vector = if (name.vector) colnames(x) else NULL
  if (is.matrix(x)) {
    if (as.vector)
      res = lapply(seq_row(x), function(i) setNames(x[i, ], ns.vector))
    else
      res = lapply(seq_row(x), function(i) setNames(as.list(x[i, ]), ns.vector))
  } else if (is.data.frame(x)) {
    if (factors.as.char)
      x = convertDataFrameCols(x, factors.as.char = TRUE)
    res = rowLapply(x, function(row) setNames(as.list(row), ns.vector))
  }
  setNames(res, ns.list)
}

#' @rdname convertRowsToList
#' @export
convertColsToList = function(x, name.list = FALSE, name.vector= FALSE,
  factors.as.char = TRUE, as.vector = TRUE) {

  # we need a special case for df and can ignore as.vector in it
  if (is.data.frame(x)) {
    if (factors.as.char)
      x = convertDataFrameCols(x, factors.as.char = TRUE)
    y = as.list(x)
    if (name.vector) {
      ns.vector = if (name.vector) colnames(x) else NULL
      y = lapply(y, function(z) setNames(z, ns.vector))
    }
    colnames(y) = if (name.list) colnames(x) else NULL
    return(y)
  }

  convertRowsToList(t(x), name.list = name.list, name.vector = name.vector,
    factors.as.char = factors.as.char, as.vector = as.vector)
}

context("convertRowsToList")

test_that("convertRowsToList", {
  expect_equal(
    convertRowsToList(matrix(1:4, 2, byrow=TRUE), as.vector = TRUE),
    list(c(1, 2), c(3, 4))
  )
  expect_equal(
    convertRowsToList(matrix(1:4, 2, byrow=TRUE), as.vector = FALSE),
    list(list(1, 2), list(3, 4))
  )
  expect_equal(
    convertRowsToList(setColNames(matrix(1:4, 2, byrow = TRUE), c("a", "b")),
      name.vector = TRUE, as.vector = FALSE),
    list(list(a=1, b=2), list(a=3, b=4))
  )
  expect_equal(
    convertRowsToList(setColNames(matrix(1:4, 2, byrow = TRUE), c("a", "b")),
      name.list = FALSE, as.vector = FALSE),
    list(list(1, 2), list(3, 4))
  )
  levs = c("a", "b")
  expect_equal(
    convertRowsToList(data.frame(a = 1:2, b = factor(c("a", "b"))),
      name.list = FALSE, factors.as.char = TRUE),
    list(list(1, "a"), list(2, "b"))
  )
  expect_equal(
    convertRowsToList(setRowNames(data.frame(a = 1:2, b = factor(c("a", "b"))), c("x", "y")),
      name.list = TRUE, name.vector = TRUE, factors.as.char = FALSE),
    list(x = list(a = 1, b = factor("a", levels = levs)), y = list(a = 2, b = factor("b", levels = levs)))
  )
})

test_that("convertColsToList", {
  expect_equal(
    convertColsToList(matrix(1:4, 2, byrow = FALSE), as.vector = TRUE),
    list(c(1, 2), c(3, 4))
  )
  expect_equal(
    convertColsToList(matrix(1:4, 2, byrow = FALSE), as.vector = FALSE),
    list(list(1, 2), list(3, 4))
  )
  expect_equal(
    convertColsToList(setRowNames(matrix(1:4, 2, byrow = FALSE), c("a", "b")),
      name.vector = TRUE, as.vector = FALSE),
    list(list(a = 1, b = 2), list(a = 3, b = 4))
  )
})


test_that("convertColsToList works with data.frame", {
  d1 = iris
  x1 = as.list(d1)
  expect_equal(convertColsToList(d1, factors.as.char = FALSE), x1)
  d2 = d1; d2$Species = as.character(d2$Species);
  x2 = as.list(d2)
  expect_equal(convertColsToList(d1, factors.as.char = TRUE), x2)
})

#' @title Load RData file and return objects in it
#'
#' @description
#' Loads an RData file and returns the objects directly instead of loading them into an environment.
#'
#' @param file [\code{character(1)}]\cr
#'   File to load.
#' @param parts [\code{character}]\cr
#'   Elements in file to load.
#'   Default is all.
#' @param simplify [\code{logical(1)}]\cr
#'   If \code{TRUE}, a list is only returned if \code{parts} and the file contain both more
#'   than 1 element, otherwise the element is directly returned.
#'   Default is \code{TRUE}.
#' @param envir [\code{environment(1)}]\cr
#'   Assign objects to this environment.
#'   Default is not to assign.
#' @param impute [\code{ANY}]\cr
#'   If \code{file} does not exists, return \code{impute} instead.
#'   Default is missing which will result in an exception if \code{file} is not found.
#' @return Either a single object or a list.
#' @export
#' @examples
#' fn = tempfile()
#' save2(file = fn, a = 1, b = 2, c = 3)
#' load2(fn, parts = "a")
#' load2(fn, parts = c("a", "c"))
load2 = function(file, parts, simplify = TRUE, envir, impute) {
  assertFlag(simplify)
  ee = new.env()
  if (!missing(impute) && !file.exists(file))
    return(impute)
  load(file, envir = ee)
  ns = ls(ee, all.names = TRUE)
  if (!missing(parts)) {
    assertCharacter(parts, any.missing = FALSE)
    d = setdiff(parts, ns)
    if (length(d) > 0L)
      stopf("File %s does not contain: %s", file, collapse(d))
  } else {
    parts = ns
  }
  if (!missing(envir)) {
    lapply(ns, function(x) assign(x, ee[[x]], envir = envir))
  }
  if (simplify) {
    if (length(ns) == 1L)
      return(ee[[ns]])
    if (length(parts) == 1L)
      return(ee[[parts]])
  }
  mget(parts, envir = ee)
}

context("load2")

test_that("load2", {
  fd = tempdir()
  fn = file.path(fd, "foo.RData")
  save2(file=fn, x=1)
  expect_equal(load2(fn), 1)
  expect_equal(load2(fn, parts="x"), 1)
  z = list(x=1)
  expect_equal(load2(fn, simplify=FALSE), z)
  expect_error(load2(fn, parts="y"), "does not contain: y")
  ee = new.env()
  load2(fn, envir=ee)
  expect_equal(ee$x, 1)

  fn2 = file.path(fd, "xxx.RData")
  expect_error(expect_warning(load2(fn2)))
  expect_equal(load2(fn2, impute = NA), NA)

  save2(file=fn, x=1, y=2)
  z = list(x=1, y=2)
  expect_equal(load2(fn), z)
  expect_equal(load2(fn, parts=c("x", "y")), z)
  expect_equal(load2(fn, parts="x"), 1)
  expect_equal(load2(fn, parts="y"), 2)
})

#' @title Collapse vector to string
#'
#' @description
#' A simple wrapper for \code{paste(x, collapse)}.
#'
#' @param x [\code{vector}]\cr
#'   Vector to collapse.
#' @param sep [\code{character(1)}]\cr
#'   Passed to \code{collapse} in \code{\link{paste}}.
#'   Default is \dQuote{,}.
#' @return [\code{character(1)}].
#' @export
#' @examples
#' collapse(c("foo", "bar"))
#' collapse(c("foo", "bar"), sep = ";")
collapse = function(x, sep = ",") {
  paste0(x, collapse = sep)
}

context("explode")

test_that("explode", {
  x = "R is a nice programming language"
  substrings = c("R", "is", "a", "nice", "programming", "language")
  sep = " "

  # split string
  exploded = explode(x, sep = sep)
  expect_equal(length(exploded), 6)
  for (i in 1:length(substrings)) {
    expect_equal(substrings[i], exploded[[i]])
  }

  # now glue the substrings together
  collapsed = collapse(exploded, sep = sep)
  expect_equal(collapsed, x)
})

#' @title Set a list element to a new value
#'
#' @description
#' This wrapper supports setting elements to \code{NULL}.
#'
#' @param obj [\code{list}]\cr
#' @param index [\code{character} | \code{integer}]\cr
#'   Index or indices where to insert the new values.
#' @param newval [any]\cr
#'   Inserted elements(s).
#'   Has to be a list if \code{index} is a vector.
#' @return [\code{list}]
#' @export
setValue = function(obj, index, newval) {
  assertList(obj)
  assert(checkCharacter(index, any.missing = FALSE), checkIntegerish(index, any.missing = FALSE))
  if (length(index) == 1L) {
    if (is.null(newval))
      obj[index] = list(NULL)
    else
      obj[index] = newval
  } else {
    assertList(newval, len = length(index))
    obj[index] = newval
  }
  return(obj)
}

context("setValue")

test_that("setValue", {
  xs1 = list(a=1, b=2)
  expect_equal(setValue(xs1, "b", 3), list(a=1, b=3))
  expect_equal(setValue(xs1, "b", NULL), list(a=1, b=NULL))
  expect_equal(setValue(xs1, "c", 3), list(a=1, b=2, c=3))
  expect_equal(setValue(xs1, c("a","b"), as.list(4:5)), list(a=4, b=5))
  expect_equal(setValue(xs1, c("b","c"), as.list(4:5)), list(a=1, b=4, c=5))
})

#' @title A wrapper for \code{\link{sort}} to sort using the \dQuote{C} collating rules
#'
#' @description
#' Sorts using the C locale for consistent sorting across platforms.
#'
#' @param ...
#'   Options passed to sort.
#' @return See \code{\link{sort}}.
#' @export
lsort = function(...) {
  cur = Sys.getlocale("LC_COLLATE")
  if (cur != "C") {
    Sys.setlocale("LC_COLLATE", "C")
    on.exit(Sys.setlocale("LC_COLLATE", cur))
  }
  sort(...)
}

#' @title Convert a list of row-vector of equal structure to a data.frame
#'
#' @description
#' Elements are arranged in columns according to their name in each
#' element of \code{rows}.
#' Variables that are not present in some row-lists, or encoded as \code{NULL}, are filled using NAs.
#'
#' @param rows [\code{list}]\cr
#'   List of rows. Each row is a list or vector of the same structure,
#'   where all corresponding elements must have the same class.
#'   It is allowed that in some rows some elements are not present, see above.
#' @param strings.as.factors [\code{logical(1)}]\cr
#'   Convert character columns to factors?
#'   Default is \code{default.stringsAsFactors()} for R < "4.1.0" and \code{FALSE} otherwise.
#' @param row.names [\code{character} | \code{integer} | \code{NULL}]\cr
#'   Row names for result.
#'   By default the names of the list \code{rows} are taken.
#' @param col.names [\code{character} | \code{integer}]\cr
#'   Column names for result.
#'   By default the names of an element of \code{rows} are taken.
#' @return [\code{data.frame}].
#' @export
#' @examples
#' convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")))
convertListOfRowsToDataFrame = function(rows, strings.as.factors = NULL,
  row.names, col.names) {
  assertList(rows)
  assertList(rows, types = "vector")
  if (!length(rows))
    return(makeDataFrame(0L, 0L))
  if (is.null(strings.as.factors)) {
    if(getRversion() < "4.1.0")
      strings.as.factors = default.stringsAsFactors()
    else
      strings.as.factors = FALSE
  }
  assertFlag(strings.as.factors)

  if (missing(row.names))
    row.names = names(rows)

  # make names
  rows = lapply(rows, function(x) setNames(x, make.names(names2(x, ""), unique = TRUE)))

  cols = unique(unlist(lapply(rows, names2)))
  if (anyMissing(cols))
    stop("All row elements must be named")
  if (!length(cols))
    return(makeDataFrame(length(rows), 0L))

  extract = function(cn) {
    tmp = lapply(rows, function(x) if (is.list(x)) x[[cn]] else unname(x[cn]))
    if (any(viapply(tmp, length) > 1L))
      stop("Rows may only contain a single value per name")
    simplify2array(replace(tmp, vlapply(tmp, is.null), NA))
  }

  d = data.frame(setNames(lapply(cols, extract), cols), row.names = row.names, stringsAsFactors = strings.as.factors)
  if (!missing(col.names))
    colnames(d) = col.names
  return(d)
}

#' @title Is one / are several files a directory?
#'
#' @description
#' If a file does not exist, \code{FALSE} is returned.
#'
#' @param ... [\code{character(1)}]\cr
#'   File names, all strings.
#' @return [\code{logical}].
#' @export
#' @examples
#' print(isDirectory(tempdir()))
#' print(isDirectory(tempfile()))
isDirectory = function(...) {
  paths = c(...)
  if (.Platform$OS.type == "windows" && getRversion() < "3.0.2")
    paths = sub("^([[:alpha:]]:)[/\\]*$", "\\1//", paths)
  x = file.info(paths)$isdir
  !is.na(x) & x
}

#' @title Is one / are several directories empty?
#'
#' @description
#' If file does not exist or is not a directory, \code{FALSE} is returned.
#'
#' @param ... [\code{character(1)}]\cr
#'   Directory names, all strings.
#' @return [\code{logical}].
#' @export
#' @examples
#' print(isEmptyDirectory(tempdir()))
#' print(isEmptyDirectory(tempfile()))
isEmptyDirectory = function(...) {
  vapply(list(...), FUN.VALUE = TRUE, FUN = function(x) {
    isDirectory(x) && length(list.files(x, all.files = TRUE, include.dirs = TRUE)) == 2L
  })
}

context("directory functions")

test_that("isDirectory", {
  expect_true(isDirectory("."))
  expect_identical(isDirectory(".", ".."), c(TRUE, TRUE))
  expect_false(isDirectory("foofoo"))
  expect_identical(isDirectory(".", "foofoo"), c(TRUE, FALSE))
})


test_that("isEmptyDirectory", {
  expect_false(isEmptyDirectory("."))
  td = tempfile()
  dir.create(td)
  expect_true(isEmptyDirectory(td))
  expect_identical(isEmptyDirectory(td, ".."), c(TRUE, FALSE))
  expect_false(isEmptyDirectory("foofoo"))
  expect_false(isEmptyDirectory(tempfile()))
})

#' @title Converts any R object to a descriptive string so it can be used in messages
#'
#' @description
#' Atomics: If of length 0 or 1, they are basically printed as they are.
#' Numerics are formated with \code{num.format}.
#' If of length greater than 1, they are collapsed witd \dQuote{,} and clipped.
#' so they do not become excessively long.
#' Expressions will be converted to plain text.
#'
#' All others: Currently, only their class is simply printed
#' like \dQuote{<data.frame>}.
#'
#' Lists: The mechanism above is applied (non-recursively) to their elements.
#' The result looks like this:
#' \dQuote{a=1, <unamed>=2, b=<data.frame>, c=<list>}.
#'
#' @param x [any]\cr
#'   The object.
#' @param num.format [\code{character(1)}]\cr
#'   Used to format numerical scalars via \code{\link{sprintf}}.
#'   Default is \dQuote{\%.4g}.
#' @param clip.len [\code{integer(1)}]\cr
#'   Used clip atomic vectors via \code{\link{clipString}}.
#'   Default is 15.
#' @return [\code{character(1)}].
#' @export
#' @examples
#' convertToShortString(list(a = 1, b = NULL, "foo", c = 1:10))
convertToShortString = function(x, num.format = "%.4g", clip.len = 15L) {

  # convert non-list object to string
  convObj = function(x) {
    cl = getClass1(x)
    string = 
      if (is.atomic(x) && !is.null(x) && length(x) == 0L)
        sprintf("%s(0)", getClass1(x))
      else if (cl == "numeric")
        paste(sprintf(num.format, x), collapse=",")
      else if (cl == "integer")
        paste(as.character(x), collapse=",")
      else if (cl == "logical")
        paste(as.character(x), collapse=",")
      else if (cl == "character")
        collapse(x)
      else if (cl == "expression")
        as.character(x)
      else
        sprintf("<%s>", cl)
    clipString(string, clip.len)
  }

  # handle only lists and not any derived data types
  if (getClass1(x) == "list") {
    if (length(x) == 0L)
      return("list()")
    ns = names2(x, missing.val = "<unnamed>")
    ss = lapply(x, convObj)
    collapse(paste(ns, "=", ss, sep = ""), ", ")
  } else {
    convObj(x)
  }
}

context("convertToShortString")

test_that("convertToShortString", {
  expect_equal(convertToShortString(1L), "1")
  expect_equal(convertToShortString(1.0), "1")
  expect_equal(convertToShortString(1.23), "1.23")
  expect_equal(convertToShortString(numeric(0)), "numeric(0)")
  expect_equal(convertToShortString(factor(c())), "factor(0)")
  expect_equal(convertToShortString(iris), "<data.frame>")
  expect_equal(convertToShortString(NULL), "<NULL>")
  expect_equal(convertToShortString(c(a=1, b=2)), "1,2")
  expect_equal(convertToShortString(expression(a + b + 3)), "a + b + 3")
  expect_equal(convertToShortString(list(a=1, 45)), "a=1, <unnamed>=45")
  expect_equal(convertToShortString(list(a=1, b=list(x=3))), "a=1, b=<list>")
  expect_equal(convertToShortString(list(a=1, b=iris)), "a=1, b=<data.frame>")
            
  expect_equal(convertToShortString(list()), "list()")
  expect_equal(convertToShortString(list(a=1)), "a=1")
  expect_equal(convertToShortString(list(a=1:2)), "a=1,2")
  expect_equal(convertToShortString(list(a=1:20)), "a=1,2,3,4,5,6,...")
  expect_equal(convertToShortString(list(a=1, 2, b=3)), "a=1, <unnamed>=2, b=3")
  expect_equal(convertToShortString(list(a=1, 2, b=data.frame())), "a=1, <unnamed>=2, b=<data.frame>")
  expect_equal(convertToShortString(list(a=identity, b=new.env(), c = NULL, d = expression(a + b + 3))), "a=<function>, b=<environment>, c=<NULL>, d=a + b + 3")

  expect_equal(convertToShortString(list(a=1, b=3.2)), "a=1, b=3.2")
  expect_equal(convertToShortString(list(a=1, b=3.223), num.format="%.2f"), "a=1.00, b=3.22")
  expect_equal(convertToShortString(list(a=1L, b=3.223), num.format="%.2f"), "a=1, b=3.22")
})

#' @title Save multiple objects to a file
#'
#' @description
#' A simple wrapper for \code{\link[base]{save}}. Understands key = value syntax to save
#' objects using arbitrary variable names. All options of \code{\link[base]{save}},
#' except \code{list} and \code{envir}, are available and passed to
#' \code{\link[base]{save}}.
#'
#' @param file
#'   File to save.
#' @param ... [\code{any}]\cr
#'   Will be converted to an environment and then passed to \code{\link[base]{save}}.
#' @param ascii
#'   See help of \code{\link[base]{save}}.
#' @param version
#'   See help of \code{\link[base]{save}}.
#' @param compress
#'   See help of \code{\link[base]{save}}.
#' @param compression_level
#'   See help of \code{\link[base]{save}}.
#' @param eval.promises
#'   See help of \code{\link[base]{save}}.
#' @param precheck
#'   See help of \code{\link[base]{save}}.
#' @return See help of \code{\link[base]{save}}.
#' @export
#' @examples
#' x = 1
#' save2(y = x, file = tempfile())
save2 = function(file, ..., ascii = FALSE, version = NULL, compress = !ascii,
  compression_level, eval.promises = TRUE, precheck = TRUE) {
  args = tryCatch(as.environment(argsAsNamedList(...)), error = function(e) stopf("Unable to convert to environment (%s)", as.character(e)))
  save(list = ls(args, all.names = TRUE), envir = args, file = file, ascii = ascii, version = version, compress = compress,
       compression_level = compression_level, eval.promises = eval.promises, precheck = precheck)
}

#' @title Require some packages
#'
#' @description
#' Packages are loaded either via \code{\link{requireNamespace}} or \code{\link{require}}.
#'
#' If some packages could not be loaded and \code{stop} is \code{TRUE}
#' the following exception is thrown:
#' \dQuote{For <why> please install the following packages: <missing packages>}.
#' If \code{why} is \code{NULL} the message is:
#' \dQuote{Please install the following packages: <missing packages>}.
#'
#' @param packs [\code{character}]\cr
#'   Names of packages.
#'   If a package name is prefixed with \dQuote{!}, it will be attached using \code{\link[base]{require}}.
#'   If a package name is prefixed with \dQuote{_}, its namespace will be loaded using \code{\link[base]{requireNamespace}}.
#'   If there is no prefix, argument \code{default.method} determines how to deal with package loading.
#' @param min.versions [\code{character}]\cr
#'   A char vector specifying required minimal version numbers for a subset of packages in \code{packs}.
#'   Must be named and all names must be in \code{packs}.
#'   The only exception is when \code{packs} is only a single string, then you are allowed to pass
#'   an unnamed version string here.
#'   Default is \code{NULL}, meaning no special version requirements
#' @param why [\code{character(1)}]\cr
#'   Short string explaining why packages are required.
#'   Default is an empty string.
#' @param stop [\code{logical(1)}]\cr
#'   Should an exception be thrown for missing packages?
#'   Default is \code{TRUE}.
#' @param suppress.warnings [\code{logical(1)}]\cr
#'   Should warnings be supressed while requiring?
#'   Default is \code{FALSE}.
#' @param default.method [\code{character(1)}]\cr
#'   If the packages are not explicitly prefixed with \dQuote{!} or \dQuote{_},
#'   this arguments determines the default. Possible values are \dQuote{attach} and
#'   \dQuote{load}.
#'   Note that the default is \dQuote{attach}, but this might/will change in a future version, so
#'   please make sure to always explicitly set this.
#' @return [\code{logical}]. Named logical vector describing which packages could be loaded (with required version).
#'   Same length as \code{packs}.
#' @export
#' @examples
#' requirePackages(c("BBmisc", "base"), why = "BBmisc example")
requirePackages = function(packs, min.versions = NULL, why = "", stop = TRUE, suppress.warnings = FALSE, default.method = "attach") {
  assertCharacter(packs, any.missing = FALSE)
  if (!is.null(min.versions)) {
    assertCharacter(min.versions)
    if (length(packs) == 1L && length(min.versions) == 1L && is.null(names(min.versions)))
      names(min.versions) = packs
    else
      assertSubset(names(min.versions), packs)
  }
  assertFlag(stop)
  assertFlag(suppress.warnings)
  assertChoice(default.method, choices = c("load", "attach"))

  char = substr(packs, 1L, 1L)
  force.attach = (char == "!")
  force.load = (char == "_")
  ns.only = if (default.method == "load") !force.attach else force.load
  packs = substr(packs, 1L + (force.load | force.attach), nchar(packs))
  suppressor = if (suppress.warnings) suppressWarnings else identity

  packs.ok = unlist(Map(function(pack, ns.only) {
    if (ns.only) {
      suppressor(requireNamespace(pack, quietly = TRUE))
    } else {
      suppressor(require(pack, character.only = TRUE))
    }
  }, pack = packs, ns.only = ns.only))

  if (stop && !all(packs.ok)) {
    ps = collapse(packs[!packs.ok])
    if (nzchar(why))
      stopf("For %s please install the following packages: %s", why, ps)
    else
      stopf("Please install the following packages: %s", ps)
  }

  if (!is.null(min.versions)) {
    packs.wrong.version = character(0L)
    for (j in seq_along(min.versions)) {
      pn = names(min.versions)[j]
      mv = min.versions[j]
      if (packageVersion(pn) < mv) {
        packs.ok[pn] = FALSE
        packs.wrong.version = c(packs.wrong.version, sprintf("%s >= %s", pn, mv))
      }
    }
    if (stop && length(packs.wrong.version) > 0L) {
      ps = collapse(packs.wrong.version)
      if (nzchar(why))
        stopf("For %s the package version requirements are not fulfilled: %s", why, ps)
      else
        stopf("The package version requirements are not fulfilled: %s", ps)
    }
  }

  return(packs.ok)
}

context("requirePackages")

test_that("requirePackages", {
  expect_equal(requirePackages("base"), c(base=TRUE))
  expect_equal(requirePackages("xxx", stop = FALSE, suppress.warnings = TRUE), c(xxx=FALSE))
  expect_error(requirePackages("xxx", suppress.warnings=TRUE), "Please install the following packages: xxx")
  expect_equal(requirePackages(c("xxx", "base"), stop=FALSE, suppress.warnings=TRUE), c(xxx=FALSE, base=TRUE))
  expect_equal(requirePackages(c("base", "xxx"), stop=FALSE, suppress.warnings=TRUE), c(base=TRUE, xxx=FALSE))
  expect_error(requirePackages(c("base", "xxx"), suppress.warnings=TRUE), "Please install the following packages: xxx")
  expect_error(requirePackages(c("base", "xxx"), why="test", suppress.warnings=TRUE), "For test please install the following packages: xxx")

  skip_if_not_installed("codetools")
  # test loading vs. attaching using the codetools package
  expect_equal(requirePackages("codetools", default.method = "load"), c(codetools=TRUE))
  expect_true("codetools" %in% loadedNamespaces())
  expect_false("package:codetools" %in% search())
  expect_equal(requirePackages("!codetools", default.method = "load"), c(codetools=TRUE))
  expect_true("package:codetools" %in% search())
})

test_that("requirePackages with min.versions", {
  expect_equal(requirePackages("base", min.versions = "0.1"), c(base=TRUE))
  expect_equal(requirePackages("base", min.versions = c(base="0.1")), c(base=TRUE))

  expect_equal(requirePackages(c("base", "stats")), c(base=TRUE, stats = TRUE))
  expect_equal(requirePackages(c("base", "stats"), min.versions = c(base = "100"), stop = FALSE), c(base=FALSE, stats = TRUE))
  expect_error(requirePackages(c("base", "stats"), min.versions = c(stats = "100")), "stats >= 100")
})

#' @title Drop named elements of an object
#'
#' @description
#' Removes named elements from a vector, list, matrix or data frame.
#'
#' @param x [any]\cr
#'   Object to drop named elements from.
#'   For a matrix or a data frames this function drops named columns via
#'   the second argument of the binary index operator \code{[,]}.
#'   Otherwise, the unary index operator \code{[]} is used for dropping.
#' @param drop [\code{character}]\cr
#'   Names of elements to drop.
#' @return Subset of object of same type as \code{x}. The object is not simplified,
#'   i.e, no dimensions are dropped as \code{[,,drop = FALSE]} is used.
#' @export
dropNamed = function(x, drop = character(0L)) {
  assertCharacter(drop, any.missing = FALSE)
  if (length(drop) == 0L)
    return(x)

  if (is.matrix(x) || is.data.frame(x))
    x[, setdiff(colnames(x), drop), drop = FALSE]
  else
    x[setdiff(names(x), drop)]
}

context("dropNamed")

test_that("dropNamed", {
  x = matrix(1:4, 2, 2)
  colnames(x) = c("a", "b")
  y = dropNamed(x, character(0))
  expect_equal(y, x)
  y = dropNamed(x, c("a"))
  expect_equal(y, x[, "b", drop=FALSE])
  y = dropNamed(x, c("a", "b"))
  expect_equal(y, x[, character(0)])

  x = as.data.frame(x)
  y = dropNamed(x, character(0))
  expect_equal(y, x)
  y = dropNamed(x, c("a"))
  expect_equal(y, x[, "b", drop=FALSE])
  y = dropNamed(x, c("a", "b"))
  expect_equal(y, x[, character(0)])

  x = list(a=1, b=2)
  y = dropNamed(x, character(0))
  expect_equal(y, x)
  y = dropNamed(x, c("a"))
  expect_equal(y, x["b"])
  y = dropNamed(x, c("a", "b"))
  expect_equal(y, x[character(0)])

  x = c(a=1, b=2)
  y = dropNamed(x, character(0))
  expect_equal(y, x)
  y = dropNamed(x, c("a"))
  expect_equal(y, x["b"])
  y = dropNamed(x, c("a", "b"))
  expect_equal(y, x[character(0)])
})

#' @title Split seconds into handy chunks of time
#'
#' @description
#' Note that a year is simply defined as exactly 365 days.
#'
#' @param seconds [\code{numeric(1)}]\cr
#'   Number of seconds. If not an integer, it is rounded down.
#' @param unit [\code{character(1)}]\cr
#'   Largest unit to split seconds into.
#'   Must be one of: \code{c("years", "days", "hours", "minutes", "seconds")}.
#'   Default is \dQuote{years}.
#' @return [\code{numeric(5)}]. A named vector containing the
#' \dQuote{years}, \dQuote{days}, \dQuote{hours}, \dQuote{minutes}
#' and \dQuote{seconds}. Units larger than the given \code{unit} are
#' \code{NA}.
#' @export
#' @examples
#' splitTime(1000)
splitTime = function(seconds, unit = "years") {
  assertNumber(seconds)
	assertChoice(unit, c("years", "days", "hours", "minutes", "seconds"))
  divider = c(31536000L, 86400L, 3600L, 60L, 1L)
  res = setNames(rep.int(NA_integer_, 5L),
                 c("years", "days", "hours", "minutes", "seconds"))
  start = which(names(res) == unit)
  for (i in start:length(divider)) {
    res[i] = seconds %/% divider[i]
    seconds = seconds - res[i] * divider[i]
  }
  ## Make sure all values are integral and do _not_ strip names:
  viapply(res, as.integer)
}

context("splitTime")

test_that("splitTime", {
  expect_equal(splitTime(0, "years"), c(years=0, days=0, hours=0, minutes=0, seconds=0))
  expect_equal(splitTime(0, "days"), c(years=NA, days=0, hours=0, minutes=0, seconds=0))
  expect_equal(splitTime(0, "hours"), c(years=NA, days=NA, hours=0, minutes=0, seconds=0))
  expect_equal(splitTime(0, "minutes"), c(years=NA, days=NA, hours=NA, minutes=0, seconds=0))
  expect_equal(splitTime(0, "seconds"), c(years=NA, days=NA, hours=NA, minutes=NA, seconds=0))

  seconds = 2 * 365 * 24 * 60 * 60
  expect_equal(splitTime(seconds, "years"), c(years=2, days=0, hours=0, minutes=0, seconds=0))
  expect_equal(splitTime(seconds, "days"), c(years=NA, days=2 * 365, hours=0, minutes=0, seconds=0))
  expect_equal(splitTime(seconds, "hours"),
    c(years=NA, days=NA, hours=2 * 365 * 24, minutes=0, seconds=0))
  expect_equal(splitTime(seconds, "minutes"),
    c(years=NA, days=NA, hours=NA, minutes=2 * 365 * 24 * 60, seconds=0))
  expect_equal(splitTime(seconds, "seconds"),
    c(years=NA, days=NA, hours=NA, minutes=NA, seconds=seconds))

  expect_true(is.integer(splitTime(100000, "minutes")))
})

#' @title Current time in seconds
#'
#' @description
#' Simple wrapper for \code{as.integer(Sys.time())}.
#'
#' @return [\code{integer(1)}].
#' @export
getUnixTime = function() {
  as.integer(Sys.time())
}

#' @title Check superset relation on two vectors
#'
#' @description
#' Checks if all elements of y are contained in x.
#'
#' @param x [\code{vector}]\cr
#'   Source vector.
#' @param y [\code{vector}]\cr
#'   Vector of the same mode as \code{x}.
#' @param strict [\code{logical(1)}]\cr
#'   Checks for strict/proper superset relation.
#' @return [\code{logical(1)}]
#'   \code{TRUE} if each element of \code{y} is also contained in \code{x}, i. e.,
#'   if \code{y} is a subset of \code{x} and \code{FALSE} otherwise.
#' @export
isSuperset = function(x, y, strict = FALSE) {
  isSubset(y, x, strict)
}

#' @title Checks whether an object is a scalar NA value
#'
#' @description
#' Checks whether object is from \code{(NA, NA_integer, NA_real_, NA_character_, NA_complex_)}.
#' @param x [any]\cr
#'   Object to check.
#' @return [\code{logical(1)}].
#' @export
isScalarNA = function(x) {
  is.atomic(x) && length(x) == 1L && is.na(x)
}

context("isScalarNA")

test_that("isScalarNA", {
  expect_true(isScalarNA(NA))
  expect_false(isScalarNA(1))
  expect_false(isScalarNA(iris))
  expect_false(isScalarNA(NULL))
  expect_false(isScalarNA("NA"))
})

#' @title A wrapper for \code{identical(x, FALSE)}
#'
#' @description
#' Tests if an object is identical to FALSE.
#'
#' @param x [any]\cr
#'   Your object.
#' @return [\code{logical(1)}].
#' @export
#' @examples
#' isFALSE(0)
#' isFALSE(FALSE)
isFALSE = function(x) {
  identical(x, FALSE)
}

#' @title A caching wrapper around load2
#'
#' @description
#' This closure returns a wrapper around \code{\link{load2}} which per
#' default caches loaded objects and returns the cached version
#' in subsequent calls.
#'
#' @param use.cache [\code{logical(1)}]\cr
#'  Enable the cache?
#'  Default is \code{TRUE}.
#' @return [\code{function()}] with argument \code{slot}
#'  (name of the slot to cache the object in, default is \dQuote{default}).
#'  All other arguments are passed down to \code{\link{load2}}.
#' @export
makeFileCache = function(use.cache = TRUE) {
  assertFlag(use.cache)
  .cache = list()

  function(file, slot = "default", ...) {
    if (use.cache) {
      if (is.null(.cache[[slot]]) || .cache[[slot]]$file != file)
        .cache[[slot]] = list(file = file, obj = load2(file = file, ...))
      return(.cache[[slot]]$obj)
    }
    return(load2(file = file, ...))
  }
}

#' @title Check if given object has certain attributes
#'
#' @description
#' Checks whether an object has all specified attributes.
#'
#' @param obj [mixed]\cr
#'   Arbitrary R object.
#' @param attribute.names [\code{character}]\cr
#'   Vector of strings, i.e., attribute names.
#' @return [\code{logical(1)}]
#'   \code{TRUE} if object \code{x} contains all attributes from \code{attributeNames}
#'   and \code{FALSE} otherwise.
#' @export
hasAttributes = function(obj, attribute.names) {
  isSubset(attribute.names, getAttributeNames(obj))
}

context("hasAttributes")

test_that("hasAttributes", {
  x = 1:10
  attribute.names = c("size", "importance")

  expect_false(hasAttributes(x, attribute.names))

  attr(x, "size") = length(x)
  expect_false(hasAttributes(x, attribute.names))

  attr(x, "importance") = "very important"
  expect_true(hasAttributes(x, attribute.names))
})

#' @title Naive multi-start version of \code{\link{optimize}} for global optimization
#'
#' @description
#' The univariate \code{\link{optimize}} can stop at arbitrarily bad points when
#' \code{f} is not unimodal. This functions mitigates this effect in a very naive way:
#' \code{interval} is subdivided into \code{nsub} equally sized subintervals,
#' \code{\link{optimize}} is run on all of them (and on the original big interval) and
#' the best obtained point is returned.
#'
#' @param f See \code{\link{optimize}}.
#' @param interval See \code{\link{optimize}}.
#' @param ... See \code{\link{optimize}}.
#' @param lower See \code{\link{optimize}}.
#' @param upper See \code{\link{optimize}}.
#' @param maximum See \code{\link{optimize}}.
#' @param tol See \code{\link{optimize}}.
#' @param nsub [\code{integer(1)}]\cr
#'   Number of subintervals. A value of 1 implies normal \code{\link{optimize}} behavior.
#'   Default is 50L.
#' @return See \code{\link{optimize}}.
#' @export
optimizeSubInts = function(f, interval, ..., lower = min(interval), upper = max(interval),
  maximum = FALSE, tol = .Machine$double.eps^0.25, nsub = 50L) {
  nsub = asCount(nsub, positive = TRUE)

  interval = c(lower, upper)
  # run on normal interval
  best = optimize(f = f, interval = interval, maximum = maximum, tol = tol)

  # run on smaller partitions
  if (nsub > 1L) {
    mult = ifelse(maximum, -1, 1)
    grid = seq(lower, upper, length.out = nsub + 1L)
    for (j in seq_len(length(grid)-1L)) {
      res = optimize(f = f, interval = c(grid[j], grid[j+1L]), maximum = maximum, tol = tol)
      if (mult * res$objective < mult * best$objective)
        best = res
    }
  }
  return(best)
}

context("optimizeSubInts")

test_that("optimizeSubInts", {

  f = function(x) sin(x) * x
  z = optimizeSubInts(f, interval = c(0, 50), nsub = 200L)
  fopt = f(pi * 3 / 2 + 14 * pi)
  expect_true(abs(fopt -  z$objective) < 1e-1)

  # test with nsub = 1, had a bug here
  f = function(x) sum(x^2)
  z = optimizeSubInts(f, interval = c(-10, 10), nsub = 1)
  expect_true(abs(z$minimum) < 1e-5)
})

#' @title Normalizes numeric data to a given scale
#'
#' @description
#' Currently implemented for numeric vectors, numeric matrices and data.frame.
#' For matrixes one can operate on rows or columns
#' For data.frames, only the numeric columns are touched, all others are left unchanged.
#' For constant vectors / rows / columns most methods fail, special behaviour for this
#' case is implemented.
#'
#' The method also handles NAs in in \code{x} and leaves them untouched.
#'
#' @param x [\code{numeric} | \code{matrix} | \code{data.frame}]\cr
#'   Input vector.
#' @param method [\code{character(1)}]\cr
#'   Normalizing method. Available are:\cr
#'   \dQuote{center}: Subtract mean.\cr
#'   \dQuote{scale}: Divide by standard deviation.\cr
#'   \dQuote{standardize}: Center and scale.\cr
#'   \dQuote{range}: Scale to a given range.\cr
#' @param range [\code{numeric(2)}]\cr
#'   Range for method \dQuote{range}.
#'   The first value represents the replacement for the min value, the second is the substitute for the max value.
#'   So it is possible to reverse the order by giving \code{range = c(1,0)}.
#'   Default is \code{c(0,1)}.
#' @param margin [\code{integer(1)}]\cr
#'   1 = rows, 2 = cols.
#'   Same is in \code{\link{apply}}
#'   Default is 1.
#' @param on.constant [\code{character(1)}]\cr
#'   How should constant vectors be treated? Only used, of \dQuote{method != center},
#'   since this methods does not fail for constant vectors. Possible actions are:\cr
#'   \dQuote{quiet}: Depending on the method, treat them quietly:\cr
#'     \dQuote{scale}: No division by standard deviation is done, input values.
#'        will be returned untouched.\cr
#'     \dQuote{standardize}: Only the mean is subtracted, no division is done.\cr
#'     \dQuote{range}: All values are mapped to the mean of the given range.\cr
#'   \dQuote{warn}: Same behaviour as \dQuote{quiet}, but print a warning message.\cr
#'   \dQuote{stop}: Stop with an error.\cr
#' @return [\code{numeric} | \code{matrix} | \code{data.frame}].
#' @seealso \code{\link{scale}}
#' @export
normalize = function(x, method = "standardize", range = c(0, 1), margin = 1L, on.constant = "quiet") {
  assertChoice(method, c("range", "standardize", "center", "scale"))
  assertNumeric(range, len = 2L, any.missing = FALSE)
  assertChoice(on.constant, c("quiet", "warn", "stop"))
  UseMethod("normalize")
}

#' @export
normalize.numeric = function(x, method = "standardize", range = c(0, 1), margin = 1L, on.constant = "quiet") {
  y = normalize2(x, method, range, on.constant = on.constant)
  # scale call below returns matrices
  if (is.matrix(y))
    y = y[,1L]
  return(y)
}

#' @export
normalize.matrix = function(x, method = "standardize", range = c(0, 1), margin = 1L, on.constant = "quiet") {
  x = apply(x, margin, normalize2, method = method, range = range, on.constant = on.constant)
  if (margin == 1L)
    x = t(x)
  return(x)
}

#' @export
normalize.data.frame = function(x, method = "standardize", range = c(0, 1), margin = 1L, on.constant = "quiet") {
  isnum = sapply(x, is.numeric)
  if (any(isnum))
    x[, isnum] = as.data.frame(lapply(x[, isnum, drop = FALSE], normalize2, method = method,
      range = range, on.constant = on.constant))
  return(x)
}

normalize2 = function(x, method, range, on.constant) {
  # is x a constant vector?
  if (length(unique(x[!is.na(x)])) == 1L) {
    switch(on.constant,
      warn = warning("Constant vector in normalization."),
      stop = stop("Constant vector in normalization."))
    switch(method,
      center = scale(x, center = TRUE, scale = FALSE),
      range = ifelse(is.na(x), NA, mean(range)),
      standardize = scale(x, center = TRUE, scale = FALSE),
      scale = x
    )
  } else {
    switch(method,
      range = (x - min(x, na.rm = TRUE)) / diff(range(x, na.rm = TRUE)) * diff(range) + range[1L],
      standardize = scale(x, center = TRUE, scale = TRUE),
      center = scale(x, center = TRUE, scale = FALSE),
      scale = scale(x, center = FALSE, scale = sd(x, na.rm = TRUE))
    )
  }
}

context("normalize")

test_that("normalize", {
  # vector
  x = runif(20)
  y = normalize(x, method = "range")
  expect_is(y, "numeric")
  expect_equal(range(y), c(0, 1))
  y = normalize(x, method = "range", range = c(-4, 2))
  expect_is(y, "numeric")
  expect_equal(range(y), c(-4, 2))
  y = normalize(x, method = "center")
  expect_is(y, "numeric")
  expect_equal(mean(y), 0)
  y = normalize(x, method = "standardize")
  expect_is(y, "numeric")
  expect_equal(mean(y), 0)
  expect_equal(sd(y), 1)

  # matrix
  x = matrix(runif(100), nrow = 5)
  y = normalize(x, margin = 1L)
  expect_is(y, "matrix")
  apply(y, 1, function(v) expect_equal(mean(v), 0))
  apply(y, 1, function(v) expect_equal(sd(v), 1))
  y = normalize(x, margin = 2L)
  apply(y, 2, function(v) expect_equal(mean(v), 0))
  apply(y, 2, function(v) expect_equal(sd(v), 1))

  # data.frame
  y = normalize(iris, method = "range", range = c(3, 4))
  expect_is(y, "data.frame")
  for (i in 1:4)
    expect_equal(range(y[, i]), c(3, 4))
  expect_equal(y[, 5L], iris$Species)

  # constant vectors
  x = rep(1, 10)
  y = normalize(x, method = "center", on.constant = "quiet")
  expect_is(y, "numeric")
  expect_equal(y, x - x)
  y = normalize(x, method = "scale", on.constant = "quiet")
  expect_is(y, "numeric")
  expect_equal(y, x)
  y = normalize(x, method = "standardize", on.constant = "quiet")
  expect_is(y, "numeric")
  expect_equal(y, x - x)
  y = normalize(x, method = "range", on.constant = "quiet", range = c(-3, 2))
  expect_is(y, "numeric")
  expect_equal(y, rep(-0.5, 10))
  expect_error(normalize(x, method = "center", on.constant = "stop"))
  expect_error(normalize(x, method = "scale", on.constant = "stop"))
  expect_error(normalize(x, method = "standardize", on.constant = "stop"))
  expect_error(normalize(x, method = "range", on.constant = "stop"))
  expect_warning(normalize(x, method = "center", on.constant = "warn"))
  expect_warning(normalize(x, method = "scale", on.constant = "warn"))
  expect_warning(normalize(x, method = "standardize", on.constant = "warn"))
  expect_warning(normalize(x, method = "range", on.constant = "warn"))
})

test_that("normalize works with NAs", {
  # vector
  x = c(1, 2, NA)
  y = normalize(x, method = "range")
  expect_equal(y, c(0, 1, NA))
  y = normalize(x, method = "center")
  expect_equal(y, c(-0.5, 0.5, NA))

  # matrix
  x = matrix(c(1, 2, 1, NA), nrow = 2L)
  y = normalize(x, margin = 2L, method = "range")
  expect_equal(y, matrix(c(0, 1, 0.5, NA), nrow = 2L))
})

#' @title Is given argument an atomic vector or factor of length 1?
#'
#' @description
#' More specific functions for scalars of a given type exist, too.
#'
#' @param x [any]\cr
#'   Argument.
#' @param na.ok [\code{logical(1)}]\cr
#'   Is \code{NA} considered a scalar?
#'   Default is \code{TRUE}.
#' @param null.ok [\code{logical(1)}]\cr
#'   Is \code{NULL} considered a scalar?
#'   Default is \code{FALSE}.
#' @param type [\code{character(1)}]\cr
#'   Allows to restrict to specific type, e.g., \dQuote{numeric}?
#'   But instead of this argument you might want to consider using \code{isScalar<Type>}.
#'   Default is \dQuote{atomic}, so no special restriction.
#' @return [\code{logical(1)}].
#' @export
isScalarValue = function(x, na.ok = TRUE, null.ok = FALSE, type = "atomic") {
  if (is.null(x))
    return(null.ok)
  # not really cool switch, but maybe fastest option
  istype = switch(type,
    "atomic" = is.atomic,
    "logical" = is.logical,
    "numeric" = is.numeric,
    "integer" = is.integer,
    "complex" = is.complex,
    "chararacter" = is.character,
    "factor" = is.factor
  )
  istype(x) && length(x) == 1L && (na.ok || !is.na(x))
}

#' @rdname isScalarValue
#' @export
isScalarLogical = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "logical")
}

#' @rdname isScalarValue
#' @export
isScalarNumeric = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "numeric")
}

#' @rdname isScalarValue
#' @export
isScalarInteger = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "integer")
}

#' @rdname isScalarValue
#' @export
isScalarComplex = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "complex")
}

#' @rdname isScalarValue
#' @export
isScalarCharacter = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "chararacter")
}

#' @rdname isScalarValue
#' @export
isScalarFactor = function(x, na.ok = TRUE, null.ok = FALSE) {
  isScalarValue(x, na.ok, null.ok, "factor")
}

context("isScalarValue")

test_that("isScalarValue", {
  expect_true(isScalarValue(1))
  expect_true(isScalarValue(1L))
  expect_true(isScalarValue("a"))
  expect_true(isScalarValue(factor("a")))
  expect_true(isScalarValue(as.complex(1)))
  expect_true(isScalarValue(NA))
  
  expect_true(isScalarNumeric(1))
  expect_true(isScalarInteger(1L))
  expect_true(isScalarCharacter("a"))
  expect_true(isScalarFactor(factor("a")))
  expect_true(isScalarComplex(as.complex(1)))
  expect_true(isScalarLogical(NA))
  
  expect_false(isScalarComplex(1L))
  expect_false(isScalarInteger(1))
  expect_false(isScalarFactor("a"))
  expect_false(isScalarCharacter(factor("a")))
  expect_false(isScalarNumeric(as.complex(1)))
  expect_false(isScalarInteger(NA))
  
  expect_false(isScalarValue(NULL))
  expect_false(isScalarValue(iris))
  expect_false(isScalarValue(1:2))
  expect_false(isScalarValue(list(1)))
  
  expect_true(isScalarValue(NULL, null.ok=TRUE))
  expect_false(isScalarValue(NULL, na.ok=FALSE))
})

#' @title Wrapper for system2 with better return type and errorhandling
#'
#' @description
#' Wrapper for \code{\link{system2}} with better return type and errorhandling.
#'
#' @param command See \code{\link{system2}}.
#' @param args See \code{\link{system2}}.
#' @param stdout See \code{\link{system2}}.
#' @param stderr See \code{\link{system2}}.
#' @param wait See \code{\link{system2}}.
#' @param ... Further arguments passed to \code{\link{system2}}.
#' @param stop.on.exit.code [\code{logical(1)}]\cr
#'   Should an exception be thrown if an exit code greater 0 is generated?
#'   Can only be used if  \code{wait} is \code{TRUE}.
#'   Default is \code{wait}.
#' @return [\code{list}].
#'   \item{exit.code [integer(1)]}{Exit code of command. Given if wait is \code{TRUE}, otherwise \code{NA}. 0L means success. 127L means command was not found}
#'   \item{output [character]}{Output of command on streams. Only given is \code{stdout} or \code{stderr} was set to \code{TRUE}, otherwise \code{NA}.}
#' @export
system3 = function(command, args = character(0L), stdout = "", stderr = "", wait = TRUE, ..., stop.on.exit.code = wait) {
  if (stop.on.exit.code && !wait)
    stopf("stop.on.exit.code is TRUE but wait is FALSE!")
  output = NA_character_
  if (isTRUE(stdout) || isTRUE(stderr)) {
    wait = TRUE
    # here we wait anyway and output of cmd is returned
    ec = 0L
    suppressWarnings({
        withCallingHandlers({
            op = system2(command = command, args = args, stdout = stdout, stderr = stderr, wait = wait, ...)
          }, warning = function(w) {
            g = gregexpr("\\d+", w$message)[[1L]]
            start = tail(g, 1L)
            len = tail(attr(g, "match.length"), 1L)
            ec <<- as.integer(substr(w$message, start, start + len - 1L))
          })
      })
  } else {
    ec = system2(command = command, args = args, stdout = stdout, stderr = stderr, wait = wait, ...)
  }
  if (wait) {
    if (isTRUE(stdout) || isTRUE(stderr))
      output = op
  }
  if (stop.on.exit.code && ec > 0L) {
    args = collapse(args, " ")
    if (length(output) == 0L)
      output = ""
    else
      output = collapse(output, "\n")
    stopf("Command: %s %s; exit code: %i; output: %s", command, args, ec, output)
  }
  list(exit.code = ec, output = output)
}

context("system3")

if (interactive()) {
  
test_that("system3", {  
  d = tempfile()
  dir.create(d)
  fn = file.path(d, "foo.bar")
  file.create(fn)
  
  # no error
  res = system3("ls", d)
  expect_equal(res, list(exit.code=0L, output=as.character(NA)))
  res = system3("ls", d, stdout=TRUE, stderr=TRUE)
  expect_equal(res, list(exit.code=0L, output="foo.bar"))
  
  # wrong command
  res = system3("xxx", stop.on.exit.code=FALSE)
  expect_equal(res, list(exit.code=127L, output=as.character(NA)))
  expect_error(system3("xxx", stop.on.exit.code=TRUE), 
    "Command: xxx ; exit code: 127; output: NA")
  
  # exit code
  res = system3("ls", "xxx", stop.on.exit.code=FALSE)
  expect_equal(res, list(exit.code=2L, output=as.character(NA)))
  res = system3("ls", "xxx", stdout=TRUE, stderr=TRUE, stop.on.exit.code=FALSE)
  msg = "ls: cannot access xxx: No such file or directory"
  expect_equal(res$exit.code, 2L)
  expect_true(grep("ls:", res$output) == 1)
  expect_true(grep("xxx", res$output) == 1)
  expect_error(system3("ls", "xxx", stdout=TRUE, stderr=TRUE, stop.on.exit.code=TRUE),
    "Command: ls xxx; exit code: 2; output: ls:")
  expect_error(system3("ls", "xxx", stdout=TRUE, stderr=TRUE, stop.on.exit.code=TRUE),
    "xxx")
  expect_error(system3("ls", c("1", "2"), stdout=TRUE, stderr=TRUE, stop.on.exit.code=TRUE),
    "Command: ls")
})
  
}

#' @title A wrapper for \code{library}
#'
#' @description
#' Tries to load packages. If the packages are not found, they will be installed from
#' the default repository. This function is intended for use in interactive sessions
#' and should not be used by other packages.
#'
#' @param ... [any]\cr
#'   Package names.
#' @return [\code{logical}]: Named logical vector determining the success
#'   of package load.
#' @export
#' @examples \dontrun{
#' lib("BBmisc", "MASS", "rpart")
#' }
lib = function(...) {
  getLib = function(pkg) {
    ok = suppressWarnings(require(pkg, character.only = TRUE))
    if (!ok && !is.error(try(install.packages(pkg)))) {
      ok = require(pkg, character.only = TRUE)
    }
    ok
  }

  pkgs = unique(c(...))
  assertCharacter(pkgs, any.missing = FALSE)
  vlapply(pkgs, getLib)
}

#' @title Wrapper for stop and sprintf
#'
#' @description
#' A wrapper for \code{\link{stop}} with \code{\link{sprintf}} applied to the arguments.
#' Notable difference is that error messages are not truncated to 1000 characters
#' by default.
#'
#' @param ... [any]\cr
#'   See \code{\link{sprintf}}.
#' @param warning.length [\code{integer(1)}]\cr
#'   Number of chars after which the error message
#'   gets truncated, see ?options.
#'   Default is 8170.
#' @return Nothing.
#' @export
#' @examples
#' err = "an error."
#' try(stopf("This is %s", err))
stopf = function(..., warning.length = 8170L) {
  msg = sprintf(...)
  obj = simpleError(msg, call = sys.call(sys.parent()))
  old.opt = getOption("warning.length")
  on.exit(options(warning.length = old.opt))
  options(warning.length = warning.length)
  stop(obj)
}

#' @title Simply a negated \code{in} operator
#'
#' @description
#' Tests if elements of x are not in y.
#'
#' @param x [\code{vector}]\cr
#'   Values that should not be in \code{y}.
#' @param y [\code{vector}]\cr
#'   Values to match against.
#' @usage x \%nin\% y
#' @rdname nin
#' @export
`%nin%` = function(x, y) {
  !match(x, y, nomatch = 0L)
}

#' @title Functions to determine the operating system
#'
#' @description
#' \describe{
#' \item{getOperatingSystem}{Simple wrapper for \code{.Platform$OS.type}, returns \code{character(1)}.}
#' \item{isUnix}{Predicate for OS string, returns \code{logical(1)}. Currently this would include Unix, Linux and Mac flavours.}
#' \item{isLinux}{Predicate for sysname string, returns \code{logical(1)}.}
#' \item{isDarwin}{Predicate for sysname string, returns \code{logical(1)}.}
#' \item{isWindows}{Predicate for OS string, returns \code{logical(1)}.}
#' }
#'
#' @return See above.
#' @export
getOperatingSystem = function() {
  .Platform$OS.type
}

#' @rdname getOperatingSystem
#' @export
isWindows = function() {
  .Platform$OS.type == "windows"
}

#' @rdname getOperatingSystem
#' @export
isUnix = function() {
  .Platform$OS.type == "unix"
}

#' @rdname getOperatingSystem
#' @export
isLinux = function() {
  isUnix() && grepl("linux", Sys.info()["sysname"], ignore.case = TRUE)
}

#' @rdname getOperatingSystem
#' @export
isDarwin = function() {
  isUnix() && grepl("darwin", Sys.info()["sysname"], ignore.case = TRUE)
}

context("getOperatingSystem")

test_that("getOperatingSystem", {
  x = getOperatingSystem()
  expect_true(is.character(x) && length(x) == 1 && nchar(x) > 0)
  x = isWindows()
  expect_true(is.logical(x) && length(x) == 1 && !is.na(x))
  x = isUnix()
  expect_true(is.logical(x) && length(x) == 1 && !is.na(x))
})

#' @title Extracts a named element from a list of lists
#'
#' @description
#' Extracts elements from a list of named lists by element name.
#'
#' @param xs [\code{list}]\cr
#'   A list of named lists.
#' @param element [\code{character}]\cr
#'   Name of element(s) to extract from the list elements of \code{xs}.
#'   What happens is this: \code{x$el1$el2....}.
#' @param element.value [any]\cr
#'   If given, \code{\link{vapply}} is used and this argument is passed to \code{FUN.VALUE}.
#'   Note that even for repeated indexing (if length(element) > 1) you only
#'   pass one value here which refers to the data type of the final result.
#' @param simplify [\code{logical(1)} | character(1)]\cr
#'   If \code{FALSE} \code{\link{lapply}} is used, otherwise \code{\link{sapply}}.
#'   If \dQuote{cols}, we expect the elements to be vectors of the same length and they are
#'   arranged as the columns of the resulting matrix.
#'   If \dQuote{rows}, likewise, but rows of the resulting matrix.
#'   Default is \code{TRUE}.
#' @param use.names [\code{logical(1)}]\cr
#'   If \code{TRUE} and \code{xs} is named, the result is named as \code{xs},
#'   otherwise the result is unnamed.
#'   Default is \code{TRUE}.
#' @return [\code{list} | simplified \code{vector} | \code{matrix}]. See above.
#' @export
#' @examples
#' xs = list(list(a = 1, b = 2), list(a = 5, b = 7))
#' extractSubList(xs, "a")
#' extractSubList(xs, "a", simplify = FALSE)
extractSubList = function(xs, element, element.value, simplify = TRUE, use.names = TRUE) {
  assertList(xs)
  assert(checkFlag(simplify), checkChoice(simplify, c("cols", "rows")))
  assertFlag(use.names)

  # we save some time here if we only do the for loop in the complicated case
  # the whole function is still not very nice due to the loop
  # extractSubList should be C code anyway i guess....
  doindex = if (length(element) == 1L) {
    function(x) x[[element]]
  } else {
    function(x) {
      for (el in element)
        x = x[[el]]
      return(x)
    }
  }

  if (!missing(element.value)) {
    ys = vapply(xs, doindex, FUN.VALUE = element.value)
  } else if (isTRUE(simplify)) {
    ys = sapply(xs, doindex, USE.NAMES = use.names)
   } else {
    ys = lapply(xs, doindex)
    if (simplify == "rows")
      ys = asMatrixRows(ys)
    else if (simplify == "cols")
      ys = asMatrixCols(ys)
  }
  ns = names(xs)
  if (use.names && !is.null(ns)) {
    if (isTRUE(simplify))
      names(ys) = ns
    else if (simplify == "rows")
      rownames(ys) = ns
    else if (simplify == "cols")
      colnames(ys) = ns
  } else {
    if (simplify %in% c("rows", "rows"))
      dimnames(ys) = NULL
    else
      names(ys) = NULL
  }
  return(ys)
}

context("extractSubList")

test_that("extractSubList", {
  xs = list(
    a = list(x = 1, y = "foo", z = matrix(1,1,1)),
    b = list(x = 2L, y = "bar", z = matrix(2,2,2))
  )
  expect_equal(extractSubList(xs, "x"), c(a = 1, b = 2))
  expect_equal(extractSubList(xs, "y"), c(a = "foo", b = "bar"))
  expect_equal(extractSubList(xs, "z"), list(a = matrix(1,1,1), b = matrix(2,2,2)))

  expect_equal(extractSubList(xs, "x", use.names = FALSE), c(1, 2))
  expect_equal(extractSubList(xs, "y", use.names = FALSE), c("foo", "bar"))
  expect_equal(extractSubList(xs, "z", use.names = FALSE), list(matrix(1,1,1), matrix(2,2,2)))

  expect_equal(extractSubList(list(), "x"), list())

  expect_equal(extractSubList(list(), "x", element.value = numeric(1)), numeric(0))
  expect_equal(extractSubList(list(), "y", element.value = character(1)), character(0))

  expect_equal(extractSubList(xs, "x", element.value = numeric(1)), c(a = 1, b = 2))
  expect_equal(extractSubList(xs, "y", element.value = character(1)), c(a = "foo", b = "bar"))

  xs = list(
    list(x = 1, y = "foo", z = matrix(1,1,1)),
    list(x = 2L, y = "bar", z = matrix(2,2,2))
  )
  expect_equal(extractSubList(xs, "y", use.names = TRUE), c("foo", "bar"))
  expect_equal(extractSubList(xs, "y", use.names = FALSE), c("foo", "bar"))

  expect_equal(
    extractSubList(list(list(a = 1:2), list(a = 3:4)), "a", simplify = "rows"),
    matrix(1:4, nrow = 2L, ncol = 2L, byrow = TRUE)
  )

  expect_equal(
    extractSubList(list(list(a = 1), list(a = 2)), "a", simplify = "rows"),
    matrix(1:2, nrow = 2L, ncol = 1)
  )
})




test_that("extractSubList works with repeated indexing", {
  xs = list(
    a = list(v = list(x = 1), w = list(y = "foo")),
    b = list(v = list(x = 2), w = list(y = "bar"))
  )
  expect_equal(extractSubList(xs, c("v", "x")), c(a = 1, b = 2))
  expect_equal(extractSubList(xs, c("w", "y")), c(a = "foo", b = "bar"))

  expect_equal(extractSubList(xs, c("v", "x"), element.value = numeric(1)), c(a = 1, b = 2))
  expect_equal(extractSubList(xs, c("w", "y"), element.value = character(1)), c(a = "foo", b = "bar"))


  expect_equal(extractSubList(xs, c("v", "x"), simplify = "rows", use.names = FALSE), matrix(c(1, 2), nrow = 2))
  expect_equal(extractSubList(xs, c("v", "x"), simplify = "cols", use.names = TRUE),
    setColNames(matrix(c(1, 2), nrow = 1), c("a", "b")))
})

#' @title Replace values in atomic vectors
#'
#' @details
#' Replaces values specified in \code{from} with values in \code{to}.
#' Regular expression matching can be enabled which calls \code{\link[base]{gsub}} iteratively
#' on \code{x} to replace all patterns in \code{from} with replacements in \code{to}.
#'
#' @param x [\code{atomic}]\cr
#' Atomic vector. If \code{x} is a factor, all replacements work on the levels.
#' @param from [\code{atomic}]\cr
#' Atomic vector with values to replace, same length as \code{to}.
#' @param to [\code{atomic}]\cr
#' Atomic vector with replacements, same length as \code{from}.
#' @param regex [\code{logical}]\cr
#' Use regular expression matching? Default is \code{FALSE}.
#' @param ignore.case [\code{logical}]\cr
#' Argument passed to \code{\link[base]{gsub}}.
#' @param perl [\code{logical}]\cr
#' Argument passed to \code{\link[base]{gsub}}.
#' @param fixed [\code{logical}]\cr
#' Argument passed to \code{\link[base]{gsub}}.
#' @return [\code{atomic}].
#' @export
#' @examples
#' # replace integers
#' x = 1:5
#' mapValues(x, c(2, 3), c(99, 100))
#'
#' # replace factor levels using regex matching
#' x = factor(c("aab", "aba", "baa"))
#' mapValues(x, "a.a", "zzz", regex = TRUE)
mapValues = function(x, from, to, regex = FALSE, ignore.case = FALSE, perl = FALSE, fixed = FALSE) {
  assertAtomic(x)
  assertAtomic(from)
  assertAtomic(to, len = length(from))
  assertFlag(regex)

  map = function(x, from, to) {
    if (regex) {
      for (i in seq_along(from))
        x = gsub(from[i], to[i], x, ignore.case = ignore.case, perl = perl, fixed = fixed)
    } else {
      m = match(x, from, nomatch = NA_integer_)
      found = !is.na(m)
      x[found] = to[m[found]]
    }
    return(x)
  }

  if (is.factor(x)) {
    levels(x) = map(levels(x), from, to)
    return(x)
  }
  return(map(x, from, to))
}

context("mapValues")

test_that("mapValues", {
  expect_equal(mapValues(1:3, 2, 3), c(1, 3, 3))
  expect_equal(mapValues(letters[1:5], letters[1:5], rev(letters[1:5])), rev(letters[1:5]))
  expect_equal(mapValues(factor(c("a", "b", "c")), "b", "zzz"), factor(c("a", "zzz", "c"), levels = c("a", "zzz", "c")))
  expect_equal(mapValues(c("aab", "aba", "baa"), "aa", "zz", regex = TRUE), c("zzb", "aba", "bzz"))
  expect_equal(mapValues(c("aab", "aba", "baa"), "^aa.+", "zz", regex = TRUE), c("zz", "aba", "baa"))

  expect_error(mapValues(iris, 1, 1), "atomic")
  expect_error(mapValues(1:10, 1:2, 1), "length")
  expect_error(mapValues(1:10, 1, 1:2), "length")
})

#' @title Parses \code{...} arguments to a named list
#'
#' @description
#' The deparsed name will be used for arguments with missing names.
#' Missing names will be set to \code{NA}.
#'
#' @param ...
#'   Arbitrary number of objects.
#' @return [\code{list}]: Named list with objects.
#' @export
#' @examples
#' z = 3
#' argsAsNamedList(x = 1, y = 2, z)
argsAsNamedList = function(...) {
  args = list(...)
  ns = names2(args)
  ns.missing = is.na(ns)
  if (any(ns.missing)) {
    ns.sub = as.character(substitute(deparse(...)))[-1L]
    ns[ns.missing] = ns.sub[ns.missing]
  }
  setNames(args, replace(ns, ns %in% c("NA", "NULL", ""), NA_character_))
}

#' @title Filter a list for NULL values
#'
#' @description
#' Removes all NULL elements from a list.
#'
#' @param li [\code{list}]\cr
#'  List.
#' @return [\code{list}].
#' @export
filterNull = function(li) {
  assertList(li)
  li[!vlapply(li, is.null)]
}

context("filterNull")

test_that("filterNull", {
  li = list(1, 2, NULL, 3)
  expect_equal(filterNull(li), list(1, 2, 3))
  expect_equal(filterNull(list()), list())
  expect_error(filterNull(iris))
})

#' @title Calculate range statistic
#'
#' @description
#' A simple wrapper for \code{diff(range(x))}, so \code{max(x) - min(x)}.
#'
#' @param x [\code{numeric}]\cr
#'   The vector.
#' @param na.rm [\code{logical(1)}]\cr
#'   If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
#'   If \code{TRUE}, NAs are disregarded.
#'   Default is \code{FALSE}
#' @return [\code{numeric(1)}].
#' @export
rangeVal = function(x, na.rm = FALSE) {
  assertNumeric(x, min.len = 1L, any.missing = TRUE)
  assertFlag(na.rm)
  if (allMissing(x))
    return(NA_real_)
  diff(range(x, na.rm = na.rm))
}

context("rangeVal")

test_that("rangeVal", {
  expect_equal(rangeVal(c(1, 5)), 4)
  expect_equal(rangeVal(1), 0)
  expect_equal(rangeVal(1:3), 2)
  
  # NAs
  expect_equal(rangeVal(c(1, 2, NA)), NA_real_)
  expect_equal(rangeVal(c(1, 2, NA), na.rm = TRUE), 1)
  expect_equal(rangeVal(c(NA_real_), na.rm = TRUE), NA_real_)
})

#' @title Create a progress bar with estimated time
#'
#' @description
#' Create a progress bar function that displays the estimated time till
#' completion and optional messages. Call the returned functions \code{set} or
#' \code{inc} during a loop to change the display.
#' Note that you are not allowed to decrease the value of the bar.
#' If you call these function without setting any of the arguments
#' the bar is simply redrawn with the current value.
#' For errorhandling use \code{error} and have a look at the example below.
#'
#' You can globally change the behavior of all bars by setting the option
#' \code{options(BBmisc.ProgressBar.style)} either to \dQuote{text} (the default)
#' or \dQuote{off}, which display no bars at all.
#'
#' You can globally change the width of all bars by setting the option
#' \code{options(BBmisc.ProgressBar.width)}. By default this is \code{getOption("width")}.
#'
#' You can globally set the stream where the output of the bar is directed by setting the option
#' \code{options(BBmisc.ProgressBar.stream)} either to \dQuote{stderr} (the default)
#' or \dQuote{stdout}. Note that using the latter will result in the bar being shown in
#' reports generated by Sweave or knitr, what you probably do not want.
#'
#' @param min [\code{numeric(1)}]\cr
#'   Minimum value, default is 0.
#' @param max [\code{numeric(1)}]\cr
#'   Maximum value, default is 100.
#' @param label [\code{character(1)}]\cr
#'   Label shown in front of the progress bar.
#'   Note that if you later set \code{msg} in the progress bar function,
#'   the message will be left-padded to the length of this label, therefore
#'   it should be at least as long as the longest message you want to display.
#'   Default is \dQuote{}.
#' @param char [\code{character(1)}]\cr
#'   A single character used to display progress in the bar.
#'   Default is \sQuote{+}.
#' @param style [\code{character(1)}]\cr
#'   Style of the progress bar. Default is set via options (see details).
#' @param width [\code{integer(1)}]\cr
#'   Width of the progress bar. Default is set via options (see details).
#' @param stream [\code{character(1)}]\cr
#'   Stream to use. Default is set via options (see details).
#' @return [\code{\link{ProgressBar}}]. A list with following functions:
#'   \item{set [function(value, msg = label)]}{Set the bar to a value and possibly display a message instead of the label.}
#'   \item{inc [function(value, msg = label)]}{Increase the bar and possibly display a message instead of the label.}
#'   \item{kill [function(clear = FALSE)]}{Kill the bar so it cannot be used anymore. Cursor is moved to new line. You can also erase its display.}
#'   \item{error [function(e)]}{Useful in \code{tryCatch} to properly display error messages below the bar. See the example.}
#' @export
#' @aliases ProgressBar
#' @examples
#' bar = makeProgressBar(max = 5, label = "test-bar")
#' for (i in 0:5) {
#'   bar$set(i)
#'   Sys.sleep(0.2)
#' }
#' bar = makeProgressBar(max = 5, label = "test-bar")
#' for (i in 1:5) {
#'   bar$inc(1)
#'   Sys.sleep(0.2)
#' }
#' # display errors properly (in next line)
#' \dontrun{
#' f = function(i) if (i>2) stop("foo")
#' bar = makeProgressBar(max = 5, label = "test-bar")
#' for (i in 1:5) {
#'   tryCatch ({
#'     f(i)
#'     bar$set(i)
#'   }, error = bar$error)
#' }
#' }
makeProgressBar = function(min = 0, max = 100, label = "", char = "+",
  style = getOption("BBmisc.ProgressBar.style", "text"),
  width = getOption("BBmisc.ProgressBar.width", getOption("width")),
  stream = getOption("BBmisc.ProgressBar.stream", "stderr")) {
  assertNumber(min)
  assertNumber(max)
  assertString(label)
  assertChoice(style, c("text", "off"))
  assertInt(width, lower = 30L)
  assertChoice(stream, c("stderr", "stdout"))

  if (style == "off")
    return(structure(list(
      set = function(value, msg = label) invisible(NULL),
      inc = function(inc, msg = label) invisible(NULL),
      kill =  function(clear = FALSE) invisible(NULL),
      error = function(e) stop(e)
    ), class = "ProgressBar"))

  mycat = if (stream == "stdout")
    function(...) cat(...)
  else
    function(...) cat(..., file = stderr())

  ## label |................................| xxx% (hh:mm:ss)
  label.width = nchar(label)
  bar.width = width - label.width - 21L
  bar = rep(" ", bar.width)

  start.time = as.integer(Sys.time())
  delta = max - min
  kill.line = "\r"
  killed = FALSE
  cur.value = min
  draw = function(value, inc, msg) {
    if (!missing(value) && !missing(inc))
      stop("You must not set value and inc!")
    else if (!missing(value))
      assertNumber(value, lower = max(min, cur.value), upper = max)
    else if (!missing(inc)) {
      assertNumber(inc, lower = 0, upper = max - cur.value)
      value = cur.value + inc
    } else {
      value = cur.value
    }
    if (!killed)  {
      # special case for min == max, weird "empty" bar, but might happen...
      if (value == max)
        rate = 1
      else
        rate = (value - min) / delta
      bin = round(rate * bar.width)
      bar[seq(bin)] <<- char
      delta.time = as.integer(Sys.time()) - start.time
      if (value == min)
        rest.time = 0
      else
        rest.time = (max - value) * (delta.time / (value - min))
      rest.time = splitTime(rest.time, "hours")
      # as a precaution, so we can _always_ print in the progress bar cat
      if (rest.time["hours"] > 99)
        rest.time[] = 99
      mycat(kill.line)
      msg = sprintf(sprintf("%%%is", label.width), msg)
      mycat(sprintf("%s |%s| %3i%% (%02i:%02i:%02i)", msg, collapse(bar, sep = ""), round(rate*100),
           rest.time["hours"], rest.time["minutes"], rest.time["seconds"]))
      if (value == max)
        kill()
      flush.console()
    }
    cur.value <<- value
  }
  clear = function(newline = TRUE) {
    mycat(kill.line)
    mycat(rep(" ", width))
    if (newline)
      mycat("\n")
  }
  kill = function(clear = FALSE) {
    if (clear)
      clear(newline = TRUE)
    else
      mycat("\n")
    killed <<- TRUE
  }
  makeS3Obj("ProgressBar",
    set = function(value, msg = label) draw(value = value, msg = msg),
    inc = function(inc, msg = label) draw(inc = inc, msg = msg),
    kill = kill,
    error = function(e) {
      kill(clear = FALSE)
      stop(e)
    }
  )
}

context("makeProgressBar")

test_that("makeProgressBar", {
  cat("\n")
  bar = makeProgressBar()
  expect_class(bar, "ProgressBar")
  for(i in 0:100) {
    bar$set(i)
    Sys.sleep(0.01)
  }
  bar = makeProgressBar(min=10, max=50, label="foo")
  for(i in 11:50) {
    bar$set(i)
    Sys.sleep(0.01)
  }
  bar = makeProgressBar(min=0.1, max=0.2)
  for(i in seq(0.1, 0.2, length.out=5)) {
    bar$set(i)
    Sys.sleep(0.1)
  }
  bar$set(0.2)
  bar$set(0.2)
  bar = makeProgressBar(max=10^6, label="          ")
  for(i in 10^seq(1:6)) {
    bar$set(i, msg=sprintf("%i", i))
    Sys.sleep(0.1)
  }

  bar = makeProgressBar(min=0, max=0)
  bar$set(0)
  bar = makeProgressBar(min=0, max=0)
  bar$inc(0)
})

test_that("makeProgressBar global options", {
  old.style = getOption("BBmisc.ProgressBar.style")
  old.width = getOption("BBmisc.ProgressBar.width")
  options(BBmisc.ProgressBar.style = "off")
  cat("\n")
  bar = makeProgressBar(max=5)
  expect_class(bar, "ProgressBar")
  for(i in 0:5) {
     expect_silent(bar$set(i))
  }
  options(BBmisc.ProgressBar.style = "text", BBmisc.ProgressBar.width = 30)
  cat("\n")
  bar = makeProgressBar(max=5)
  for(i in 0:5) {
     expect_output(print(bar$set(i)))
  }
  options(BBmisc.ProgressBar.style = old.style, BBmisc.ProgressBar.width = old.width)
})

#' @title Wrapper for cat and sprintf
#'
#' @description
#' A simple wrapper for \code{cat(sprintf(...))}.
#'
#' @param ... [any]\cr
#'   See \code{\link{sprintf}}.
#' @param file [\code{character(1)}]\cr
#'   See \code{\link{cat}}.
#'   Default is \dQuote{}.
#' @param append [\code{logical(1)}]\cr
#'   See \code{\link{cat}}.
#'   Default is \code{FALSE}.
#' @param newline [\code{logical(1)}]\cr
#'   Append newline at the end?
#'   Default is \code{TRUE}.
#' @return Nothing.
#' @export
#' @examples
#' msg = "a message."
#' catf("This is %s", msg)
catf = function(..., file = "", append = FALSE, newline = TRUE) {
  cat(sprintf(...), ifelse(newline, "\n", ""), sep = "", file = file, append = append)
}

context("printToChar")

test_that("printToChar", {
  if (!interactive()) {
  z = list()
  class(z) = "foo"
  print.foo <<- function(x, ...) catf("bar")
  s = printToChar(z)
  expect_equal(s, "bar")
  print.foo <<- function(x, ...) catf("bar\nblubb")
  s = printToChar(z)
  expect_equal(s, "bar\nblubb")
  }
})

#' @title A wrapper for \code{attr(x, which) = y}
#'
#' @description
#' Sets an attribute on an object and returns the modified object.
#'
#' @param x [any]\cr
#'  Your object.
#' @param which [\code{character(1)}]\cr
#'  Name of the attribute to set
#' @param value [\code{ANY}]\cr
#'  Value for the attribute.
#' @return Changed object \code{x}.
#' @export
#' @examples
#' setAttribute(list(), "foo", 1)
setAttribute = function(x, which, value) {
  attr(x, which) = value
  x
}

context("setAttribute")

test_that("setAttribute", {
  x = 1:9
  x = setAttribute(x, "foo", "bar")
  x = setAttribute(x, "dim", c(3,3))
  expect_equal(attr(x, "foo"), "bar")
  expect_equal(nrow(x), 3)
  expect_equal(ncol(x), 3)
})

#' @title Converts storage type of a matrix
#'
#' @description
#' Works by setting \code{\link{mode}}.
#'
#' @param x [\code{matrix}]\cr.
#'   Matrix to convert.
#' @param type [\code{character(1)}]\cr
#'   New storage type.
#' @return [\code{matrix}].
#' @note \code{as.mytype} drops dimension when used on a matrix.
#' @export
convertMatrixType = function(x, type) {
  assertMatrix(x)
  assertChoice(type, c("integer", "numeric", "complex", "character", "logical"))
  storage.mode(x) = type
  return(x)
}

#' @title Prints object to a string / character vector
#'
#' @description
#' Captures the output of print() as a character string.
#'
#' @param x [any]\cr
#'   Object to print
#' @param collapse [\code{character(1)}]\cr
#'   Used to collapse multiple lines.
#'   \code{NULL} means no collapsing, vector is returned.
#'   Default is \dQuote{\\n}.
#' @return [\code{character}].
#' @export
#' @examples
#' x = data.frame(a = 1:2, b = 3:4)
#' str(printToChar(x))
printToChar = function(x, collapse = "\n") {
  rval = NULL
  con = textConnection("rval", "w", local = TRUE)
  sink(con)
  on.exit({
    sink()
    close(con)
  })
  print(x)
  if (!is.null(collapse))
    paste(rval, collapse = collapse)
  else
    rval
}

#' @title Check that a list contains only elements of a required type
#'
#' @description
#' Check that argument is a list and contains only elements of a required type.
#' Throws exception if check is not passed.
#' Note that argument is evaluated when checked.
#'
#' @param xs [\code{list}]\cr
#'   Argument.
#' @param cl [\code{character(1)}]\cr
#'   Class that elements must have. Checked with \code{is}.
#' @return Nothing.
#' @export
#' @examples
#' xs = as.list(1:3)
#' checkListElementClass(xs, "numeric")
checkListElementClass = function(xs, cl) {
  assertList(xs)
  s = deparse(substitute(xs))
  lapply(seq_along(xs), function(i) {
    x = xs[[i]]
    if(!(is(x, cl)))
      stop("List ", s, " has element of wrong type ", class(x)[1L], " at position ", i, ". Should be: ", cl)
  })
  invisible(NULL)
}

context("checkListElementClass")

test_that("checkListElementClass", {
  checkListElementClass(list(1, 5), cl="numeric")
  expect_error(checkListElementClass(list(1, "a"), cl="numeric"), "numeric")
  
  xs = list("a", "b")
  checkListElementClass(xs, "character")
  expect_error(checkListElementClass(xs, "integer"), "character")
})

#' @title Can some strings be used for column or list element names without problems?
#'
#' @description
#' Checks if strings are valid R names that won't cause problems as column names.
#'
#' @param x [\code{character}]\cr
#'   Character vector to check.
#' @param unique [\code{logical(1)}]\cr
#'   Should the names be unique?
#'   Default is \code{TRUE}.
#' @return [\code{logical}]. One Boolean entry for each string in \code{x}.
#'   If the entries are not unique and \code{unique} is enabled, the first duplicate will
#'   be \code{FALSE}.
#' @export
isValidName = function(x, unique = TRUE) {
  if (!is.character(x))
    x = as.character(x)
  # check that make.names does not change the string (otherwise it would be invalid),
  # names are unique (for e.g. colnames) and stuff like ..1 is disallowed
  x == make.names(x, isTRUE(unique)) & !grepl("^\\.\\.[0-9]$", x)
}

context("isValidName")

test_that("isValidName", {
  expect_true(isValidName("a"))
  expect_true(all(isValidName(c("a", "b"))))
  expect_equal(isValidName(c("a", "a")), c(TRUE, FALSE))
  expect_true(all(isValidName(c("a", "a"), unique=FALSE)))
  expect_equal(isValidName(c("x", "..1")), c(TRUE, FALSE))
})

#FIXME: remove

#' @title Deprecated function. Do not use!
#'
#' @description
#' These functions are deprecated and will be removed in future versions.
#'
#' @param df No text
#' @param chars.as.factor No text
#' @param factors.as.char No text
#' @param ints.as.num No text
#' @param logicals.as.factor No text
#' @param x No text
#' @param num.format No text
#' @param clip.len No text
#'
#' @name deprecated
#' @rdname deprecated
NULL

#' @export
#' @rdname deprecated
convertDfCols = convertDataFrameCols

#' @export
#' @rdname deprecated
listToShortString = convertToShortString

#' @title Blow up single scalars / objects to vectors /  list by replication
#'
#' @description
#' Useful for standard argument conversion where a user can input a single
#' element, but this has to be replicated now n times for a resulting vector or list.
#'
#' @param x [any]\cr
#'   Input element.
#' @param n [\code{integer(1)}]\cr
#'   Desired length.
#'   Default is 1 (the most common case).
#' @param cl [\code{character}*]\cr
#'   Only do the operation if \code{x} inherits from this one of these classes,
#'   otherwise simply let \code{x} pass.
#'   Default is \code{NULL} which means to always do the operation.
#' @param names [\code{character}*] \cr
#'   Names for result.
#'   Default is \code{NULL}, which means no names.
#' @param ensure.list [\code{logical(1)}]\cr
#'   Should \code{x} be wrapped in a list in any case?
#'   Default is \code{FALSE}, i.e., if \code{x} is a scalar value, a vector is
#'   returned.
#' @return Ether a vector or list of length \code{n} with replicated \code{x} or \code{x} unchanged..
#' @export
ensureVector = function(x, n = 1L, cl = NULL, names = NULL, ensure.list = FALSE) {
  n = convertInteger(n)
  assertCount(n)
  assertFlag(ensure.list)

  doit = isScalarValue(x) || !is.atomic(x)
  if (!is.null(cl)) {
    assertCharacter(cl, min.len = 1L, any.missing = FALSE, all.missing = FALSE)
    doit = doit && inherits(x, cl)
  }

  if (doit) {
    if (isScalarValue(x) && !ensure.list) {
      xs = rep(x, n)
    } else {
      xs = replicate(n, x, simplify = FALSE)
    }

    if (!is.null(names)) {
      assertCharacter(names, len = n, any.missing = FALSE)
      names(xs) = names
    }
    return(xs)
  } else {
    return(x)
  }
}

context("ensureVector")

test_that("ensureVector", {
  # default is n = 1L
  expect_equal(ensureVector("a"), "a")
  expect_equal(ensureVector("a", n = 2L), c("a", "a"))
  expect_equal(ensureVector("a", n = 2L, cl = "integer"),  "a")
  expect_equal(ensureVector(1, n = 1), c(1))

  expect_equal(ensureVector(c("a", "b"), n = 10L), c("a", "b"))

  expect_equal(ensureVector(iris, n = 1L), list(iris))
  expect_equal(ensureVector(iris, n = 2L, cl = "matrix"), iris)
  expect_equal(ensureVector(iris, n = 2L, cl = "data.frame"), list(iris, iris))
  expect_equal(ensureVector(iris, n = 2L), list(iris, iris))
  expect_equal(ensureVector(iris, n = 2L, names = c("a", "b")), list(a = iris, b = iris))

  # check ensure.list argument
  expect_equal(ensureVector("a", ensure.list = TRUE), list("a"))
  expect_equal(ensureVector(3, n = 3L, ensure.list = TRUE, names = letters[1:3]), list(a = 3, b = 3, c = 3))
})

#' @title Converts a string into a quoted expression
#'
#' @description
#' Works the same as if you would have entered the expression and called
#' \code{\link{quote}} on it.
#'
#' @param s [\code{character(1)}]\cr
#'   Expression as string.
#' @param env [\code{numeric(1)}]\cr
#'   Environment for expression.
#'   Default is \code{parent.frame()}
#' @return Quoted expression.
#' @export
#' @examples
#' asQuoted("x == 3")
asQuoted = function(s, env = parent.frame()) {
  assertString(s)
  structure(parse(text = s)[1L], env = env, class = "quoted")[[1L]]
}

#' @title Split up a string into substrings
#'
#' @description
#' Split up a string into substrings according to a seperator.
#'
#' @param x [\code{character}]\cr
#'   Source string.
#' @param sep [\code{character}]\cr
#'   Seperator whcih is used to split \code{x} into substrings.
#'   Default is \dQuote{ }.
#' @return [\code{vector}]
#'   Vector of substrings.
#' @export
#' @examples
#' explode("foo bar")
#' explode("comma,seperated,values", sep = ",")
explode = function(x, sep = " ") {
  assertString(x)
  assertString(sep)
  #FIXME: why perl?
  x.exploded = strsplit(x, sep, perl = TRUE)
  return(x.exploded[[1L]])
}

#' @title Converts columns in a data frame to characters, factors or numerics
#'
#' @description
#' Converts columns of a data frame based on specified conversion flags.
#'
#' @param df [\code{data.frame}]\cr
#'   Data frame.
#' @param chars.as.factor [\code{logical(1)}]\cr
#'   Should characters be converted to factors?
#'   Default is \code{FALSE}.
#' @param factors.as.char [\code{logical(1)}]\cr
#'   Should characters be converted to factors?
#'   Default is \code{FALSE}.
#' @param ints.as.num [\code{logical(1)}]\cr
#'   Should integers be converted to numerics?
#'   Default is \code{FALSE}.
#' @param logicals.as.factor [\code{logical(1)}]\cr
#'   Should logicals be converted to factors?
#'   Default is \code{FALSE}.
#' @export
#' @return [\code{data.frame}].
convertDataFrameCols = function(df, chars.as.factor = FALSE, factors.as.char = FALSE, ints.as.num = FALSE, logicals.as.factor = FALSE) {
  assertDataFrame(df)
  assertFlag(chars.as.factor)
  assertFlag(factors.as.char)
  assertFlag(ints.as.num)
  assertFlag(logicals.as.factor)
  df = x = as.list(df)

  if (chars.as.factor) {
    i = vlapply(df, is.character)
    if (any(i))
      x[i] = lapply(x[i], factor)
  }

  if (factors.as.char) {
    i = vlapply(df, is.factor)
    if (any(i))
      x[i] = lapply(x[i], as.character)
  }

  if (ints.as.num) {
    i = vlapply(df, is.integer)
    if (any(i))
      x[i] = lapply(x[i], as.double)
  }

  if (logicals.as.factor) {
    i = vlapply(df, is.logical)
    if (any(i))
      x[i] = lapply(x[i], factor, levels = c("TRUE", "FALSE"))
  }

  as.data.frame(x, stringsAsFactors = FALSE)
}

context("convertDataFrameCols")

test_that("convertDataFrameCols", {
  df = data.frame(x=1:2, y=c("a", "b"), z=factor(c("x", "y")), stringsAsFactors=FALSE)
  df2 = convertDataFrameCols(df, chars.as.factor=TRUE)
  expect_true(is.numeric(df2$x))
  expect_true(is.factor(df2$y))
  expect_true(is.factor(df2$z))
  expect_equal(df$x, df2$x)
  expect_equal(df$y, as.character(df2$y))
  expect_equal(df$z, df2$z)

  df2 = convertDataFrameCols(df, factors.as.char=TRUE)
  expect_true(is.numeric(df2$x))
  expect_true(is.character(df2$y))
  expect_true(is.character(df2$z))
  expect_equal(df$x, df2$x)
  expect_equal(df$y, df2$y)
  expect_equal(as.character(df$z), df2$z)
  
  df2 = convertDataFrameCols(df, ints.as.num=TRUE)
  expect_true(is.double(df2$x))
  expect_true(is.character(df2$y))
  expect_true(is.factor(df2$z))
  expect_equal(df$x, df2$x)
  expect_equal(df$y, df2$y)
  expect_equal(df$z, df2$z)

  df2 = convertDataFrameCols(df, chars.as.factor=TRUE, factors.as.char=TRUE, ints.as.num=TRUE)
  expect_true(is.double(df2$x))
  expect_true(is.factor(df2$y))
  expect_true(is.character(df2$z))
})

#' @title Call \code{lapply} on an object and return a data.frame
#'
#' @description
#' Applies a function \code{fun} on each element of input \code{x}
#' and combines the results as \code{data.frame} columns.
#' The results will get replicated to have equal length
#' if necessary and possible.
#'
#' @param x [\code{data.frame}]\cr
#'   Data frame.
#' @param fun [\code{function}]\cr
#'   The function to apply.
#' @param ... [any]\cr
#'   Further arguments passed down to \code{fun}.
#' @param col.names [\code{character(1)}]\cr
#'   Column names for result.
#'   Default are the names of \code{x}.
#' @export
#' @return [\code{data.frame}].
dapply = function(x, fun, ..., col.names) {
  assertFunction(fun)

  x = lapply(x, fun, ...)

  if (missing(col.names)) {
    ns = names2(x)
    missing = which(is.na(ns))
    if (length(missing))
      names(x) = replace(ns, missing, paste0("Var.", missing))
  } else {
    assertCharacter(col.names, len = length(x), any.missing = FALSE)
    names(x) = col.names
  }

  n = unique(viapply(x, length))
  if (length(n) > 1L) {
    max.n = max(n)
    if (any(max.n %% n))
      stop("Arguments imply differing number of rows: ", collapse(n, ", "))
    x = lapply(x, rep_len, length.out = max.n)
    n = max.n
  }
  attr(x, "row.names") = seq_len(n)
  attr(x, "class") = "data.frame"
  return(x)
}

context("dapply")

test_that("dapply", {
  d = dapply(1:3, function(x) rep(x, 2))
  expect_is(d, "data.frame")
  expect_equal(dim(d), c(2L, 3L))
  expect_equal(colnames(d), c("Var.1", "Var.2", "Var.3"))

  d = dapply(1:2, function(x) x, col.names=c("a", "b"))
  expect_is(d, "data.frame")
  expect_equal(dim(d), c(1L, 2L))
  expect_equal(colnames(d), c("a", "b"))

  d1 = dapply(iris, computeMode)
  d2 = dapply(iris, computeMode, col.names=letters[1:5])
  expect_equal(dim(d1), c(1L, 5L))
  expect_equal(dim(d2), c(1L, 5L))
  expect_equal(names(d1), names(iris))
  expect_equal(names(d2), letters[1:5])
})

#' @title Capitalize strings in a vector
#'
#' @description
#' Capitalise first word or all words of a character vector.
#' Lower back of vector element or word, respectively.
#' 
#' @param x [\code{character(n)}]\cr
#'   Vector of character elements to capitalize.
#' @param all.words [\code{logical(1)}]\cr
#'   If \code{TRUE} all words of each vector element are capitalized.
#'   \code{FALSE} capitalizes the first word of each vector element.
#' @param lower.back [\code{logical(1)}]\cr
#'   \code{TRUE} lowers the back of each word or vector element (depends on \code{all.words}).
#' @return Capitalized vector: [\code{character(n)}].
#' @export
#' @examples
#' capitalizeStrings(c("the taIl", "wags The dOg", "That looks fuNny!"))
#' capitalizeStrings(c("the taIl", "wags The dOg", "That looks fuNny!")
#' , all.words = TRUE, lower.back = TRUE)
capitalizeStrings = function(x, all.words = FALSE, lower.back = FALSE) {
  assertCharacter(x)
  assertLogical(all.words, any.missing = FALSE, len = 1L)
  assertLogical(lower.back, any.missing = FALSE, len = 1L)
  
  if (all.words) {
    pattern = "([[:alnum:]])([[:alnum:]]*)"
    replacement = "\\U\\1"
    if (lower.back) {
      replacement = paste0(replacement, "\\L\\2")
    } else {
      replacement = paste0(replacement, "\\E\\2")
    }
  } else {
    pattern = "^([[:alnum:]])"
    replacement = "\\U\\1"
    if (lower.back) {
      pattern = paste0(pattern, "(.*)")
      replacement = paste0(replacement, "\\L\\2")
    }
  }
  
  return(gsub(pattern, replacement, x, perl = TRUE))
}

context("capitalizeStrings")

test_that("capitalizeStrings", {
  testString = c("the taIl", "wags The dOg", "That Looks fuNny!")
  
  expect_equal(capitalizeStrings(testString)
               , c("The taIl", "Wags The dOg", "That Looks fuNny!"))
  expect_equal(capitalizeStrings(testString, all.words = TRUE)
               , c("The TaIl", "Wags The DOg", "That Looks FuNny!"))
  expect_equal(capitalizeStrings(testString, all.words = TRUE, lower.back = TRUE)
               , c("The Tail", "Wags The Dog", "That Looks Funny!"))
  expect_equal(capitalizeStrings(testString, lower.back = TRUE)
               , c("The tail", "Wags the dog", "That looks funny!"))
})

#' @title Calculates symmetric set difference between two sets
#'
#' @description
#' Returns elements that are in either set but not in both.
#'
#' @param x [\code{vector}]\cr
#'   Set 1.
#' @param y [\code{vector}]\cr
#'   Set 2.
#' @return [\code{vector}].
#' @export
symdiff = function(x, y) {
  setdiff(union(x, y), intersect(x, y))
}

context("symdiff")

test_that("symdiff", {
  expect_equal(symdiff(c(1, 2), 1), 2)
  expect_equal(symdiff(c(1, 2), numeric(0)), c(1, 2))
  expect_equal(symdiff("a", "b"), c("a", "b"))
  expect_equal(symdiff("a", "a"), character(0))
})

#' @title Wrapper for \code{rownames(x) = y}, \code{colnames(x) = y}
#'
#' @description
#' Sets row or column names on a matrix or data frame and returns the modified object.
#'
#' @param x [\code{matrix} | \code{data.frame}]\cr
#'   Matrix or data.frame.
#' @param names [\code{character}]\cr
#'   New names for rows / columns.  
#' @return Changed object \code{x}.
#' @export
#' @examples
#' setColNames(matrix(1:4, 2, 2), c("a", "b"))
setRowNames = function(x, names) {
  rownames(x) = names
  x
}

#' @rdname setRowNames
#' @export
setColNames = function(x, names) {
  colnames(x) = names
  x
}

context("setRowColNames")

test_that("setRowColNames", {
  x = y = matrix(1:4, 2, 2)
  rownames(y) = c("a", "b")
  expect_equal(setRowNames(x, c("a", "b")), y)    
  colnames(y) = c("c", "d")
  expect_equal(setColNames(setRowNames(x, c("a", "b")), c("c", "d")), y)    
})

context("sortByCol")

test_that("sortByCol", {
  d1 = setRowNames(data.frame(x = c(2, 3, 1), y = c("a", "c", "b")), c("1", "2", "3"))

  d2 = sortByCol(d1, "x")
  d3 = setRowNames(data.frame(x = c(1, 2, 3), y = c("b", "a", "c")), c(3, 1, 2))
  expect_equal(d2, d3)

  d2 = sortByCol(d1, "x", asc = FALSE)
  d3 = setRowNames(data.frame(x = c(3, 2, 1), y = c("c", "a", "b")), c(2, 1, 3))
  expect_equal(d2, d3)

  d2 = sortByCol(d1, c("x", "y"))
  d3 = setRowNames(data.frame(x = c(1, 2, 3), y = c("b", "a", "c")), c(3, 1, 2))
  expect_equal(d2, d3)

  d2 = sortByCol(d1, "y")
  d3 = setRowNames(data.frame(x = c(2, 1, 3), y = c("a", "b", "c")), c(1, 3, 2))
  expect_equal(d2, d3)

  # real tie breaker
  d1 = data.frame(x = c(2, 2, 1), y = c("a", "b", "c"))
  d2 = sortByCol(d1, c("x", "y"))
  d3 = data.frame(x = c(1, 2, 2), y = c("c", "a", "b"))
  expect_equal(d2, d3, check.attributes = FALSE)
  d2 = sortByCol(d1, c("x", "y"), asc = c(TRUE, FALSE))
  d3 = data.frame(x = c(1, 2, 2), y = c("c", "b", "a"))
  expect_equal(d2, d3, check.attributes = FALSE)

  # one col
  d1 = setRowNames(data.frame(x = c(1, 2)), c(1, 2))
  d2 = sortByCol(d1, "x", asc = FALSE)
  d3 = setRowNames(data.frame(x = c(2, 1)), c(2, 1))
  expect_equal(d2, d3)
})

context("convertListOfRowstoDataFrame")

test_that("convertListOfRowstoDataFrame", {
  df1 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")),
    strings.as.factors = FALSE)
  df2 = data.frame(x = 1:2, y = c("a", "b"), stringsAsFactors = FALSE)
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(c(x = "1", y = "a"), list(x = "2", y = "b")),
    strings.as.factors = FALSE)
  df2 = data.frame(x=c("1", "2"), y=c("a", "b"), stringsAsFactors = FALSE)
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(c("1", "a"), c("2", "b")), strings.as.factors = FALSE,
    col.names=c("x", "y"))
  df2 = data.frame(x=c("1", "2"), y=c("a", "b"), stringsAsFactors = FALSE)
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(list(a = 1, b = 1), list(b = 12)))
  df2 = convertListOfRowsToDataFrame(list(c(a = 1, b = 1), c(b = 12)))
  expect_equal(df1, df2)

  # names
  df1 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")),
    strings.as.factors = FALSE, row.names = c("r1", "r2"))
  df2 = setRowNames(data.frame(x = 1:2, y = c("a", "b"), stringsAsFactors = FALSE), c("r1", "r2"))
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")),
    strings.as.factors = FALSE, row.names = 1:2)
  df2 = setRowNames(data.frame(x = 1:2, y = c("a", "b"), stringsAsFactors = FALSE), 1:2)
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")),
    strings.as.factors = FALSE, col.names = c("c1", "c2"))
  df2 = data.frame(c1 = 1:2, c2 = c("a", "b"), stringsAsFactors = FALSE)
  expect_equal(df1, df2)

  df1 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")),
    strings.as.factors = FALSE, col.names = 1:2)
  df2 = setColNames(data.frame(1:2, c("a", "b"), stringsAsFactors = FALSE), 1:2)
  expect_equal(df1, df2)
})


test_that("convertListOfRowsToDataFrame works with missing stuff in the rows", {

  df1 = setColNames(data.frame(1:2, c("a", NA), stringsAsFactors = FALSE), 1:2)
  df2 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2)),
    strings.as.factors = FALSE, col.names = 1:2)
  df3 = convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = NULL)),
    strings.as.factors = FALSE, col.names = 1:2)
  expect_equal(df1, df2)
  expect_equal(df1, df3)

  df1 = setColNames(data.frame(c(1, NA), c(NA, NA), stringsAsFactors = FALSE), 1:2)
  df2 = convertListOfRowsToDataFrame(list(list(x = 1), list(y = NULL)),
    strings.as.factors = FALSE, col.names = 1:2)
  df3 = convertListOfRowsToDataFrame(list(list(x = 1, y = NULL), list(y = NULL)),
    strings.as.factors = FALSE, col.names = 1:2)
  expect_equal(df1, df2)
  expect_equal(df1, df3)
})

context("makeDataFrame")

test_that("makeDataFrame", {
  df1 = makeDataFrame(0, 0)
  df2 = data.frame()
  expect_equal(df1, df2)
  df1 = makeDataFrame(3, 0)
  df2 = data.frame(matrix(nrow = 3, ncol = 0))
  expect_equal(df1, df2)
  df1 = makeDataFrame(0, 2, "character")
  df2 = data.frame(setColNames(matrix("a", nrow = 0, ncol = 2), c("V1", "V2")), stringsAsFactors = FALSE)
  expect_equal(df1, df2)
  df1 = makeDataFrame(3, 1, "integer")
  df2 = data.frame(V1 = integer(3))
  expect_equal(df1, df2)
  df1 = makeDataFrame(3, 2, "integer")
  df2 = as.data.frame(matrix(0L, 3, 2))
  expect_equal(df1, df2)
  df1 = makeDataFrame(3, 2, init = "bb")
  df2 = as.data.frame(matrix("bb", 3, 2), stringsAsFactors = FALSE)
  expect_equal(df1, df2)
  df1 = makeDataFrame(3, 2, c("numeric", "integer"))
  df2 = data.frame(V1 = numeric(3), V2 = integer(3), stringsAsFactors = FALSE)
  expect_equal(df1, df2)

  # names
  df1 = makeDataFrame(1, 2, "integer", row.names = c("r1"), col.names = c("c1", "c2"))
  df2 = setRowNames(data.frame(c1 = 0, c2 = 0), "r1")
  expect_equal(df1, df2)
  df1 = makeDataFrame(1, 2, "integer", row.names = 1L, col.names = c("c1", "c2"))
  df2 = setRowNames(data.frame(c1 = 0, c2 = 0), 1L)
  expect_equal(df1, df2)
  df1 = makeDataFrame(1, 2, "integer", row.names = NULL, col.names = 1:2)
  df2 = setColNames(data.frame(c1 = 0, c2 = 0), 1:2)
  expect_equal(df1, df2)
})

#' @title Sort the rows of a data.frame according to one or more columns
#'
#' @description
#' Sorts a data frame by one or more columns in ascending or descending order.
#'
#' @param x [\code{data.frame}]\cr
#'   Data.frame to sort.
#' @param col [\code{character}]\cr
#'   One or more column names to sort \code{x} by.
#'   In order of preference.
#' @param asc [\code{logical}]\cr
#'   Sort ascending (or descending)?
#'   One value per entry of \code{col}.
#'   If a scalar logical is passed, it is replicated.
#'   Default is \code{TRUE}.
#' @return [\code{data.frame}].
#' @export
sortByCol = function(x, col, asc = TRUE) {
  assertDataFrame(x)
  assertSubset(col, colnames(x))
  m = length(col)
  assertLogical(asc, min.len = 1L, any.missing = FALSE)
  if (length(asc) == 1L)
    asc = rep(asc, m)

  asc = ifelse(asc, 1, -1)
  args = as.list(x[, col, drop = FALSE])
  # convert col to orderable numeric and multiply with factor
  args = Map(function(a, b) xtfrm(a) * b, args, asc)
  # now order the numerics and permute df
  o = do.call(order, args)
  return(x[o, , drop = FALSE])
}

#' @title Simple logger which outputs to a file
#'
#' @description
#' Creates a simple file logger closure to log to a file, including time stamps.
#' An optional buffer holds the last few log messages.
#'
#' @param logfile [\code{character(1)}]\cr
#'   File to log to.
#' @param touch [\code{logical(1)}]\cr
#'   Should the file be created before the first log message?
#'   Default is \code{FALSE}.
#' @param keep [\code{integer(1)}]\cr
#'   Number of log messages to keep in memory for quick access.
#'   Default is \code{10}.
#' @return [\code{\link{SimpleFileLogger}}]. A list with following functions:
#'   \item{log [function(msg)]}{Send log message.}
#'   \item{getMessages [function(n)]}{Get last \code{n} log messages.}
#'   \item{clear [function()]}{Resets logger and deletes log file.}
#'   \item{getSize [function()]}{Returns the number of logs written.}
#'   \item{getLogfile [function()]}{Returns the full file name logs are written to.}
#' @export
#' @aliases SimpleFileLogger
makeSimpleFileLogger = function(logfile, touch = FALSE, keep = 10L) {
  assertString(logfile)
  assertFlag(touch)
  keep = asCount(keep)
  assertDirectoryExists(dirname(logfile), "w")
  if (touch && !file.create(logfile))
    stopf("Could not create file '%s'", logfile)
  if (keep)
    buffer = circularBuffer("character", keep)
  n.lines = 0L

  makeS3Obj("SimpleFileLogger",
    log = function(msg) {
      if (keep)
        buffer$push(msg)
      if (!touch && n.lines == 0L && !file.create(logfile))
        stopf("Could not create file '%s'", logfile)
      catf("<%s> %s", as.character(Sys.time()), msg, file = logfile, append = TRUE, newline = TRUE)
      n.lines <<- n.lines + 1L
    },
    getMessages = function(n) {
      if (!keep || n > keep)
        return(sub("^<.+> (.*)", "\\1", tail(readLines(logfile), n)))
      buffer$get(n)
    },
    clear = function() {
      if (keep)
        buffer$clear()
      n.lines <<- 0L
      file.remove(logfile)
    },
    getSize = function() {
      n.lines
    },
    getLogfile = function() {
      logfile
    }
  )
}

circularBuffer = function(type, capacity) {
  st = vector(type, capacity)
  stored = 0L
  pos = 0L

  list(
    push = function(x) {
      pos <<- pos %% capacity + 1L
      stored <<- min(capacity, stored + 1L)
      st[[pos]] <<- x
    },
    get = function(n = 1L) {
      head(st[rev((seq_len(stored) + pos - 1L) %% stored + 1L)], n)
    },
    stored = function() {
      stored
    },
    clear = function() {
      st <<- vector(type, capacity)
      stored <<- 0
      pos <<- 0
    }
  )
}

context("makeSimpleFileLogger")

test_that("makeSimpleFileLogger", {
  fn = tempfile()
  logger = makeSimpleFileLogger(fn)
  expect_identical(class(logger), "SimpleFileLogger")

  expect_equal(logger$getSize(), 0)
  msg1 = "xxx111xxx"
  logger$log(msg1)
  expect_true(grepl(msg1, readLines(fn)))
  expect_identical(msg1, logger$getMessages(1))
  expect_equal(logger$getSize(), 1)

  expect_true(file.exists(fn))
  logger$clear()
  expect_false(file.exists(fn))
})

test_that("message order",  {
  fn = tempfile()
  msg1 = "xxx111xxx"
  msg2 = "xxx222xxx"
  for (keep in c(0, 10)) {
    logger = makeSimpleFileLogger(fn)
    logger$log(msg1)
    logger$log(msg2)
    expect_identical(grepl("xxx[0-9]+xxx$", readLines(fn)), c(TRUE, TRUE))
    expect_identical(grepl("^xxx[0-9]+xxx$", logger$getMessages(2)), c(TRUE, TRUE))
    expect_identical(logger$getMessages(1), msg2)
    expect_identical(logger$getMessages(2), c(msg2, msg1))
  }
})

#' @title Execute a function call similar to \code{do.call}
#'
#' @description
#' This function is supposed to be a replacement for \code{\link[base]{do.call}} in situations
#' where you need to pass big R objects.
#' Unlike \code{\link[base]{do.call}}, this function allows to pass objects via \code{...}
#' to avoid a copy.
#'
#' @param fun [\code{character(1)}]\cr
#'   Name of the function to call.
#' @param ... [any]\cr
#'   Arguments to \code{fun}. Best practice is to specify them in a \code{key = value} syntax.
#' @param .args [\code{list}]\cr
#'   Arguments to \code{fun} as a (named) list. Will be passed after arguments in \code{...}.
#'   Default is \code{list()}.
#' @return Return value of \code{fun}.
#' @export
#' @examples \dontrun{
#'   library(microbenchmark)
#'   x = 1:1e7
#'   microbenchmark(do.call(head, list(x, n = 1)), do.call2("head", x, n = 1))
#' }
do.call2 = function(fun, ..., .args = list()) {
  assertString(fun)
  ddd = match.call(expand.dots = FALSE)$...
  expr = as.call(c(list(as.name(fun)), ddd, lapply(substitute(.args)[-1L], identity)))
  eval.parent(expr, n = 1L)
}

context("do.call2")

test_that("do.call2", {
  f = function(...) list(...)
  expect_equal(do.call2("f", a=1, b=2), list(a=1, b=2))
  expect_equal(do.call2("f", .args=list(a=1, b=2)), list(a=1, b=2))
  expect_equal(do.call2("f", a=1, .args=list(b=2)), list(a=1, b=2))

  df = iris
  expect_equal(do.call2("f", df), list(df))
  expect_equal(do.call2("f", .args = list(df)), list(df))

  f = function(x, data) data[[x]]
  expect_equal(do.call2("f", "Species", data=iris), iris$Species)
  expect_equal(do.call2("f", "Species", iris), iris$Species)
  expect_equal(do.call2("f", data = iris, "Species"), iris$Species)
  expect_equal(do.call2("f", "Species", .args = list(data = iris)), iris$Species)
  expect_equal(do.call2("f", data = iris, .args = list(x = "Species")), iris$Species)

  expect_error(do.call2(mean, 1:10), "string")
})

#' @title Chunk elements of vectors into blocks of nearly equal size
#'
#' @description
#' In case of shuffling and vectors that cannot be chunked evenly,
#' it is chosen randomly which levels / chunks will receive 1 element less.
#' If you do not shuffle, always the last chunks will receive 1 element less.
#'
#' @param x [ANY]\cr
#'   Vector, list or other type supported by \code{\link[base]{split}}.
#' @param chunk.size [\code{integer(1)}]\cr
#'   Requested number of elements in each chunk.
#'   Cannot be used in combination with \code{n.chunks} or \code{props}.
#'   If \code{x} cannot be evenly chunked, some chunks will have less elements.
#' @param n.chunks [\code{integer(1)}]\cr
#'   Requested number of chunks.
#'   If more chunks than elements in \code{x} are requested, empty chunks are
#'   dropped.
#'   Can not be used in combination with \code{chunks.size} or \code{props}.
#' @param props [\code{numeric}]\cr
#'   Vector of proportions for chunk sizes.
#'   Empty chunks may occur, depending on the length of \code{x} and the given
#'   proportions.
#'   Cannot be used in combination with \code{chunks.size} or \code{n.chunks}.
#' @param shuffle [\code{logical(1)}]\cr
#'   Shuffle \code{x}?
#'   Default is \code{FALSE}.
#' @return [unnamed \code{list}] of chunks.
#' @export
#' @examples
#' xs = 1:10
#' chunk(xs, chunk.size = 3)
#' chunk(xs, n.chunks = 2)
#' chunk(xs, n.chunks = 2, shuffle = TRUE)
#' chunk(xs, props = c(7, 3))
chunk = function(x, chunk.size, n.chunks, props, shuffle = FALSE) {
  assertFlag(shuffle)
  method = c("chunk.size", "n.chunks", "props")
  method = method[!c(missing(chunk.size), missing(n.chunks), missing(props))]
  if (length(method) != 1L)
    stop("You must provide exactly one of 'chunk.size', 'n.chunks' or 'props'")
  nx = length(x)

  ch = switch(method,
    chunk.size = {
      chunk.size = convertInteger(chunk.size)
      assertCount(chunk.size, positive = TRUE)
      getNChunks(nx, nx %/% chunk.size + (nx %% chunk.size > 0L), shuffle)
    },
    n.chunks = {
      n.chunks = convertInteger(n.chunks)
      assertCount(n.chunks, positive = TRUE)
      getNChunks(nx, n.chunks, shuffle)
    },
    props = {
      assertNumeric(props, min.len = 1L, any.missing = FALSE, lower = 0)
      props = props / sum(props)
      ch = factor(rep.int(seq_along(props), round(props * nx, digits = 0L)),
        levels = seq_along(props))
      if (shuffle) sample(ch) else ch
    })

  unname(split(x, ch))
}

getNChunks = function(nx, n.chunks, shuffle) {
  n.chunks = min(n.chunks, nx)
  if (shuffle) {
    c(sample(seq(0L, (nx %/% n.chunks) * n.chunks - 1L) %% n.chunks),
      sample(n.chunks, nx %% n.chunks) - 1L)
  } else {
    sort(seq.int(0L, nx - 1L) %% n.chunks)
  }
}

context("chunk")

test_that("chunk", {
  # normal chunk.size
  x = 1:9
  ch = chunk(x, chunk.size=3)
  expect_equal(ch, list(1:3, 4:6, 7:9))

  # normal n.chunks
  x = 1:9
  ch = chunk(x, n.chunks=3)
  expect_equal(ch, list(1:3, 4:6, 7:9))

  # chunk.size uneven
  x = 1:10
  ch = chunk(x, chunk.size=3)
  expect_equal(ch, list(1:3, 4:6, 7:8, 9:10))

  # n.chunks uneven
  ch = chunk(1:9, n.chunks=4)
  expect_equal(length(ch), 4)

  x = letters[1:10]
  ch = chunk(x, n.chunks = 2)
  expect_equal(ch, list(letters[1:5], letters[6:10]))

  # errors
  x = letters[1:10]
  expect_error(chunk(x, chunk.size=1, n.chunks=3))
  expect_error(chunk(x, chunk.size=1:2))
  expect_error(chunk(x, n.chunks=list()))

  x = as.list(letters[1:10])
  ch = chunk(x, chunk.size=5)
  expect_equal(ch, list(as.list(letters[1:5]), as.list(letters[6:10])))

  x = letters
  ch = chunk(x, chunk.size=4, shuffle=TRUE)
  expect_equal(sort(letters), sort(unlist(ch)))
  expect_true(all(sapply(ch, length) %in% c(3, 4)))

  # test that smaller levels get chosen randomly
  x = 1:5
  counts = sapply(1:100, function(i) {
    ch = chunk(x, chunk.size=3, shuffle=TRUE)
    sapply(ch, length) == 2
  })
  counts = rowSums(counts)
  expect_true(all(counts > 30))

  # test proportions
  x = 1:10
  ch = chunk(x, props = c(3, 7))
  expect_equal(sapply(ch, length), c(3, 7))
  expect_equal(unlist(ch), x)

  expect_true(length(chunk(x, props=1)) == 1L)

})

context("btwn")

test_that("btwn", {
  y = c(-1L,5L,Inf)
  expect_equal(1L:3L %btwn% y, c(TRUE, TRUE, TRUE))
  expect_equal(-2L:-1L %btwn% y, c(FALSE,TRUE))
  y = 5L
  expect_equal(5L %btwn% y, TRUE)
  expect_equal(1L %btwn% y, FALSE)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itostr.R
\name{itostr}
\alias{itostr}
\title{Convert Integers to Strings}
\usage{
itostr(x, base = 10L)
}
\arguments{
\item{x}{[\code{integer}]\cr
Vector of integers to convert.}

\item{base}{[\code{integer(1)}]\cr
Base for conversion. Values between 2 and 36 (inclusive) are allowed.}
}
\value{
\code{character(length(x))}.
}
\description{
This is the counterpart of \code{\link[base]{strtoi}}.
For a base greater than \sQuote{10}, letters \sQuote{a} to \sQuote{z}
are used to represent \sQuote{10} to \sQuote{35}.
}
\examples{
# binary representation of the first 10 natural numbers
itostr(1:10, 2)

# base36 encoding of a large number
itostr(1e7, 36)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/warningf.R
\name{warningf}
\alias{warningf}
\title{Wrapper for warning and sprintf}
\usage{
warningf(..., immediate = TRUE, warning.length = 8170L)
}
\arguments{
\item{...}{[any]\cr
See \code{\link{sprintf}}.}

\item{immediate}{[\code{logical(1)}]\cr
See \code{\link{warning}}.
Default is \code{TRUE}.}

\item{warning.length}{[\code{integer(1)}]\cr
Number of chars after which the warning message
gets truncated, see ?options.
Default is 8170.}
}
\value{
Nothing.
}
\description{
A wrapper for \code{\link{warning}} with \code{\link{sprintf}} applied to the arguments.
}
\examples{
msg = "a warning"
warningf("this is \%s", msg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertListOfRowsToDataFrame.R
\name{convertListOfRowsToDataFrame}
\alias{convertListOfRowsToDataFrame}
\title{Convert a list of row-vector of equal structure to a data.frame}
\usage{
convertListOfRowsToDataFrame(
  rows,
  strings.as.factors = NULL,
  row.names,
  col.names
)
}
\arguments{
\item{rows}{[\code{list}]\cr
List of rows. Each row is a list or vector of the same structure,
where all corresponding elements must have the same class.
It is allowed that in some rows some elements are not present, see above.}

\item{strings.as.factors}{[\code{logical(1)}]\cr
Convert character columns to factors?
Default is \code{default.stringsAsFactors()} for R < "4.1.0" and \code{FALSE} otherwise.}

\item{row.names}{[\code{character} | \code{integer} | \code{NULL}]\cr
Row names for result.
By default the names of the list \code{rows} are taken.}

\item{col.names}{[\code{character} | \code{integer}]\cr
Column names for result.
By default the names of an element of \code{rows} are taken.}
}
\value{
[\code{data.frame}].
}
\description{
Elements are arranged in columns according to their name in each
element of \code{rows}.
Variables that are not present in some row-lists, or encoded as \code{NULL}, are filled using NAs.
}
\examples{
convertListOfRowsToDataFrame(list(list(x = 1, y = "a"), list(x = 2, y = "b")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/system3.R
\name{system3}
\alias{system3}
\title{Wrapper for system2 with better return type and errorhandling}
\usage{
system3(
  command,
  args = character(0L),
  stdout = "",
  stderr = "",
  wait = TRUE,
  ...,
  stop.on.exit.code = wait
)
}
\arguments{
\item{command}{See \code{\link{system2}}.}

\item{args}{See \code{\link{system2}}.}

\item{stdout}{See \code{\link{system2}}.}

\item{stderr}{See \code{\link{system2}}.}

\item{wait}{See \code{\link{system2}}.}

\item{...}{Further arguments passed to \code{\link{system2}}.}

\item{stop.on.exit.code}{[\code{logical(1)}]\cr
Should an exception be thrown if an exit code greater 0 is generated?
Can only be used if  \code{wait} is \code{TRUE}.
Default is \code{wait}.}
}
\value{
[\code{list}].
  \item{exit.code [integer(1)]}{Exit code of command. Given if wait is \code{TRUE}, otherwise \code{NA}. 0L means success. 127L means command was not found}
  \item{output [character]}{Output of command on streams. Only given is \code{stdout} or \code{stderr} was set to \code{TRUE}, otherwise \code{NA}.}
}
\description{
Wrapper for \code{\link{system2}} with better return type and errorhandling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isProperlyNamed.R
\name{isProperlyNamed}
\alias{isProperlyNamed}
\title{Are all elements of a list / vector uniquely named?}
\usage{
isProperlyNamed(x)
}
\arguments{
\item{x}{[\code{vector}]\cr
The vector or list.}
}
\value{
[\code{logical(1)}].
}
\description{
\code{NA} or \dQuote{} are not allowed as names.
}
\examples{
isProperlyNamed(list(1))
isProperlyNamed(list(a = 1))
isProperlyNamed(list(a = 1, 2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strrepeat.R
\name{strrepeat}
\alias{strrepeat}
\title{Repeat and join a string}
\usage{
strrepeat(x, n, sep = "")
}
\arguments{
\item{x}{[character]\cr
Vector of characters.}

\item{n}{[\code{integer(1)}]\cr
Times the vector \code{x} is repeated.}

\item{sep}{[\code{character(1)}]\cr
Separator to use to collapse the vector of characters.}
}
\value{
\code{character(1)}.
}
\description{
Repeats a string n times and joins the results.
}
\examples{
strrepeat("x", 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isSuperset.R
\name{isSuperset}
\alias{isSuperset}
\title{Check superset relation on two vectors}
\usage{
isSuperset(x, y, strict = FALSE)
}
\arguments{
\item{x}{[\code{vector}]\cr
Source vector.}

\item{y}{[\code{vector}]\cr
Vector of the same mode as \code{x}.}

\item{strict}{[\code{logical(1)}]\cr
Checks for strict/proper superset relation.}
}
\value{
[\code{logical(1)}]
  \code{TRUE} if each element of \code{y} is also contained in \code{x}, i. e.,
  if \code{y} is a subset of \code{x} and \code{FALSE} otherwise.
}
\description{
Checks if all elements of y are contained in x.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dropNamed.R
\name{dropNamed}
\alias{dropNamed}
\title{Drop named elements of an object}
\usage{
dropNamed(x, drop = character(0L))
}
\arguments{
\item{x}{[any]\cr
Object to drop named elements from.
For a matrix or a data frames this function drops named columns via
the second argument of the binary index operator \code{[,]}.
Otherwise, the unary index operator \code{[]} is used for dropping.}

\item{drop}{[\code{character}]\cr
Names of elements to drop.}
}
\value{
Subset of object of same type as \code{x}. The object is not simplified,
  i.e, no dimensions are dropped as \code{[,,drop = FALSE]} is used.
}
\description{
Removes named elements from a vector, list, matrix or data frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getClass1.R
\name{getClass1}
\alias{getClass1}
\title{Wrapper for \code{class(x)[1]}}
\usage{
getClass1(x)
}
\arguments{
\item{x}{[any]\cr
Input object.}
}
\value{
[\code{character(1)}].
}
\description{
Returns the first class of an object.
}
\note{
\code{getClass} is a function in \code{methods}. Do not confuse.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertDataFrameCols.R
\name{convertDataFrameCols}
\alias{convertDataFrameCols}
\title{Converts columns in a data frame to characters, factors or numerics}
\usage{
convertDataFrameCols(
  df,
  chars.as.factor = FALSE,
  factors.as.char = FALSE,
  ints.as.num = FALSE,
  logicals.as.factor = FALSE
)
}
\arguments{
\item{df}{[\code{data.frame}]\cr
Data frame.}

\item{chars.as.factor}{[\code{logical(1)}]\cr
Should characters be converted to factors?
Default is \code{FALSE}.}

\item{factors.as.char}{[\code{logical(1)}]\cr
Should characters be converted to factors?
Default is \code{FALSE}.}

\item{ints.as.num}{[\code{logical(1)}]\cr
Should integers be converted to numerics?
Default is \code{FALSE}.}

\item{logicals.as.factor}{[\code{logical(1)}]\cr
Should logicals be converted to factors?
Default is \code{FALSE}.}
}
\value{
[\code{data.frame}].
}
\description{
Converts columns of a data frame based on specified conversion flags.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq.R
\name{seq_row}
\alias{seq_row}
\alias{seq_col}
\title{Generate sequences along rows or cols}
\usage{
seq_row(x)

seq_col(x)
}
\arguments{
\item{x}{[\code{data.frame} | \code{matrix}]\cr
Data frame, matrix or any object which supports \code{\link[base]{nrow}}
or \code{\link[base]{ncol}}, respectively.}
}
\value{
Vector of type [\code{integer}].
}
\description{
A simple convenience wrapper around \code{\link[base]{seq_len}}.
}
\examples{
data(iris)
seq_row(iris)
seq_col(iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asMatrix.R
\name{asMatrixCols}
\alias{asMatrixCols}
\alias{asMatrixRows}
\title{Extracts a named element from a list of lists}
\usage{
asMatrixCols(xs, row.names, col.names)

asMatrixRows(xs, row.names, col.names)
}
\arguments{
\item{xs}{[\code{list}]\cr
A list of vectors of the same length.}

\item{row.names}{[\code{character} | \code{integer} | \code{NULL}]\cr
Row names of result.
Default is to take the names of the elements of \code{xs}.}

\item{col.names}{[\code{character} | \code{integer} | \code{NULL}]\cr
Column names of result.
Default is to take the names of the elements of \code{xs}.}
}
\value{
[\code{matrix}].
}
\description{
Converts a list of vectors into a matrix with vectors as columns or rows.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pause.R
\name{pause}
\alias{pause}
\title{Pause in interactive mode and continue on <Enter>.}
\usage{
pause()
}
\description{
Pause in interactive mode and continue on <Enter>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explode.R
\name{explode}
\alias{explode}
\title{Split up a string into substrings}
\usage{
explode(x, sep = " ")
}
\arguments{
\item{x}{[\code{character}]\cr
Source string.}

\item{sep}{[\code{character}]\cr
Seperator whcih is used to split \code{x} into substrings.
Default is \dQuote{ }.}
}
\value{
[\code{vector}]
  Vector of substrings.
}
\description{
Split up a string into substrings according to a seperator.
}
\examples{
explode("foo bar")
explode("comma,seperated,values", sep = ",")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/messagef.R
\name{messagef}
\alias{messagef}
\title{Wrapper for message and sprintf}
\usage{
messagef(..., .newline = TRUE)
}
\arguments{
\item{...}{[any]\cr
See \code{\link{sprintf}}.}

\item{.newline}{[logical(1)]\cr
Add a newline to the message. Default is \code{TRUE}.}
}
\value{
Nothing.
}
\description{
A simple wrapper for \code{message(sprintf(...))}.
}
\examples{
msg = "a message"
warningf("this is \%s", msg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDataFrame.R
\name{makeDataFrame}
\alias{makeDataFrame}
\title{Initialize data.frame in a convenient way}
\usage{
makeDataFrame(
  nrow,
  ncol,
  col.types,
  init,
  row.names = NULL,
  col.names = sprintf("V\%i", seq_len(ncol))
)
}
\arguments{
\item{nrow}{[\code{integer(1)}]\cr
Nubmer of rows.}

\item{ncol}{[\code{integer(1)}]\cr
Number of columns.}

\item{col.types}{[\code{character(ncol)} | \code{character(1)}]\cr
Data types of columns.
If you only pass one type, it will be replicated.
Supported are all atomic modes also supported by
\code{\link[base]{vector}}, i.e. all common data frame types except factors.}

\item{init}{[any]\cr
Scalar object to initialize all elements of the data.frame.
You do not need to specify \code{col.types} if you pass this.}

\item{row.names}{[\code{character} | \code{integer} | \code{NULL}]\cr
Row names.
Default is \code{NULL}.}

\item{col.names}{[\code{character} | \code{integer}]\cr
Column names.
Default is \dQuote{V1}, \dQuote{V2}, and so on.}
}
\description{
Creates a data frame with specified dimensions, column types and optional initial values.
}
\examples{
print(makeDataFrame(3, 2, init = 7))
print(makeDataFrame(3, 2, "logical"))
print(makeDataFrame(3, 2, c("logical", "numeric")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symdiff.R
\name{symdiff}
\alias{symdiff}
\title{Calculates symmetric set difference between two sets}
\usage{
symdiff(x, y)
}
\arguments{
\item{x}{[\code{vector}]\cr
Set 1.}

\item{y}{[\code{vector}]\cr
Set 2.}
}
\value{
[\code{vector}].
}
\description{
Returns elements that are in either set but not in both.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeProgressBar.R
\name{makeProgressBar}
\alias{makeProgressBar}
\alias{ProgressBar}
\title{Create a progress bar with estimated time.}
\usage{
makeProgressBar(min = 0, max = 100, label = "", char = "+",
  style = getOption("BBmisc.ProgressBar.style", "text"),
  width = getOption("BBmisc.ProgressBar.width", getOption("width")),
  stream = getOption("BBmisc.ProgressBar.stream", "stderr"))
}
\arguments{
\item{min}{[\code{numeric(1)}]\cr
Minimum value, default is 0.}

\item{max}{[\code{numeric(1)}]\cr
Maximum value, default is 100.}

\item{label}{[\code{character(1)}]\cr
Label shown in front of the progress bar.
Note that if you later set \code{msg} in the progress bar function,
the message will be left-padded to the length of this label, therefore
it should be at least as long as the longest message you want to display.
Default is \dQuote{}.}

\item{char}{[\code{character(1)}]\cr
A single character used to display progress in the bar.
Default is \sQuote{+}.}

\item{style}{[\code{character(1)}]\cr
Style of the progress bar. Default is set via options (see details).}

\item{width}{[\code{integer(1)}]\cr
Width of the progress bar. Default is set via options (see details).}

\item{stream}{[\code{character(1)}]\cr
Stream to use. Default is set via options (see details).}
}
\value{
[\code{\link{ProgressBar}}]. A list with following functions:
  \item{set [\code{function(value, msg = label)}]}{Set the bar to a value and possibly display a message instead of the label.}
  \item{inc [\code{function(value, msg = label)}]}{Increase the bar and possibly display a message instead of the label.}
  \item{kill [\code{function(clear = FALSE)}]}{Kill the bar so it cannot be used anymore. Cursor is moved to new line. You can also erase its display.}
  \item{error [\code{function(e)}]}{Useful in \code{tryCatch} to properly display error messages below the bar. See the example.}
}
\description{
Create a progress bar function that displays the estimated time till
completion and optional messages. Call the returned functions \code{set} or
\code{inc} during a loop to change the display.
Note that you are not allowed to decrease the value of the bar.
If you call these function without setting any of the arguments
the bar is simply redrawn with the current value.
For errorhandling use \code{error} and have a look at the example below.

You can globally change the behavior of all bars by setting the option
\code{options(BBmisc.ProgressBar.style)} either to \dQuote{text} (the default)
or \dQuote{off}, which display no bars at all.

You can globally change the width of all bars by setting the option
\code{options(BBmisc.ProgressBar.width)}. By default this is \code{getOption("width")}.

You can globally set the stream where the output of the bar is directed by setting the option
\code{options(BBmisc.ProgressBar.stream)} either to \dQuote{stderr} (the default)
or \dQuote{stdout}. Note that using the latter will result in the bar being shown in
reports generated by Sweave or knitr, what you probably do not want.
}
\examples{
bar = makeProgressBar(max = 5, label = "test-bar")
for (i in 0:5) {
  bar$set(i)
  Sys.sleep(0.2)
}
bar = makeProgressBar(max = 5, label = "test-bar")
for (i in 1:5) {
  bar$inc(1)
  Sys.sleep(0.2)
}
# display errors properly (in next line)
\dontrun{
f = function(i) if (i>2) stop("foo")
bar = makeProgressBar(max = 5, label = "test-bar")
for (i in 1:5) {
  tryCatch ({
    f(i)
    bar$set(i)
  }, error = bar$error)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addClasses.R
\name{addClasses}
\alias{addClasses}
\title{A wrapper to add to the class attribute}
\usage{
addClasses(x, classes)
}
\arguments{
\item{x}{[any]\cr
Your object.}

\item{classes}{[\code{character}]\cr
Classes to add. Will be added in front (specialization).}
}
\value{
Changed object \code{x}.
}
\description{
Adds classes to the class attribute of an object.
}
\examples{
x = list()
print(class(x))
x = addClasses(x, c("foo1", "foo2"))
print(class(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertToShortString.R
\name{convertToShortString}
\alias{convertToShortString}
\title{Converts any R object to a descriptive string so it can be used in messages}
\usage{
convertToShortString(x, num.format = "\%.4g", clip.len = 15L)
}
\arguments{
\item{x}{[any]\cr
The object.}

\item{num.format}{[\code{character(1)}]\cr
Used to format numerical scalars via \code{\link{sprintf}}.
Default is \dQuote{\%.4g}.}

\item{clip.len}{[\code{integer(1)}]\cr
Used clip atomic vectors via \code{\link{clipString}}.
Default is 15.}
}
\value{
[\code{character(1)}].
}
\description{
Atomics: If of length 0 or 1, they are basically printed as they are.
Numerics are formated with \code{num.format}.
If of length greater than 1, they are collapsed witd \dQuote{,} and clipped.
so they do not become excessively long.
Expressions will be converted to plain text.

All others: Currently, only their class is simply printed
like \dQuote{<data.frame>}.

Lists: The mechanism above is applied (non-recursively) to their elements.
The result looks like this:
\dQuote{a=1, <unamed>=2, b=<data.frame>, c=<list>}.
}
\examples{
convertToShortString(list(a = 1, b = NULL, "foo", c = 1:10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/btwn.R
\name{\%btwn\%}
\alias{\%btwn\%}
\title{Check if some values are covered by the range of the values in a second vector}
\usage{
x \%btwn\% y
}
\arguments{
\item{x}{[\code{numeric(n)}]\cr
Value(s) that should be within the range of \code{y}.}

\item{y}{[\code{numeric}]\cr
Numeric vector which defines the range.}
}
\value{
[\code{logical(n)}]. For each value in \code{x}: Is it in the range of \code{y}?
}
\description{
Tests whether values in x are between the minimum and maximum of y.
}
\examples{
x = 3
y = c(-1,2,5)
x \%btwn\% y
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setValue.R
\name{setValue}
\alias{setValue}
\title{Set a list element to a new value}
\usage{
setValue(obj, index, newval)
}
\arguments{
\item{obj}{[\code{list}]\cr}

\item{index}{[\code{character} | \code{integer}]\cr
Index or indices where to insert the new values.}

\item{newval}{[any]\cr
Inserted elements(s).
Has to be a list if \code{index} is a vector.}
}
\value{
[\code{list}]
}
\description{
This wrapper supports setting elements to \code{NULL}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAttributeNames.R
\name{getAttributeNames}
\alias{getAttributeNames}
\title{Helper function for determining the vector of attribute names of a given object}
\usage{
getAttributeNames(obj)
}
\arguments{
\item{obj}{[any]\cr
Source object.}
}
\value{
[\code{character}]
  Vector of attribute names for the source object.
}
\description{
Returns the names of all attributes of an object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setRowColNames.R
\name{setRowNames}
\alias{setRowNames}
\alias{setColNames}
\title{Wrapper for \code{rownames(x) = y}, \code{colnames(x) = y}}
\usage{
setRowNames(x, names)

setColNames(x, names)
}
\arguments{
\item{x}{[\code{matrix} | \code{data.frame}]\cr
Matrix or data.frame.}

\item{names}{[\code{character}]\cr
New names for rows / columns.}
}
\value{
Changed object \code{x}.
}
\description{
Sets row or column names on a matrix or data frame and returns the modified object.
}
\examples{
setColNames(matrix(1:4, 2, 2), c("a", "b"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/namedList.R
\name{namedList}
\alias{namedList}
\title{Create named list, possibly initialized with a certain element}
\usage{
namedList(names, init)
}
\arguments{
\item{names}{[\code{character}]\cr
Names of elements.}

\item{init}{[valid R expression]\cr
If given all list elements are initialized to this, otherwise
\code{NULL} is used.}
}
\value{
[\code{list}].
}
\description{
Even an empty list will always be named.
}
\examples{
namedList(c("a", "b"))
namedList(c("a", "b"), init = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractSubList.R
\name{extractSubList}
\alias{extractSubList}
\title{Extracts a named element from a list of lists}
\usage{
extractSubList(xs, element, element.value, simplify = TRUE, use.names = TRUE)
}
\arguments{
\item{xs}{[\code{list}]\cr
A list of named lists.}

\item{element}{[\code{character}]\cr
Name of element(s) to extract from the list elements of \code{xs}.
What happens is this: \code{x$el1$el2....}.}

\item{element.value}{[any]\cr
If given, \code{\link{vapply}} is used and this argument is passed to \code{FUN.VALUE}.
Note that even for repeated indexing (if length(element) > 1) you only
pass one value here which refers to the data type of the final result.}

\item{simplify}{[\code{logical(1)} | character(1)]\cr
If \code{FALSE} \code{\link{lapply}} is used, otherwise \code{\link{sapply}}.
If \dQuote{cols}, we expect the elements to be vectors of the same length and they are
arranged as the columns of the resulting matrix.
If \dQuote{rows}, likewise, but rows of the resulting matrix.
Default is \code{TRUE}.}

\item{use.names}{[\code{logical(1)}]\cr
If \code{TRUE} and \code{xs} is named, the result is named as \code{xs},
otherwise the result is unnamed.
Default is \code{TRUE}.}
}
\value{
[\code{list} | simplified \code{vector} | \code{matrix}]. See above.
}
\description{
Extracts elements from a list of named lists by element name.
}
\examples{
xs = list(list(a = 1, b = 2), list(a = 5, b = 7))
extractSubList(xs, "a")
extractSubList(xs, "a", simplify = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowLapply.R
\name{rowLapply}
\alias{rowLapply}
\alias{rowSapply}
\title{Apply function to rows of a data frame}
\usage{
rowLapply(df, fun, ..., unlist = FALSE)

rowSapply(df, fun, ..., unlist = FALSE, simplify = TRUE, use.names = TRUE)
}
\arguments{
\item{df}{[\code{data.frame}]\cr
Data frame.}

\item{fun}{[\code{function}]\cr
Function to apply. Rows are passed as list or vector,
depending on argument \code{unlist}, as first argument.}

\item{...}{[\code{ANY}]\cr
Additional arguments for \code{fun}.}

\item{unlist}{[\code{logical(1)}]\cr
Unlist the row? Note that automatic conversion may be triggered for
lists of mixed data types
Default is \code{FALSE}.}

\item{simplify}{[\code{logical(1)} | character(1)]\cr
Should the result be simplified?
See \code{\link{sapply}}.
If \dQuote{cols}, we expect the call results to be vectors of the same length and they are
arranged as the columns of the resulting matrix.
If \dQuote{rows}, likewise, but rows of the resulting matrix.
Default is \code{TRUE}.}

\item{use.names}{[\code{logical(1)}]\cr
Should result be named by the row names of \code{df}?
Default is \code{TRUE}.}
}
\value{
[\code{list} or simplified object]. Length is \code{nrow(df)}.
}
\description{
Just like an \code{\link[base]{lapply}} on data frames,
but on the rows.
}
\examples{
 rowLapply(iris, function(x) x$Sepal.Length + x$Sepal.Width)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/directory.R
\name{isEmptyDirectory}
\alias{isEmptyDirectory}
\title{Is one / are several directories empty?}
\usage{
isEmptyDirectory(...)
}
\arguments{
\item{...}{[\code{character(1)}]\cr
Directory names, all strings.}
}
\value{
[\code{logical}].
}
\description{
If file does not exist or is not a directory, \code{FALSE} is returned.
}
\examples{
print(isEmptyDirectory(tempdir()))
print(isEmptyDirectory(tempfile()))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capitalizeStrings.R
\name{capitalizeStrings}
\alias{capitalizeStrings}
\title{Capitalize strings in a vector}
\usage{
capitalizeStrings(x, all.words = FALSE, lower.back = FALSE)
}
\arguments{
\item{x}{[\code{character(n)}]\cr
Vector of character elements to capitalize.}

\item{all.words}{[\code{logical(1)}]\cr
If \code{TRUE} all words of each vector element are capitalized.
\code{FALSE} capitalizes the first word of each vector element.}

\item{lower.back}{[\code{logical(1)}]\cr
\code{TRUE} lowers the back of each word or vector element (depends on \code{all.words}).}
}
\value{
Capitalized vector: [\code{character(n)}].
}
\description{
Capitalise first word or all words of a character vector.
Lower back of vector element or word, respectively.
}
\examples{
capitalizeStrings(c("the taIl", "wags The dOg", "That looks fuNny!"))
capitalizeStrings(c("the taIl", "wags The dOg", "That looks fuNny!")
, all.words = TRUE, lower.back = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFirstLast.R
\name{getFirst}
\alias{getFirst}
\alias{getLast}
\title{Get the first/last element of a list/vector}
\usage{
getFirst(x)

getLast(x)
}
\arguments{
\item{x}{[\code{list} | \code{vector}]\cr
The list or vector.}
}
\value{
Selected element. The element name is dropped.
}
\description{
Returns the first or last element of a list or vector.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asQuoted.R
\name{asQuoted}
\alias{asQuoted}
\title{Converts a string into a quoted expression}
\usage{
asQuoted(s, env = parent.frame())
}
\arguments{
\item{s}{[\code{character(1)}]\cr
Expression as string.}

\item{env}{[\code{numeric(1)}]\cr
Environment for expression.
Default is \code{parent.frame()}}
}
\value{
Quoted expression.
}
\description{
Works the same as if you would have entered the expression and called
\code{\link{quote}} on it.
}
\examples{
asQuoted("x == 3")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getOperatingSystem.R
\name{getOperatingSystem}
\alias{getOperatingSystem}
\alias{isWindows}
\alias{isUnix}
\alias{isLinux}
\alias{isDarwin}
\title{Functions to determine the operating system}
\usage{
getOperatingSystem()

isWindows()

isUnix()

isLinux()

isDarwin()
}
\value{
See above.
}
\description{
\describe{
\item{getOperatingSystem}{Simple wrapper for \code{.Platform$OS.type}, returns \code{character(1)}.}
\item{isUnix}{Predicate for OS string, returns \code{logical(1)}. Currently this would include Unix, Linux and Mac flavours.}
\item{isLinux}{Predicate for sysname string, returns \code{logical(1)}.}
\item{isDarwin}{Predicate for sysname string, returns \code{logical(1)}.}
\item{isWindows}{Predicate for OS string, returns \code{logical(1)}.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkArg.R
\name{checkArg}
\alias{checkArg}
\title{Check for a function argument}
\usage{
checkArg(
  x,
  cl,
  s4 = FALSE,
  len,
  min.len,
  max.len,
  choices,
  subset,
  lower = NA,
  upper = NA,
  na.ok = TRUE,
  formals
)
}
\arguments{
\item{x}{[any]\cr
Argument.}

\item{cl}{[\code{character}]\cr
Class that argument must \dQuote{inherit} from.
If multiple classes are given, \code{x} must \dQuote{inherit} from at least one of these.
See also argument \code{s4}.}

\item{s4}{[\code{logical(1)}]\cr
If \code{TRUE}, use \code{is} for checking class \code{cl}, otherwise use \code{\link{inherits}}, which
implies that only S3 classes are correctly checked. This is done for speed reasons
as calling \code{\link{is}} is pretty slow.
Default is \code{FALSE}.}

\item{len}{[\code{integer(1)}]\cr
Length that argument must have.
Not checked if not passed, which is the default.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length that argument must have.
Not checked if not passed, which is the default.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length that argument must have.
Not checked if not passed, which is the default.}

\item{choices}{[any]\cr
Discrete number of choices, expressed by a vector of R objects.
If passed, argument must be identical to one of these and nothing else is checked.}

\item{subset}{[any]\cr
Discrete number of choices, expressed by a vector of R objects.
If passed, argument must be identical to a subset of these and nothing else is checked.}

\item{lower}{[\code{numeric(1)}]\cr
Lower bound for numeric vector arguments.
Default is \code{NA}, which means not required.}

\item{upper}{[\code{numeric(1)}]\cr
Upper bound for numeric vector arguments.
Default is \code{NA}, which means not required.}

\item{na.ok}{[\code{logical(1)}]\cr
Is it ok if a vector argument contains NAs?
Default is \code{TRUE}.}

\item{formals}{[\code{character}]\cr
If this is passed, \code{x} must be a function.
It is then checked that \code{formals} are the names of the
(first) formal arguments in the signature of \code{x}.
Meaning \code{checkArg(function(a, b), formals = "a")} is ok.
Default is missing.}
}
\value{
Nothing.
}
\description{
Throws exception if checks are not passed. Note that argument is evaluated when checked.

This function is superseded by the package \pkg{checkmate} and might get deprecated
in the future. Please
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mapValues.R
\name{mapValues}
\alias{mapValues}
\title{Replace values in atomic vectors}
\usage{
mapValues(
  x,
  from,
  to,
  regex = FALSE,
  ignore.case = FALSE,
  perl = FALSE,
  fixed = FALSE
)
}
\arguments{
\item{x}{[\code{atomic}]\cr
Atomic vector. If \code{x} is a factor, all replacements work on the levels.}

\item{from}{[\code{atomic}]\cr
Atomic vector with values to replace, same length as \code{to}.}

\item{to}{[\code{atomic}]\cr
Atomic vector with replacements, same length as \code{from}.}

\item{regex}{[\code{logical}]\cr
Use regular expression matching? Default is \code{FALSE}.}

\item{ignore.case}{[\code{logical}]\cr
Argument passed to \code{\link[base]{gsub}}.}

\item{perl}{[\code{logical}]\cr
Argument passed to \code{\link[base]{gsub}}.}

\item{fixed}{[\code{logical}]\cr
Argument passed to \code{\link[base]{gsub}}.}
}
\value{
[\code{atomic}].
}
\description{
Replace values in atomic vectors
}
\details{
Replaces values specified in \code{from} with values in \code{to}.
Regular expression matching can be enabled which calls \code{\link[base]{gsub}} iteratively
on \code{x} to replace all patterns in \code{from} with replacements in \code{to}.
}
\examples{
# replace integers
x = 1:5
mapValues(x, c(2, 3), c(99, 100))

# replace factor levels using regex matching
x = factor(c("aab", "aba", "baa"))
mapValues(x, "a.a", "zzz", regex = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeSimpleFileLogger.R
\name{makeSimpleFileLogger}
\alias{makeSimpleFileLogger}
\alias{SimpleFileLogger}
\title{Simple logger which outputs to a file}
\usage{
makeSimpleFileLogger(logfile, touch = FALSE, keep = 10L)
}
\arguments{
\item{logfile}{[\code{character(1)}]\cr
File to log to.}

\item{touch}{[\code{logical(1)}]\cr
Should the file be created before the first log message?
Default is \code{FALSE}.}

\item{keep}{[\code{integer(1)}]\cr
Number of log messages to keep in memory for quick access.
Default is \code{10}.}
}
\value{
[\code{\link{SimpleFileLogger}}]. A list with following functions:
  \item{log [function(msg)]}{Send log message.}
  \item{getMessages [function(n)]}{Get last \code{n} log messages.}
  \item{clear [function()]}{Resets logger and deletes log file.}
  \item{getSize [function()]}{Returns the number of logs written.}
  \item{getLogfile [function()]}{Returns the full file name logs are written to.}
}
\description{
Creates a simple file logger closure to log to a file, including time stamps.
An optional buffer holds the last few log messages.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/directory.R
\name{isDirectory}
\alias{isDirectory}
\title{Is one / are several files a directory?}
\usage{
isDirectory(...)
}
\arguments{
\item{...}{[\code{character(1)}]\cr
File names, all strings.}
}
\value{
[\code{logical}].
}
\description{
If a file does not exist, \code{FALSE} is returned.
}
\examples{
print(isDirectory(tempdir()))
print(isDirectory(tempfile()))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertMatrixType.R
\name{convertMatrixType}
\alias{convertMatrixType}
\title{Converts storage type of a matrix}
\usage{
convertMatrixType(x, type)
}
\arguments{
\item{x}{[\code{matrix}]\cr.
Matrix to convert.}

\item{type}{[\code{character(1)}]\cr
New storage type.}
}
\value{
[\code{matrix}].
}
\description{
Works by setting \code{\link{mode}}.
}
\note{
\code{as.mytype} drops dimension when used on a matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize.R
\name{normalize}
\alias{normalize}
\title{Normalizes numeric data to a given scale.}
\usage{
normalize(x, method = "standardize", range = c(0, 1), margin = 1L,
  on.constant = "quiet")
}
\arguments{
\item{x}{[\code{numeric} | \code{matrix} | \code{data.frame}]\cr
Input vector.}

\item{method}{[\code{character(1)}]\cr
Normalizing method. Available are:\cr
\dQuote{center}: Subtract mean.\cr
\dQuote{scale}: Divide by standard deviation.\cr
\dQuote{standardize}: Center and scale.\cr
\dQuote{range}: Scale to a given range.\cr}

\item{range}{[\code{numeric(2)}]\cr
Range for method \dQuote{range}.
The first value represents the replacement for the min value, the second is the substitute for the max value.
So it is possible to reverse the order by giving \code{range = c(1,0)}.
Default is \code{c(0,1)}.}

\item{margin}{[\code{integer(1)}]\cr
1 = rows, 2 = cols.
Same is in \code{\link{apply}}
Default is 1.}

\item{on.constant}{[\code{character(1)}]\cr
How should constant vectors be treated? Only used, of \dQuote{method != center},
since this methods does not fail for constant vectors. Possible actions are:\cr
\dQuote{quiet}: Depending on the method, treat them quietly:\cr
  \dQuote{scale}: No division by standard deviation is done, input values.
     will be returned untouched.\cr
  \dQuote{standardize}: Only the mean is subtracted, no division is done.\cr
  \dQuote{range}: All values are mapped to the mean of the given range.\cr
\dQuote{warn}: Same behaviour as \dQuote{quiet}, but print a warning message.\cr
\dQuote{stop}: Stop with an error.\cr}
}
\value{
[\code{numeric} | \code{matrix} | \code{data.frame}].
}
\description{
Currently implemented for numeric vectors, numeric matrices and data.frame.
For matrixes one can operate on rows or columns
For data.frames, only the numeric columns are touched, all others are left unchanged.
For constant vectors / rows / columns most methods fail, special behaviour for this
case is implemented.

The method also handles NAs in in \code{x} and leaves them untouched.
}
\seealso{
\code{\link{scale}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/which.first.R
\name{which.first}
\alias{which.first}
\alias{which.last}
\title{Find the index of first/last \code{TRUE} value in a logical vector}
\usage{
which.first(x, use.names = TRUE)

which.last(x, use.names = TRUE)
}
\arguments{
\item{x}{[\code{logical}]\cr
Logical vector.}

\item{use.names}{[\code{logical(1)}]\cr
If \code{TRUE} and \code{x} is named, the result is also
named.}
}
\value{
[\code{integer(1)} | \code{integer(0)}].
  Returns the index of the first/last \code{TRUE} value in \code{x} or
  an empty integer vector if none is found.
}
\description{
Returns the index of the first or last TRUE value in a logical vector.
}
\examples{
 which.first(c(FALSE, TRUE))
 which.last(c(FALSE, FALSE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setClasses.R
\name{setClasses}
\alias{setClasses}
\title{A wrapper for \code{class(x) = classes}}
\usage{
setClasses(x, classes)
}
\arguments{
\item{x}{[any]\cr
Your object.}

\item{classes}{[\code{character}]\cr
New classes.}
}
\value{
Changed object \code{x}.
}
\description{
Sets the class attribute of an object and returns the modified object.
}
\examples{
setClasses(list(), c("foo1", "foo2"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hasAttributes.R
\name{hasAttributes}
\alias{hasAttributes}
\title{Check if given object has certain attributes}
\usage{
hasAttributes(obj, attribute.names)
}
\arguments{
\item{obj}{[mixed]\cr
Arbitrary R object.}

\item{attribute.names}{[\code{character}]\cr
Vector of strings, i.e., attribute names.}
}
\value{
[\code{logical(1)}]
  \code{TRUE} if object \code{x} contains all attributes from \code{attributeNames}
  and \code{FALSE} otherwise.
}
\description{
Checks whether an object has all specified attributes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stopf.R
\name{stopf}
\alias{stopf}
\title{Wrapper for stop and sprintf}
\usage{
stopf(..., warning.length = 8170L)
}
\arguments{
\item{...}{[any]\cr
See \code{\link{sprintf}}.}

\item{warning.length}{[\code{integer(1)}]\cr
Number of chars after which the error message
gets truncated, see ?options.
Default is 8170.}
}
\value{
Nothing.
}
\description{
A wrapper for \code{\link{stop}} with \code{\link{sprintf}} applied to the arguments.
Notable difference is that error messages are not truncated to 1000 characters
by default.
}
\examples{
err = "an error."
try(stopf("This is \%s", err))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isScalarNA.R
\name{isScalarNA}
\alias{isScalarNA}
\title{Checks whether an object is a scalar NA value}
\usage{
isScalarNA(x)
}
\arguments{
\item{x}{[any]\cr
Object to check.}
}
\value{
[\code{logical(1)}].
}
\description{
Checks whether object is from \code{(NA, NA_integer, NA_real_, NA_character_, NA_complex_)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argsAsNamedList.R
\name{argsAsNamedList}
\alias{argsAsNamedList}
\title{Parses \code{...} arguments to a named list}
\usage{
argsAsNamedList(...)
}
\arguments{
\item{...}{Arbitrary number of objects.}
}
\value{
[\code{list}]: Named list with objects.
}
\description{
The deparsed name will be used for arguments with missing names.
Missing names will be set to \code{NA}.
}
\examples{
z = 3
argsAsNamedList(x = 1, y = 2, z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nin.R
\name{\%nin\%}
\alias{\%nin\%}
\title{Simply a negated \code{in} operator}
\usage{
x \%nin\% y
}
\arguments{
\item{x}{[\code{vector}]\cr
Values that should not be in \code{y}.}

\item{y}{[\code{vector}]\cr
Values to match against.}
}
\description{
Tests if elements of x are not in y.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterNull.R
\name{filterNull}
\alias{filterNull}
\title{Filter a list for NULL values}
\usage{
filterNull(li)
}
\arguments{
\item{li}{[\code{list}]\cr
List.}
}
\value{
[\code{list}].
}
\description{
Removes all NULL elements from a list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dapply.R
\name{dapply}
\alias{dapply}
\title{Call \code{lapply} on an object and return a data.frame}
\usage{
dapply(x, fun, ..., col.names)
}
\arguments{
\item{x}{[\code{data.frame}]\cr
Data frame.}

\item{fun}{[\code{function}]\cr
The function to apply.}

\item{...}{[any]\cr
Further arguments passed down to \code{fun}.}

\item{col.names}{[\code{character(1)}]\cr
Column names for result.
Default are the names of \code{x}.}
}
\value{
[\code{data.frame}].
}
\description{
Applies a function \code{fun} on each element of input \code{x}
and combines the results as \code{data.frame} columns.
The results will get replicated to have equal length
if necessary and possible.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coalesce.R
\name{coalesce}
\alias{coalesce}
\title{Returns first non-missing, non-null argument}
\usage{
coalesce(...)
}
\arguments{
\item{...}{[any]\cr
Arguments.}
}
\value{
[any].
}
\description{
Returns first non-missing, non-null argument, otherwise \code{NULL}.

We have to perform some pretty weird \code{\link{tryCatch}} stuff internally,
so you should better not pass complex function calls into the arguments that can throw exceptions,
as these will be completely muffled, and return \code{NULL} in the end.
}
\examples{
f = function(x,y) {
  print(coalesce(NULL, x, y))
}
f(y = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suppressAll.R
\name{suppressAll}
\alias{suppressAll}
\title{Suppresses all output except for errors}
\usage{
suppressAll(expr)
}
\arguments{
\item{expr}{[valid R expression]\cr
Expression.}
}
\value{
Return value of expression invisibly.
}
\description{
Evaluates an expression and suppresses all output except for errors, 
meaning: prints, messages, warnings and package startup messages.
}
\examples{
suppressAll({
  print("foo")
  message("foo")
  warning("foo")
})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isFALSE.R
\name{isFALSE}
\alias{isFALSE}
\title{A wrapper for \code{identical(x, FALSE)}}
\usage{
isFALSE(x)
}
\arguments{
\item{x}{[any]\cr
Your object.}
}
\value{
[\code{logical(1)}].
}
\description{
Tests if an object is identical to FALSE.
}
\examples{
isFALSE(0)
isFALSE(FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FileCache.R
\name{makeFileCache}
\alias{makeFileCache}
\title{A caching wrapper around load2}
\usage{
makeFileCache(use.cache = TRUE)
}
\arguments{
\item{use.cache}{[\code{logical(1)}]\cr
Enable the cache?
Default is \code{TRUE}.}
}
\value{
[\code{function()}] with argument \code{slot}
 (name of the slot to cache the object in, default is \dQuote{default}).
 All other arguments are passed down to \code{\link{load2}}.
}
\description{
This closure returns a wrapper around \code{\link{load2}} which per
default caches loaded objects and returns the cached version
in subsequent calls.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isExpensiveExampleOk.R
\name{isExpensiveExampleOk}
\alias{isExpensiveExampleOk}
\title{Conditional checking for expensive examples}
\usage{
isExpensiveExampleOk()
}
\value{
[\code{logical(1)}].
}
\description{
Queries environment variable \dQuote{R_EXPENSIVE_EXAMPLE_OK}.
Returns \code{TRUE} iff set exactly to \dQuote{TRUE}.
This allows conditional checking of expensive examples in packages
via R CMD CHECK, so they are not run on CRAN, but at least
on your local computer.
A better option than \dQuote{dont_run} in many cases, where such examples
are not checked at all.
}
\examples{
# extremely costly random number generation, that we dont want checked on CRAN
if (isExpensiveExampleOk()) {
  runif(1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setAttribute.R
\name{setAttribute}
\alias{setAttribute}
\title{A wrapper for \code{attr(x, which) = y}}
\usage{
setAttribute(x, which, value)
}
\arguments{
\item{x}{[any]\cr
Your object.}

\item{which}{[\code{character(1)}]\cr
Name of the attribute to set}

\item{value}{[\code{ANY}]\cr
Value for the attribute.}
}
\value{
Changed object \code{x}.
}
\description{
Sets an attribute on an object and returns the modified object.
}
\examples{
setAttribute(list(), "foo", 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizeSubInts.R
\name{optimizeSubInts}
\alias{optimizeSubInts}
\title{Naive multi-start version of \code{\link{optimize}} for global optimization}
\usage{
optimizeSubInts(
  f,
  interval,
  ...,
  lower = min(interval),
  upper = max(interval),
  maximum = FALSE,
  tol = .Machine$double.eps^0.25,
  nsub = 50L
)
}
\arguments{
\item{f}{See \code{\link{optimize}}.}

\item{interval}{See \code{\link{optimize}}.}

\item{...}{See \code{\link{optimize}}.}

\item{lower}{See \code{\link{optimize}}.}

\item{upper}{See \code{\link{optimize}}.}

\item{maximum}{See \code{\link{optimize}}.}

\item{tol}{See \code{\link{optimize}}.}

\item{nsub}{[\code{integer(1)}]\cr
Number of subintervals. A value of 1 implies normal \code{\link{optimize}} behavior.
Default is 50L.}
}
\value{
See \code{\link{optimize}}.
}
\description{
The univariate \code{\link{optimize}} can stop at arbitrarily bad points when
\code{f} is not unimodal. This functions mitigates this effect in a very naive way:
\code{interval} is subdivided into \code{nsub} equally sized subintervals,
\code{\link{optimize}} is run on all of them (and on the original big interval) and
the best obtained point is returned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rangeVal.R
\name{rangeVal}
\alias{rangeVal}
\title{Calculate range statistic}
\usage{
rangeVal(x, na.rm = FALSE)
}
\arguments{
\item{x}{[\code{numeric}]\cr
The vector.}

\item{na.rm}{[\code{logical(1)}]\cr
If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
If \code{TRUE}, NAs are disregarded.
Default is \code{FALSE}}
}
\value{
[\code{numeric(1)}].
}
\description{
A simple wrapper for \code{diff(range(x))}, so \code{max(x) - min(x)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensureVector.R
\name{ensureVector}
\alias{ensureVector}
\title{Blow up single scalars / objects to vectors /  list by replication}
\usage{
ensureVector(x, n = 1L, cl = NULL, names = NULL, ensure.list = FALSE)
}
\arguments{
\item{x}{[any]\cr
Input element.}

\item{n}{[\code{integer(1)}]\cr
Desired length.
Default is 1 (the most common case).}

\item{cl}{[\code{character}*]\cr
Only do the operation if \code{x} inherits from this one of these classes,
otherwise simply let \code{x} pass.
Default is \code{NULL} which means to always do the operation.}

\item{names}{[\code{character}*] \cr
Names for result.
Default is \code{NULL}, which means no names.}

\item{ensure.list}{[\code{logical(1)}]\cr
Should \code{x} be wrapped in a list in any case?
Default is \code{FALSE}, i.e., if \code{x} is a scalar value, a vector is
returned.}
}
\value{
Ether a vector or list of length \code{n} with replicated \code{x} or \code{x} unchanged..
}
\description{
Useful for standard argument conversion where a user can input a single
element, but this has to be replicated now n times for a resulting vector or list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isScalarValue.R
\name{isScalarValue}
\alias{isScalarValue}
\alias{isScalarLogical}
\alias{isScalarNumeric}
\alias{isScalarInteger}
\alias{isScalarComplex}
\alias{isScalarCharacter}
\alias{isScalarFactor}
\title{Is given argument an atomic vector or factor of length 1?}
\usage{
isScalarValue(x, na.ok = TRUE, null.ok = FALSE, type = "atomic")

isScalarLogical(x, na.ok = TRUE, null.ok = FALSE)

isScalarNumeric(x, na.ok = TRUE, null.ok = FALSE)

isScalarInteger(x, na.ok = TRUE, null.ok = FALSE)

isScalarComplex(x, na.ok = TRUE, null.ok = FALSE)

isScalarCharacter(x, na.ok = TRUE, null.ok = FALSE)

isScalarFactor(x, na.ok = TRUE, null.ok = FALSE)
}
\arguments{
\item{x}{[any]\cr
Argument.}

\item{na.ok}{[\code{logical(1)}]\cr
Is \code{NA} considered a scalar?
Default is \code{TRUE}.}

\item{null.ok}{[\code{logical(1)}]\cr
Is \code{NULL} considered a scalar?
Default is \code{FALSE}.}

\item{type}{[\code{character(1)}]\cr
Allows to restrict to specific type, e.g., \dQuote{numeric}?
But instead of this argument you might want to consider using \code{isScalar<Type>}.
Default is \dQuote{atomic}, so no special restriction.}
}
\value{
[\code{logical(1)}].
}
\description{
More specific functions for scalars of a given type exist, too.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getUnixTime.R
\name{getUnixTime}
\alias{getUnixTime}
\title{Current time in seconds}
\usage{
getUnixTime()
}
\value{
[\code{integer(1)}].
}
\description{
Simple wrapper for \code{as.integer(Sys.time())}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse.R
\name{collapse}
\alias{collapse}
\title{Collapse vector to string}
\usage{
collapse(x, sep = ",")
}
\arguments{
\item{x}{[\code{vector}]\cr
Vector to collapse.}

\item{sep}{[\code{character(1)}]\cr
Passed to \code{collapse} in \code{\link{paste}}.
Default is \dQuote{,}.}
}
\value{
[\code{character(1)}].
}
\description{
A simple wrapper for \code{paste(x, collapse)}.
}
\examples{
collapse(c("foo", "bar"))
collapse(c("foo", "bar"), sep = ";")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isSubset.R
\name{isSubset}
\alias{isSubset}
\title{Check subset relation on two vectors}
\usage{
isSubset(x, y, strict = FALSE)
}
\arguments{
\item{x}{[\code{vector}]\cr
Source vector.}

\item{y}{[\code{vector}]\cr
Vector of the same mode as \code{x}.}

\item{strict}{[\code{logical(1)}]\cr
Checks for strict/proper subset relation.}
}
\value{
[\code{logical(1)}]
  \code{TRUE} if each element of \code{x} is also contained in \code{y}, i. e.,
  if \code{x} is a subset of \code{y} and \code{FALSE} otherwise.
}
\description{
Checks if all elements of x are contained in y.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catf.R
\name{catf}
\alias{catf}
\title{Wrapper for cat and sprintf}
\usage{
catf(..., file = "", append = FALSE, newline = TRUE)
}
\arguments{
\item{...}{[any]\cr
See \code{\link{sprintf}}.}

\item{file}{[\code{character(1)}]\cr
See \code{\link{cat}}.
Default is \dQuote{}.}

\item{append}{[\code{logical(1)}]\cr
See \code{\link{cat}}.
Default is \code{FALSE}.}

\item{newline}{[\code{logical(1)}]\cr
Append newline at the end?
Default is \code{TRUE}.}
}
\value{
Nothing.
}
\description{
A simple wrapper for \code{cat(sprintf(...))}.
}
\examples{
msg = "a message."
catf("This is \%s", msg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_error.R
\name{is.error}
\alias{is.error}
\title{Is return value of try an exception?}
\usage{
is.error(x)
}
\arguments{
\item{x}{[any]\cr
  Any object, usually the return value of \code{\link[base]{try}},
\code{\link[base]{tryCatch}}, or a function which may return a
\code{\link[base]{simpleError}}.}
}
\value{
[\code{logical(1)}].
}
\description{
Checks if an object is of class \dQuote{try-error} or
\dQuote{error}.
}
\examples{
x = try(stop("foo"))
print(is.error(x))
x = 1
print(is.error(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printToChar.R
\name{printToChar}
\alias{printToChar}
\title{Prints object to a string / character vector}
\usage{
printToChar(x, collapse = "\\n")
}
\arguments{
\item{x}{[any]\cr
Object to print}

\item{collapse}{[\code{character(1)}]\cr
Used to collapse multiple lines.
\code{NULL} means no collapsing, vector is returned.
Default is \dQuote{\\n}.}
}
\value{
[\code{character}].
}
\description{
Captures the output of print() as a character string.
}
\examples{
x = data.frame(a = 1:2, b = 3:4)
str(printToChar(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binPack.R
\name{binPack}
\alias{binPack}
\title{Simple bin packing}
\usage{
binPack(x, capacity)
}
\arguments{
\item{x}{[\code{numeric}]\cr
Numeric vector of elements to group.}

\item{capacity}{[\code{numeric(1)}]\cr
Maximum capacity of each bin, i.e., elements will be grouped
so their sum does not exceed this limit.}
}
\value{
[\code{integer}]. Integer with values \dQuote{1} to \dQuote{n.bins}
  indicating bin membership.
}
\description{
Maps numeric items in \code{x} into groups with sum
less or equal than \code{capacity}.
A very simple greedy algorithm is used, which is not really optimized
for speed. This is a convenience function for smaller vectors, not
a competetive solver for the real binbacking problem.
If an element of \code{x} exceeds \code{capacity}, an error
is thrown.
}
\examples{
x = 1:10
bp = binPack(x, 11)
xs = split(x, bp)
print(xs)
print(sapply(xs, sum))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkListElementClass.R
\name{checkListElementClass}
\alias{checkListElementClass}
\title{Check that a list contains only elements of a required type}
\usage{
checkListElementClass(xs, cl)
}
\arguments{
\item{xs}{[\code{list}]\cr
Argument.}

\item{cl}{[\code{character(1)}]\cr
Class that elements must have. Checked with \code{is}.}
}
\value{
Nothing.
}
\description{
Check that argument is a list and contains only elements of a required type.
Throws exception if check is not passed.
Note that argument is evaluated when checked.
}
\examples{
xs = as.list(1:3)
checkListElementClass(xs, "numeric")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/requirePackages.R
\name{requirePackages}
\alias{requirePackages}
\title{Require some packages}
\usage{
requirePackages(
  packs,
  min.versions = NULL,
  why = "",
  stop = TRUE,
  suppress.warnings = FALSE,
  default.method = "attach"
)
}
\arguments{
\item{packs}{[\code{character}]\cr
Names of packages.
If a package name is prefixed with \dQuote{!}, it will be attached using \code{\link[base]{require}}.
If a package name is prefixed with \dQuote{_}, its namespace will be loaded using \code{\link[base]{requireNamespace}}.
If there is no prefix, argument \code{default.method} determines how to deal with package loading.}

\item{min.versions}{[\code{character}]\cr
A char vector specifying required minimal version numbers for a subset of packages in \code{packs}.
Must be named and all names must be in \code{packs}.
The only exception is when \code{packs} is only a single string, then you are allowed to pass
an unnamed version string here.
Default is \code{NULL}, meaning no special version requirements}

\item{why}{[\code{character(1)}]\cr
Short string explaining why packages are required.
Default is an empty string.}

\item{stop}{[\code{logical(1)}]\cr
Should an exception be thrown for missing packages?
Default is \code{TRUE}.}

\item{suppress.warnings}{[\code{logical(1)}]\cr
Should warnings be supressed while requiring?
Default is \code{FALSE}.}

\item{default.method}{[\code{character(1)}]\cr
If the packages are not explicitly prefixed with \dQuote{!} or \dQuote{_},
this arguments determines the default. Possible values are \dQuote{attach} and
\dQuote{load}.
Note that the default is \dQuote{attach}, but this might/will change in a future version, so
please make sure to always explicitly set this.}
}
\value{
[\code{logical}]. Named logical vector describing which packages could be loaded (with required version).
  Same length as \code{packs}.
}
\description{
Packages are loaded either via \code{\link{requireNamespace}} or \code{\link{require}}.

If some packages could not be loaded and \code{stop} is \code{TRUE}
the following exception is thrown:
\dQuote{For <why> please install the following packages: <missing packages>}.
If \code{why} is \code{NULL} the message is:
\dQuote{Please install the following packages: <missing packages>}.
}
\examples{
requirePackages(c("BBmisc", "base"), why = "BBmisc example")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRelativePath.R
\name{getRelativePath}
\alias{getRelativePath}
\title{Construct a path relative to another}
\usage{
getRelativePath(to, from = getwd(), ignore.case = isWindows())
}
\arguments{
\item{to}{[\code{character(1)}]\cr
Where the relative path should point to.}

\item{from}{[\code{character(1)}]\cr
From which part to start.
Default is \code{\link[base]{getwd}}.}

\item{ignore.case}{[\code{logical(1)}]\cr
Should path comparisons be made case insensitve?
Default is \code{TRUE} on Windows systems and \code{FALSE} on other systems.}
}
\value{
[character(1)]: A relative path.
}
\description{
Constructs a relative path from path \code{from} to path \code{to}.
If this is not possible (i.e. different drive letters on windows systems),
\code{NA} is returned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortByCol.R
\name{sortByCol}
\alias{sortByCol}
\title{Sort the rows of a data.frame according to one or more columns}
\usage{
sortByCol(x, col, asc = TRUE)
}
\arguments{
\item{x}{[\code{data.frame}]\cr
Data.frame to sort.}

\item{col}{[\code{character}]\cr
One or more column names to sort \code{x} by.
In order of preference.}

\item{asc}{[\code{logical}]\cr
Sort ascending (or descending)?
One value per entry of \code{col}.
If a scalar logical is passed, it is replicated.
Default is \code{TRUE}.}
}
\value{
[\code{data.frame}].
}
\description{
Sorts a data frame by one or more columns in ascending or descending order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isValidName.R
\name{isValidName}
\alias{isValidName}
\title{Can some strings be used for column or list element names without problems?}
\usage{
isValidName(x, unique = TRUE)
}
\arguments{
\item{x}{[\code{character}]\cr
Character vector to check.}

\item{unique}{[\code{logical(1)}]\cr
Should the names be unique?
Default is \code{TRUE}.}
}
\value{
[\code{logical}]. One Boolean entry for each string in \code{x}.
  If the entries are not unique and \code{unique} is enabled, the first duplicate will
  be \code{FALSE}.
}
\description{
Checks if strings are valid R names that won't cause problems as column names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save2.R
\name{save2}
\alias{save2}
\title{Save multiple objects to a file}
\usage{
save2(
  file,
  ...,
  ascii = FALSE,
  version = NULL,
  compress = !ascii,
  compression_level,
  eval.promises = TRUE,
  precheck = TRUE
)
}
\arguments{
\item{file}{File to save.}

\item{...}{[\code{any}]\cr
Will be converted to an environment and then passed to \code{\link[base]{save}}.}

\item{ascii}{See help of \code{\link[base]{save}}.}

\item{version}{See help of \code{\link[base]{save}}.}

\item{compress}{See help of \code{\link[base]{save}}.}

\item{compression_level}{See help of \code{\link[base]{save}}.}

\item{eval.promises}{See help of \code{\link[base]{save}}.}

\item{precheck}{See help of \code{\link[base]{save}}.}
}
\value{
See help of \code{\link[base]{save}}.
}
\description{
A simple wrapper for \code{\link[base]{save}}. Understands key = value syntax to save
objects using arbitrary variable names. All options of \code{\link[base]{save}},
except \code{list} and \code{envir}, are available and passed to
\code{\link[base]{save}}.
}
\examples{
x = 1
save2(y = x, file = tempfile())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert.R
\name{insert}
\alias{insert}
\title{Insert elements from one list/vector into another list/vector}
\usage{
insert(xs1, xs2, elements)
}
\arguments{
\item{xs1}{[\code{list}]\cr
First list/vector.}

\item{xs2}{[\code{list}]\cr
Second vector/list. Must be fully and uniquely named.}

\item{elements}{[\code{character}]\cr
Elements from \code{xs2} to insert into \code{xs1}.
Default is all.}
}
\value{
\code{x1} with replaced elements from \code{x2}.
}
\description{
Inserts elements from \code{xs2} into \code{xs1} by name,
overwriting elements of equal names.
}
\examples{
xs1 = list(a = 1, b = 2)
xs2 = list(b = 1, c = 4)
insert(xs1, xs2)
insert(xs1, xs2, elements = "c")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertRowsToList.R
\name{convertRowsToList}
\alias{convertRowsToList}
\alias{convertColsToList}
\title{Convert rows (columns) of data.frame or matrix to lists}
\usage{
convertRowsToList(
  x,
  name.list = TRUE,
  name.vector = FALSE,
  factors.as.char = TRUE,
  as.vector = TRUE
)

convertColsToList(
  x,
  name.list = FALSE,
  name.vector = FALSE,
  factors.as.char = TRUE,
  as.vector = TRUE
)
}
\arguments{
\item{x}{[\code{matrix} | \code{data.frame}]\cr
Object to convert.}

\item{name.list}{[\code{logical(1)}]\cr
Name resulting list with names of rows (cols) of \code{x}?
Default is \code{FALSE}.}

\item{name.vector}{[\code{logical(1)}]\cr
Name vector elements in resulting list with names of cols (rows) of \code{x}?
Default is \code{FALSE}.}

\item{factors.as.char}{[\code{logical(1)}]\cr
If \code{x} is a data.frame, convert factor columns to
string elements in the resulting lists?
Default is \code{TRUE}.}

\item{as.vector}{[\code{logical(1)}]\cr
If \code{x} is a matrix, store rows as vectors in the resulting list - or otherwise as lists?
Default is \code{TRUE}.}
}
\value{
[\code{list} of lists or vectors].
}
\description{
For each row, one list/vector is constructed, each entry of
the row becomes a list/vector element.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getUsedFactorLevels.R
\name{getUsedFactorLevels}
\alias{getUsedFactorLevels}
\title{Determines used factor levels}
\usage{
getUsedFactorLevels(x)
}
\arguments{
\item{x}{[\code{factor}]\cr
The factor.}
}
\value{
[\code{character}]
}
\description{
Determines the factor levels of a factor type vector
that are actually occuring in it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/do.call2.R
\name{do.call2}
\alias{do.call2}
\title{Execute a function call similar to \code{do.call}}
\usage{
do.call2(fun, ..., .args = list())
}
\arguments{
\item{fun}{[\code{character(1)}]\cr
Name of the function to call.}

\item{...}{[any]\cr
Arguments to \code{fun}. Best practice is to specify them in a \code{key = value} syntax.}

\item{.args}{[\code{list}]\cr
Arguments to \code{fun} as a (named) list. Will be passed after arguments in \code{...}.
Default is \code{list()}.}
}
\value{
Return value of \code{fun}.
}
\description{
This function is supposed to be a replacement for \code{\link[base]{do.call}} in situations
where you need to pass big R objects.
Unlike \code{\link[base]{do.call}}, this function allows to pass objects via \code{...}
to avoid a copy.
}
\examples{
\dontrun{
  library(microbenchmark)
  x = 1:1e7
  microbenchmark(do.call(head, list(x, n = 1)), do.call2("head", x, n = 1))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapsef.R
\name{collapsef}
\alias{collapsef}
\title{Collapse vector to string}
\usage{
collapsef(..., sep = ",")
}
\arguments{
\item{...}{[any]\cr
See \code{\link{sprintf}}.}

\item{sep}{[\code{character(1)}]\cr
See \code{\link{collapse}}.}
}
\value{
[\code{character(1)}].
}
\description{
A simple wrapper for \code{collapse(sprintf, ...)}.

Useful for vectorized call to \code{\link{sprintf}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib.R
\name{lib}
\alias{lib}
\title{A wrapper for \code{library}}
\usage{
lib(...)
}
\arguments{
\item{...}{[any]\cr
Package names.}
}
\value{
[\code{logical}]: Named logical vector determining the success
  of package load.
}
\description{
Tries to load packages. If the packages are not found, they will be installed from
the default repository. This function is intended for use in interactive sessions
and should not be used by other packages.
}
\examples{
\dontrun{
lib("BBmisc", "MASS", "rpart")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMaxColIndex.R
\name{getMaxIndexOfRows}
\alias{getMaxIndexOfRows}
\alias{getMinIndexOfRows}
\alias{getMaxIndexOfCols}
\alias{getMinIndexOfCols}
\title{Find row- or columnwise the index of the maximal / minimal element in a matrix}
\usage{
getMaxIndexOfRows(x, weights = NULL, ties.method = "random", na.rm = FALSE)

getMinIndexOfRows(x, weights = NULL, ties.method = "random", na.rm = FALSE)

getMaxIndexOfCols(x, weights = NULL, ties.method = "random", na.rm = FALSE)

getMinIndexOfCols(x, weights = NULL, ties.method = "random", na.rm = FALSE)
}
\arguments{
\item{x}{[\code{matrix(n,m)}] \cr
Numerical input matrix.}

\item{weights}{[\code{numeric}]\cr
Weights (same length as number of rows/cols).
If these are specified, the index is selected from the weighted elements
(see \code{\link{getMaxIndex}}).
Default is \code{NULL} which means no weights.}

\item{ties.method}{[\code{character(1)}]\cr
How should ties be handled?
Possible are: \dQuote{random}, \dQuote{first}, \dQuote{last}.
Default is \dQuote{random}.}

\item{na.rm}{[\code{logical(1)}]\cr
If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
If \code{TRUE}, NAs are disregarded.
Default is \code{FALSE}}
}
\value{
[\code{integer(n)}].
}
\description{
\code{getMaxIndexOfRows} returns the index of the maximal element of each row.
\code{getMinIndexOfRows} returns the index of the minimal element of each row.
\code{getMaxIndexOfCols} returns the index of the maximal element of each col.
\code{getMinIndexOfCols} returns the index of the minimal element of each col.
If a corresponding vector (row or col) is empty, possibly after NA removal, -1 is returned
as index.
}
\examples{
x = matrix(runif(5 * 3), ncol = 3)
print(x)
print(getMaxIndexOfRows(x))
print(getMinIndexOfRows(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeS3Obj.R
\name{makeS3Obj}
\alias{makeS3Obj}
\title{Simple constructor for S3 objects based on lists}
\usage{
makeS3Obj(classes, ...)
}
\arguments{
\item{classes}{[\code{character}]\cr
Class(es) for constructed object.}

\item{...}{[any]\cr
Key-value pairs for class members.}
}
\value{
Object.
}
\description{
Simple wrapper for \code{as.list} and \code{\link{setClasses}}.
}
\examples{
makeS3Obj("car", speed = 100, color = "red")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printStrToChar.R
\name{printStrToChar}
\alias{printStrToChar}
\title{Print \code{str(x)} of an object to a string / character vector}
\usage{
printStrToChar(x, collapse = "\\n")
}
\arguments{
\item{x}{[any]\cr
Object to print}

\item{collapse}{[\code{character(1)}]\cr
Used to collapse multiple lines.
\code{NULL} means no collapsing, vector is returned.
Default is \dQuote{\\n}.}
}
\value{
[\code{character}].
}
\description{
Captures the output of str() as a character string.
}
\examples{
printStrToChar(iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeMode.R
\name{computeMode}
\alias{computeMode}
\title{Compute statistical mode of a vector (value that occurs most frequently)}
\usage{
computeMode(x, ties.method = "random", na.rm = TRUE)
}
\arguments{
\item{x}{[\code{vector}]\cr
Factor, character, integer, numeric or logical vector.}

\item{ties.method}{[\code{character(1)}]\cr
\dQuote{first}, \dQuote{random}, \dQuote{last}: Decide which value to take in case of ties.
Default is \dQuote{random}.}

\item{na.rm}{[\code{logical(1)}]\cr
If \code{TRUE}, missing values in the data removed.
if \code{FALSE}, they are used as a separate level and this level could therefore
be returned as the most frequent one.
Default is \code{TRUE}.}
}
\value{
Modal value of length 1, data type depends on data type of \code{x}.
}
\description{
Works for integer, numeric, factor and character vectors.
The implementation is currently not extremely efficient.
}
\examples{
computeMode(c(1,2,3,3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor.R
\name{cFactor}
\alias{cFactor}
\title{Combine multiple factors and return a factor}
\usage{
cFactor(...)
}
\arguments{
\item{...}{[\code{factor}]\cr
The factors.}
}
\value{
[\code{factor}].
}
\description{
Note that function does not inherit from \code{\link{c}} to not change R semantics behind your back when this
package is loaded.
}
\examples{
f1 = factor(c("a", "b"))
f2 = factor(c("b", "c"))
print(c(f1, f2))
print(cFactor(f1, f2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vapply.R
\name{vlapply}
\alias{vlapply}
\alias{viapply}
\alias{vnapply}
\alias{vcapply}
\title{Apply a function with a predefined return value}
\usage{
vlapply(x, fun, ..., use.names = TRUE)

viapply(x, fun, ..., use.names = TRUE)

vnapply(x, fun, ..., use.names = TRUE)

vcapply(x, fun, ..., use.names = TRUE)
}
\arguments{
\item{x}{[\code{vector} or \code{list}]\cr
Object to apply function on.}

\item{fun}{[\code{function}]\cr
Function to apply on each element of \code{x}.}

\item{...}{[\code{ANY}]\cr
Additional arguments for \code{fun}.}

\item{use.names}{[\code{logical(1)}]\cr
Should result be named?
Default is \code{TRUE}.}
}
\description{
These are just wrappers around \code{\link[base]{vapply}} with
argument \code{FUN.VALUE} set.
The function is expected to return a single \code{logical}, \code{integer},
\code{numeric} or \code{character} value, depending on the second letter
of the function name.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz_deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{convertDfCols}
\alias{listToShortString}
\title{Deprecated function. Do not use!}
\usage{
convertDfCols(
  df,
  chars.as.factor = FALSE,
  factors.as.char = FALSE,
  ints.as.num = FALSE,
  logicals.as.factor = FALSE
)

listToShortString(x, num.format = "\%.4g", clip.len = 15L)
}
\arguments{
\item{df}{No text}

\item{chars.as.factor}{No text}

\item{factors.as.char}{No text}

\item{ints.as.num}{No text}

\item{logicals.as.factor}{No text}

\item{x}{No text}

\item{num.format}{No text}

\item{clip.len}{No text}
}
\description{
These functions are deprecated and will be removed in future versions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertInteger.R
\name{convertIntegers}
\alias{convertIntegers}
\title{Conversion for integer vector}
\usage{
convertIntegers(x)
}
\arguments{
\item{x}{[any]\cr
Argument.}
}
\value{
Either an integer vector if conversion was done or \code{x} unchanged.
}
\description{
Convert numeric vector to integer vector if the numeric vector fully represents
an integer vector,
e.g. \code{c(1, 5)} to \code{c(1L, 5L)}.
Otherwise the argument is returned unchanged.
}
\examples{
str(convertIntegers(1.0))
str(convertIntegers(1.3))
str(convertIntegers(c(1.0, 2.0)))
str(convertIntegers("foo"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/names2.R
\name{names2}
\alias{names2}
\title{Replacement for names which always returns a vector}
\usage{
names2(x, missing.val = NA_character_)
}
\arguments{
\item{x}{[\code{ANY}]\cr
Object, probably named.}

\item{missing.val}{[\code{ANY}]\cr
Value to set for missing names. Default is \code{NA_character_}.}
}
\value{
[\code{character}]: vector of the same length as \code{x}.
}
\description{
A simple wrapper for \code{\link[base]{names}}.
Returns a vector even if no names attribute is set.
Values \code{NA} and \code{""} are treated as missing and
replaced with the value provided in \code{missing.val}.
}
\examples{
x = 1:3
names(x)
names2(x)
names(x[1:2]) = letters[1:2]
names(x)
names2(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splitTime.R
\name{splitTime}
\alias{splitTime}
\title{Split seconds into handy chunks of time}
\usage{
splitTime(seconds, unit = "years")
}
\arguments{
\item{seconds}{[\code{numeric(1)}]\cr
Number of seconds. If not an integer, it is rounded down.}

\item{unit}{[\code{character(1)}]\cr
Largest unit to split seconds into.
Must be one of: \code{c("years", "days", "hours", "minutes", "seconds")}.
Default is \dQuote{years}.}
}
\value{
[\code{numeric(5)}]. A named vector containing the
\dQuote{years}, \dQuote{days}, \dQuote{hours}, \dQuote{minutes}
and \dQuote{seconds}. Units larger than the given \code{unit} are
\code{NA}.
}
\description{
Note that a year is simply defined as exactly 365 days.
}
\examples{
splitTime(1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clipString.R
\name{clipString}
\alias{clipString}
\title{Shortens strings to a given length}
\usage{
clipString(x, len, tail = "...")
}
\arguments{
\item{x}{[\code{character}]\cr
Vector of strings.}

\item{len}{[\code{integer(1)}]\cr
Absolute length the string should be clipped to, including \code{tail}.
Note that you cannot clip to a shorter length than \code{tail}.}

\item{tail}{[\code{character(1)}]\cr
If the string has to be shortened at least 1 character, the final characters will be \code{tail}.
Default is \dQuote{...}.}
}
\value{
[\code{character(1)}].
}
\description{
Clips strings to a maximum length, appending a tail if shortened.
}
\examples{
print(clipString("abcdef", 10))
print(clipString("abcdef", 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chunk.R
\name{chunk}
\alias{chunk}
\title{Chunk elements of vectors into blocks of nearly equal size}
\usage{
chunk(x, chunk.size, n.chunks, props, shuffle = FALSE)
}
\arguments{
\item{x}{[ANY]\cr
Vector, list or other type supported by \code{\link[base]{split}}.}

\item{chunk.size}{[\code{integer(1)}]\cr
Requested number of elements in each chunk.
Cannot be used in combination with \code{n.chunks} or \code{props}.
If \code{x} cannot be evenly chunked, some chunks will have less elements.}

\item{n.chunks}{[\code{integer(1)}]\cr
Requested number of chunks.
If more chunks than elements in \code{x} are requested, empty chunks are
dropped.
Can not be used in combination with \code{chunks.size} or \code{props}.}

\item{props}{[\code{numeric}]\cr
Vector of proportions for chunk sizes.
Empty chunks may occur, depending on the length of \code{x} and the given
proportions.
Cannot be used in combination with \code{chunks.size} or \code{n.chunks}.}

\item{shuffle}{[\code{logical(1)}]\cr
Shuffle \code{x}?
Default is \code{FALSE}.}
}
\value{
[unnamed \code{list}] of chunks.
}
\description{
In case of shuffling and vectors that cannot be chunked evenly,
it is chosen randomly which levels / chunks will receive 1 element less.
If you do not shuffle, always the last chunks will receive 1 element less.
}
\examples{
xs = 1:10
chunk(xs, chunk.size = 3)
chunk(xs, n.chunks = 2)
chunk(xs, n.chunks = 2, shuffle = TRUE)
chunk(xs, props = c(7, 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printHead.R
\name{printHead}
\alias{printHead}
\title{More meaningful \code{head(df)} output}
\usage{
printHead(x, n = 6L)
}
\arguments{
\item{x}{[\code{data.frame} | \code{matrix} | \code{list} | \code{vector}]\cr
Object.}

\item{n}{[\code{integer(1)}]\cr
Single positive integer: number of rows for a matrix/data.frame or number of
elements for vectors/lists respectively.}
}
\value{
Nothing.
}
\description{
The behaviour is similar to \code{print(head(x, n))}. The difference is, that if
the number of rows in a data.frame/matrix or the number of elements in a list
or vector is larger than \code{n}, additional information is printed about
the total number of rows or elements respectively.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splitPath.R
\name{splitPath}
\alias{splitPath}
\title{Split a path into components}
\usage{
splitPath(path)
}
\arguments{
\item{path}{[\code{character(1)}]\cr
Path to split as string}
}
\value{
\code{named list}: List with components \dQuote{drive} (\code{character(1)}
 and \dQuote{path} (\code{character(n)}.
}
\description{
The first normalized path is split on forward and backward slashes and its components returned as
character vector. The drive or network home are extracted separately on windows systems and
empty on all other systems.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toRangeStr.R
\name{toRangeStr}
\alias{toRangeStr}
\title{Convert a numerical vector into a range string}
\usage{
toRangeStr(x, range.sep = " - ", block.sep = ", ")
}
\arguments{
\item{x}{[\code{integer}]\cr
Vector to convert into a range string.}

\item{range.sep}{[\code{character(1)}]\cr
Separator between the first and last element of a range of consecutive
elements in \code{x}.
Default is \dQuote{ - }.}

\item{block.sep}{[\code{character(1)}]\cr
Separator between non consecutive elements of \code{x} or ranges.
Default is \dQuote{, }.}
}
\value{
[\code{character(1)}]
}
\description{
Converts a vector of integers into a compact range string representation.
}
\examples{
x = sample(1:10, 7)
toRangeStr(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMaxIndex.R
\name{getMaxIndex}
\alias{getMaxIndex}
\alias{getMinIndex}
\alias{getBestIndex}
\title{Return index of maximal/minimal/best element in numerical vector}
\usage{
getMaxIndex(x, weights = NULL, ties.method = "random", na.rm = FALSE)

getMinIndex(x, weights = NULL, ties.method = "random", na.rm = FALSE)

getBestIndex(x, weights = NULL, minimize = TRUE, ...)
}
\arguments{
\item{x}{[\code{numeric}]\cr
Input vector.}

\item{weights}{[\code{numeric}]\cr
Weights (same length as \code{x}).
If these are specified, the index is selected from \code{x * w}.
Default is \code{NULL} which means no weights.}

\item{ties.method}{[\code{character(1)}]\cr
How should ties be handled?
Possible are: \dQuote{random}, \dQuote{first}, \dQuote{last}.
Default is \dQuote{random}.}

\item{na.rm}{[\code{logical(1)}]\cr
If \code{FALSE}, NA is returned if an NA is encountered in \code{x}.
If \code{TRUE}, NAs are disregarded.
Default is \code{FALSE}}

\item{minimize}{[\code{logical(1)}]\cr
Minimal element is considered best?
Default is \code{TRUE}.}

\item{...}{[any]\cr
Further arguments passed down to the delegate.}
}
\value{
[\code{integer(1)}].
}
\description{
If \code{x} is empty or only contains NAs which are to be removed,
-1 is returned.
}
\note{
Function \code{getBestIndex} is a simple wrapper for \code{getMinIndex} or
\code{getMaxIndex} respectively depending on the argument \code{minimize}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load2.R
\name{load2}
\alias{load2}
\title{Load RData file and return objects in it}
\usage{
load2(file, parts, simplify = TRUE, envir, impute)
}
\arguments{
\item{file}{[\code{character(1)}]\cr
File to load.}

\item{parts}{[\code{character}]\cr
Elements in file to load.
Default is all.}

\item{simplify}{[\code{logical(1)}]\cr
If \code{TRUE}, a list is only returned if \code{parts} and the file contain both more
than 1 element, otherwise the element is directly returned.
Default is \code{TRUE}.}

\item{envir}{[\code{environment(1)}]\cr
Assign objects to this environment.
Default is not to assign.}

\item{impute}{[\code{ANY}]\cr
If \code{file} does not exists, return \code{impute} instead.
Default is missing which will result in an exception if \code{file} is not found.}
}
\value{
Either a single object or a list.
}
\description{
Loads an RData file and returns the objects directly instead of loading them into an environment.
}
\examples{
fn = tempfile()
save2(file = fn, a = 1, b = 2, c = 3)
load2(fn, parts = "a")
load2(fn, parts = c("a", "c"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsort.R
\name{lsort}
\alias{lsort}
\title{A wrapper for \code{\link{sort}} to sort using the \dQuote{C} collating rules}
\usage{
lsort(...)
}
\arguments{
\item{...}{Options passed to sort.}
}
\value{
See \code{\link{sort}}.
}
\description{
Sorts using the C locale for consistent sorting across platforms.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertInteger.R
\name{convertInteger}
\alias{convertInteger}
\title{Conversion for single integer}
\usage{
convertInteger(x)
}
\arguments{
\item{x}{[any]\cr
Argument.}
}
\value{
Either a single integer if conversion was done or \code{x} unchanged.
}
\description{
Convert single numeric to integer only if the numeric represents a single integer,
e.g. 1 to 1L.
Otherwise the argument is returned unchanged.
}
\examples{
str(convertInteger(1.0))
str(convertInteger(1.3))
str(convertInteger(c(1.0, 2.0)))
str(convertInteger("foo"))
}
